@@ -155,15 +155,12 @@
         (sb-posix:syscall-errno c))))
   #.sb-posix::enoent)
 
-(deftest rmdir.error.2
-  (handler-case
-      (sb-posix:rmdir *this-file*)
-    (sb-posix:syscall-error (c)
-      (sb-posix:syscall-errno c)))
-  #-win32
-  #.sb-posix::enotdir
-  #+win32
-  #.sb-posix::einval)
+(test-util:with-test (:name :rmdir.error.2)
+  (let ((err (handler-case (sb-posix:rmdir *this-file*)
+               (sb-posix:syscall-error (c) (sb-posix:syscall-errno c)))))
+    (assert #-unix (eql err #.sb-posix:einval)
+            ;; non-writable parent dir may return EACCES instead of ENOTDIR
+            #+unix (member err `(,sb-posix:enotdir ,sb-posix:eacces)))))
 
 (deftest rmdir.error.3
   (handler-case
@@ -546,9 +543,10 @@
     (assert (equal (sort posix-readdir #'string<)
                    (sort cl-directory #'string<)))))
 
-(deftest write.1
-    (progn
-      (let ((fd (sb-posix:open (merge-pathnames "write-test.txt" *test-directory*)
+(test-util:with-test (:name :write.1)
+  (multiple-value-bind (n str)
+     (test-util:with-scratch-file (tmpname)
+      (let ((fd (sb-posix:open tmpname
                                (logior sb-posix:o-creat sb-posix:o-wronly)
                                (logior sb-posix:s-irusr sb-posix:s-iwusr)))
             (retval nil))
@@ -557,10 +555,8 @@
                (setf retval (sb-posix:write fd (sb-sys:vector-sap buf) 3)))
           (sb-posix:close fd))
 
-        (with-open-file (inf (merge-pathnames "write-test.txt" *test-directory*)
-                             :direction :input)
-          (values retval (read-line inf)))))
-  3 "foo")
+        (with-open-file (inf tmpname) (values retval (read-line inf)))))
+    (assert (and (eql n 3) (equal str "foo")))))
 
 #-(or android win32)
 (deftest pwent.1
@@ -691,23 +687,25 @@
   ;; The error tests are in the order of exposition from SUSv3.
   #-freebsd
   (deftest readlink.error.1
-      (let* ((subdir-pathname (merge-pathnames
-                               (make-pathname
-                                :directory '(:relative "readlink.error.1"))
-                               *test-directory*))
-             (link-pathname (make-pathname :name "readlink.error.1"
-                                           :defaults subdir-pathname)))
-        (sb-posix:mkdir subdir-pathname #o777)
-        (sb-posix:symlink "/" link-pathname)
-        (sb-posix:chmod subdir-pathname 0)
-        (unwind-protect
-             (handler-case (sb-posix:readlink link-pathname)
-               (sb-posix:syscall-error (c)
-                 (sb-posix:syscall-errno c)))
-          (ignore-errors
-           (sb-posix:chmod subdir-pathname #o777)
-           (sb-posix:unlink link-pathname)
-           (sb-posix:rmdir subdir-pathname))))
+    (if (zerop (sb-posix:getuid))
+        sb-posix:eacces
+        (let* ((subdir-pathname (merge-pathnames
+                                 (make-pathname
+                                  :directory '(:relative "readlink.error.1"))
+                                 *test-directory*))
+               (link-pathname (make-pathname :name "readlink.error.1"
+                                             :defaults subdir-pathname)))
+          (sb-posix:mkdir subdir-pathname #o777)
+          (sb-posix:symlink "/" link-pathname)
+          (sb-posix:chmod subdir-pathname 0)
+          (unwind-protect
+               (handler-case (sb-posix:readlink link-pathname)
+                 (sb-posix:syscall-error (c)
+                   (sb-posix:syscall-errno c)))
+            (ignore-errors
+             (sb-posix:chmod subdir-pathname #o777)
+             (sb-posix:unlink link-pathname)
+             (sb-posix:rmdir subdir-pathname)))))
     #.sb-posix:eacces)
   (deftest readlink.error.2
       (let* ((non-link-pathname (make-pathname :name "readlink.error.2"
