@@ -18,7 +18,6 @@
   (let ((args (copy-list args)))
     (remf args :weakness)
     (remf args :synchronized)
-    (remf args :finalizer)
     (let ((hash-fun (getf args :hash-function)))
       (when hash-fun
         (assert (eq (getf args :test) 'eq))
@@ -201,6 +200,7 @@
   name)
 
 (declaim (declaration enable-package-locks disable-package-locks))
+(declaim (declaration sb-c::tlab))
 
 ;;; The XC-STRICT-CL and SB-XC packages are called COMMON-LISP in the
 ;;; target image.
@@ -233,6 +233,9 @@
         *cl-package*
         p)))
 
+(defun possibly-base-stringize (s) (coerce (the string s) 'simple-base-string))
+(defun possibly-base-stringize-to-heap (s) (coerce (the string s) 'simple-base-string))
+
 ;;; printing structures
 
 (defun default-structure-print (structure stream depth)
@@ -253,18 +256,14 @@
 ;;   - DEBUG-SOURCE, COMPILED-DEBUG-INFO, COMPILED-DEBUG-FUN-{something}
 ;;   - HEAP-ALIEN-INFO and ALIEN-{something}-TYPE
 ;;   - COMMA
-#-metaspace (defmacro wrapper-friend (x) x)
-(defun %instance-wrapper (instance)
-  (declare (notinline classoid-wrapper))
-  (classoid-wrapper (find-classoid (type-of instance))))
+(defun %instance-layout (instance)
+  (declare (notinline classoid-layout))
+  (classoid-layout (find-classoid (type-of instance))))
 (defun %instance-length (instance)
-  (declare (notinline wrapper-length))
+  (declare (notinline layout-length))
   ;; In the target, it is theoretically possible to have %INSTANCE-LENGTH
   ;; exceeed layout length, but in the cross-compiler they're the same.
-  (wrapper-length (%instance-wrapper instance)))
-(defun layout-id (x)
-  (declare (notinline sb-kernel::wrapper-id))
-  (sb-kernel::wrapper-id x))
+  (layout-length (%instance-layout instance)))
 
 (defun %find-position (item seq from-end start end key test)
   (let ((position (position item seq :from-end from-end
@@ -330,6 +329,9 @@
 ;;;; in lieu of #+sb-xc-host elsewere which messes up toplevel form numbers.
 (in-package "SB-C")
 
+;;; For macro lambdas that are processed by the host
+(declaim (declaration top-level-form))
+
 ;;; The opposite of *undefined-fun-allowlist* - if certain full calls
 ;;; are seen, it is probably the result of a missed transform and/or
 ;;; misconfiguration.
@@ -382,3 +384,18 @@
   `(with-output-to-string (,var) ,@body))
 
 (defun source-location ())
+
+;;; %SYMBOL-INFO is a primitive object accessor defined in 'objdef.lisp'
+;;; But in the host Lisp, there is no such thing. Instead, SYMBOL-%INFO
+;;; is kept as a property on the host symbol.
+(declaim (inline symbol-%info))
+(defun symbol-%info (symbol) (get symbol :sb-xc-globaldb-info))
+(defun symbol-dbinfo (symbol) (symbol-%info symbol))
+
+(defun sys-copy-struct (x) (copy-structure x))
+(defun ensure-heap-list (x) (copy-list x))
+
+(defun range< (l x h) (< l x h))
+(defun range<= (l x h) (<= l x h))
+(defun range<<= (l x h) (and (< l x) (<= x h)))
+(defun range<=< (l x h) (and (<= l x) (< x h)))
