@@ -21,6 +21,7 @@
           (dstate (make-dstate nil))
           (sap (int-sap 0))
           (seg (sb-disassem::%make-segment :sap-maker (lambda () sap))))
+     (declare (ignorable mov-ea mov-imm-acc))
      (macrolet ((do-functions ((fun-var addr-var) &body body)
                   ;; Loop over all embedded functions
                   `(dotimes (fun-index (code-n-entries ,',code-var))
@@ -46,11 +47,9 @@
 
 (defun sb-vm::collect-immobile-code-relocs ()
   (let ((code-components
-         (make-array 20000 :element-type '(unsigned-byte 32)
-                           :fill-pointer 0 :adjustable t))
+         (make-array 20000 :element-type 'sb-vm:word :fill-pointer 0 :adjustable t))
         (relocs
-         (make-array 100000 :element-type '(unsigned-byte 32)
-                            :fill-pointer 0 :adjustable t))
+         (make-array 100000 :element-type 'sb-vm:word :fill-pointer 0 :adjustable t))
         (seg (sb-disassem::%make-segment
               :sap-maker (lambda () (error "Bad sap maker")) :virtual-location 0))
         (dstate (make-dstate nil)))
@@ -200,7 +199,7 @@
     (let ((fun-entry (sb-vm::fdefn-raw-addr fdefn))
           (fdefn-entry (sb-vm::fdefn-entry-address fdefn)))
       (flet ((code-statically-links-fdefn-p (code)
-               (multiple-value-bind (fdefns-start count) (sb-vm::code-header-fdefn-range code)
+               (multiple-value-bind (fdefns-start count) (code-header-fdefn-range code)
                  (dotimes (i count)
                    (let ((constant (code-header-ref code (+ fdefns-start i))))
                      (when (or (eq constant fdefn)
