@@ -86,7 +86,7 @@
          (funcall (built-in-classoid-predicate type) object)
          (and (or (%instancep object)
                   (functionp object))
-              (classoid-typep (wrapper-of object) type object))))
+              (classoid-typep (layout-of object) type object))))
     (union-type
      (some (lambda (union-type-type) (recurse object union-type-type))
            (union-type-types type)))
@@ -100,13 +100,11 @@
     #+sb-simd-pack
     (simd-pack-type
      (and (simd-pack-p object)
-          (let* ((tag (%simd-pack-tag object)))
-            (= (sbit (simd-pack-type-element-type type) tag) 1))))
+          (logbitp (%simd-pack-tag object) (simd-pack-type-tag-mask type))))
     #+sb-simd-pack-256
     (simd-pack-256-type
      (and (simd-pack-256-p object)
-          (let* ((tag (%simd-pack-256-tag object)))
-            (= (sbit (simd-pack-256-type-element-type type) tag) 1))))
+          (logbitp (%simd-pack-256-tag object) (simd-pack-256-type-tag-mask type))))
     (character-set-type
      (test-character-type type))
     (negation-type
@@ -152,8 +150,8 @@
                       (block nil
                         (classoid-typep
                          (typecase object
-                           (instance (%instance-wrapper object))
-                           (funcallable-instance (%fun-wrapper object))
+                           (instance (%instance-layout object))
+                           (funcallable-instance (%fun-layout object))
                            (t (return)))
                          (cdr (truly-the cons cache))
                          object)))
@@ -164,19 +162,6 @@
           (setf (car cache) fun)
           (funcall fun cache object)))))
 
-;;; Do a type test from a class cell, allowing forward reference and
-;;; redefinition.
-(defun classoid-cell-typep (cell object)
-  (let ((layout (typecase object
-                  (instance (%instance-layout object))
-                  (funcallable-instance (%fun-layout object))
-                  (t (return-from classoid-cell-typep))))
-        (classoid (classoid-cell-classoid cell)))
-    (unless classoid
-      (error "The class ~S has not yet been defined."
-             (classoid-cell-name cell)))
-    (classoid-typep (layout-friend layout) classoid object)))
-
 ;;; Return true of any object which is either a funcallable-instance,
 ;;; or an ordinary instance that is not a structure-object.
 (declaim (inline %pcl-instance-p))
@@ -193,9 +178,9 @@
 ;;; Try to ensure that the object's layout is up-to-date only if it is an instance
 ;;; or funcallable-instance of other than a static or structure classoid type.
 (defun update-object-layout (object)
-  (wrapper-friend (if (%pcl-instance-p object)
-                      (sb-pcl::check-wrapper-validity object)
-                      (wrapper-of object))))
+  (if (%pcl-instance-p object)
+      (sb-pcl::check-wrapper-validity object)
+      (layout-of object)))
 
 ;;; Test whether OBJ-LAYOUT is from an instance of CLASSOID.
 
@@ -227,7 +212,7 @@
 ;;; of thousands of iterations of 'classoid-typep.impure.lisp'
 
 (defun classoid-typep (obj-layout classoid object)
-  (declare (type wrapper obj-layout))
+  (declare (type layout obj-layout))
   ;; FIXME & KLUDGE: We could like to grab the *WORLD-LOCK* here (to ensure that
   ;; class graph doesn't change while we're doing the typep test), but in
   ;; practice that causes trouble -- deadlocking against the compiler
@@ -242,28 +227,41 @@
              ;; that we're testing against. Whether obj-layout is "valid" has no relevance.
              ;; This is racy though because %ENSURE-CLASSOID-VALID should return
              ;; the most up-to-date layout for the classoid, but it doesn't. Oh well.
-             (%ensure-classoid-valid classoid (classoid-wrapper classoid) "typep")
-             (values obj-layout (classoid-wrapper classoid)))
+             (%ensure-classoid-valid classoid (classoid-layout classoid) "typep")
+             (values obj-layout (classoid-layout classoid)))
             (t
              ;; And this case is even more racy, naturally.
-             (do ((layout (classoid-wrapper classoid) (classoid-wrapper classoid))
+             (do ((layout (classoid-layout classoid) (classoid-layout classoid))
                   (i 0 (+ i 1))
                   (obj-layout obj-layout))
-                 ((and (not (wrapper-invalid obj-layout))
-                       (not (wrapper-invalid layout)))
+                 ((and (not (layout-invalid obj-layout))
+                       (not (layout-invalid layout)))
                   (values obj-layout layout))
                (aver (< i 2))
                (%ensure-classoid-valid classoid layout "typep")
-               (when (zerop (wrapper-clos-hash obj-layout))
+               (when (zerop (layout-clos-hash obj-layout))
                  (setq obj-layout (sb-pcl::check-wrapper-validity object))))))
     ;; FIXME: if LAYOUT is for a structure, use the STRUCTURE-IS-A test
     ;; which avoids iterating.
     (or (eq obj-layout layout)
-        (let ((obj-inherits (wrapper-inherits obj-layout)))
+        (let ((obj-inherits (layout-inherits obj-layout)))
           (dotimes (i (length obj-inherits) nil)
             (when (eq (svref obj-inherits i) layout)
               (return t)))))))
 
+;;; Do a type test from a class cell, allowing forward reference and
+;;; redefinition.
+(defun classoid-cell-typep (cell object)
+  (let ((layout (typecase object
+                  (instance (%instance-layout object))
+                  (funcallable-instance (%fun-layout object))
+                  (t (return-from classoid-cell-typep))))
+        (classoid (classoid-cell-classoid (truly-the classoid-cell cell))))
+    (unless classoid
+      (error "The class ~S has not yet been defined."
+             (classoid-cell-name cell)))
+    (classoid-typep layout classoid object)))
+
 (declaim (end-block))
 
 ;;; If TYPE is a type that we can do a compile-time test on, then
@@ -299,7 +297,7 @@
          (if (if (csubtypep type (specifier-type 'function))
                  (funcallable-instance-p obj)
                  (%instancep obj))
-             (if (eq (classoid-wrapper type)
+             (if (eq (classoid-layout type)
                      (info :type :compiler-layout (classoid-name type)))
                  (values (sb-xc:typep obj type) t)
                  (values nil nil))
@@ -318,17 +316,10 @@
      ;; to happen in more places too. (Like the array hairy type
      ;; testing.)
      (if (unknown-type-p type)
-         (let ((spec (unknown-type-specifier type)))
-           ;; KLUDGE: Work around the fact that we load PCL after we
-           ;; type test certain things defined there. This is somewhat
-           ;; suboptimal because this is really only a concern during
-           ;; warm load.
-           (if (member spec '(class sb-pcl::condition-class))
+         (let ((type (specifier-type (unknown-type-specifier type))))
+           (if (unknown-type-p type)
                (values nil nil)
-               (let ((type (specifier-type spec)))
-                 (if (unknown-type-p type)
-                     (values nil nil)
-                     (ctypep obj type)))))
+               (ctypep obj type)))
          ;; Now the tricky stuff.
          (let ((predicate (cadr (hairy-type-specifier type))))
            (case predicate
@@ -407,3 +398,78 @@ Experimental."
   ;; just deem it invalid.
   (not (null (ignore-errors
                (type-or-nil-if-unknown type-specifier t)))))
+
+;;; This is like TYPE-OF, only we return a CTYPE structure instead of
+;;; a type specifier, and we try to return the type most useful for
+;;; type checking, rather than trying to come up with the one that the
+;;; user might find most informative.
+;;;
+;;; The result is always hash-consed, and in most cases there is only a very tiny amount
+;;; of work to decide what to return. Also, constants get their type stored in their LEAF,
+;;; so there is little to no advantage to using DEFUN-CACHED for this.
+(defun ctype-of (x)
+  (macrolet (#+sb-simd-pack
+             (simd-subtype (tag base-type &aux (n-ets (length +simd-pack-element-types+)))
+               `(let ((tag ,tag))
+                  (svref ,(map-into (make-array (1+ n-ets)
+                                                :initial-element (specifier-type base-type))
+                                    (lambda (x) (specifier-type `(,base-type ,x)))
+                                    +simd-pack-element-types+)
+                         (if (<= 0 tag ,(1- n-ets)) tag ,n-ets)))))
+    (typecase x
+      (function
+       (if (funcallable-instance-p x)
+           (classoid-of x)
+           ;; This is the only case that conses now. Apparently %FUN-FTYPE has to TYPEXPAND
+           ;; the type as stored, which generally uses SFUNCTION so that we can represent
+           ;; the strict number of return values. TYPEXPAND of course conses.
+           ;; But I seriously doubt that this function is often called on functions.
+           (let ((type (sb-impl::%fun-ftype x)))
+             (if (typep type '(cons (eql function))) ; sanity test
+                 (specifier-type type) ; cached
+                 (classoid-of x)))))
+      (symbol (make-eql-type x)) ; hash-consed
+      (number (ctype-of-number x)) ; hash-consed
+      (array
+       ;; The main difficulty here is that DIMENSIONS have to be constructed
+       ;; to pass to %MAKE-ARRAY-TYPE but with care we can usually avoid consing.
+       (let ((etype (sb-vm::array-element-ctype x))
+             (rank (array-rank x))
+             (complexp (not (simple-array-p x))))
+         (case rank
+           (0 (%make-array-type '() complexp etype etype))
+           ((1 2 3)
+            (dx-let ((dims (list (array-dimension x 0) nil nil)))
+              (case rank
+                (1 (setf (cdr dims) nil))
+                (2 (setf (cadr dims) (array-dimension x 1)
+                         (cddr dims) nil))
+                (t (setf (cadr dims) (array-dimension x 1)
+                         (caddr dims) (array-dimension x 2))))
+              (%make-array-type dims complexp etype etype)))
+           (t
+            (let ((dims (make-list rank)))
+              ;; Need ALLOCATE-LIST-ON-STACK for this decl. Can't use vop-exists-p
+              ;; because can't macroexpand into DECLARE. Maybe sharp-dot it ?
+              #+x86-64 (declare (truly-dynamic-extent dims))
+              (dotimes (i rank)
+                (setf (nth i dims) (array-dimension x i)))
+           (%make-array-type dims complexp etype etype))))))
+      (cons
+       (make-cons-type (if (eq (car x) x)
+                           (specifier-type 'cons)
+                           (ctype-of (car x)))
+                       (if (consp (cdr x))
+                           (specifier-type 'cons)
+                           (ctype-of (cdr x)))))
+      (character ; Why not return an EQL type?
+       (typecase x
+         (standard-char (specifier-type 'standard-char))
+         (base-char (specifier-type 'base-char))
+         (t (specifier-type 'extended-char))))
+      #+sb-simd-pack
+      (simd-pack (simd-subtype (%simd-pack-tag x) simd-pack))
+      #+sb-simd-pack-256
+      (simd-pack-256 (simd-subtype (%simd-pack-256-tag x) simd-pack-256))
+      (t
+       (classoid-of x)))))
