@@ -182,7 +182,9 @@ corresponds to NAME, or NIL if there is none."
     (locally
         (declare (optimize (sb-c::float-accuracy 0)))
       (let ((result (alien-funcall (extern-alien "open" (function int c-string int &optional int))
-                                   path (logior flags) mode)))
+                                   path (logior flags
+                                                #+largefile o_largefile)
+                                   mode)))
         (if (minusp result)
             (values nil (get-errno))
             (values result 0))))))
@@ -549,7 +551,7 @@ avoiding atexit(3) hooks, etc. Otherwise exit(2) is called."
 (defun unix-ioctl (fd cmd arg)
   (declare (type unix-fd fd)
            (type word cmd))
-  (void-syscall ("ioctl" int unsigned-long (* char)) fd cmd arg))
+  (void-syscall ("ioctl" int unsigned-long &optional (* char)) fd cmd arg))
 
 ;;;; sys/resource.h
 
@@ -868,8 +870,9 @@ avoiding atexit(3) hooks, etc. Otherwise exit(2) is called."
 
 (define-alien-routine get-timezone int
   (when time-t)
-  ;; KLUDGE: the runtime `boolean' is defined as `int', but the alien
-  ;; type is N-WORD-BITS wide.
+  ;; BOOLEAN is N-WORD-BITS normally. Reduce it to an unsigned int in size.
+  ;; But we can't just put UNSIGNED-INT here because the clients of this function
+  ;; want to receive a T or NIL, not a 1 or 0.
   (daylight-savings-p (boolean 32) :out))
 #-win32
 (defun nanosleep (secs nsecs)
