@@ -117,6 +117,10 @@
 (defknown sign-extend ((signed-byte 64) t) fixnum
     (foldable flushable movable))
 
+(defoptimizer (sign-extend derive-type) ((x size))
+  (when (sb-c:constant-lvar-p size)
+    (specifier-type `(signed-byte ,(sb-c:lvar-value size)))))
+
 (define-vop (sign-extend)
   (:translate sign-extend)
   (:policy :fast-safe)
@@ -200,11 +204,10 @@
   (:args)
   (:arg-types (:constant simple-string))
   (:info foreign-symbol)
-  (:temporary (:scs (interior-reg)) lip)
   (:results (res :scs (sap-reg)))
   (:result-types system-area-pointer)
   (:generator 2
-    (load-inline-constant res `(:fixup ,foreign-symbol :foreign) lip)))
+    (load-foreign-symbol res foreign-symbol)))
 
 (define-vop (foreign-symbol-dataref-sap)
   (:translate foreign-symbol-dataref-sap)
@@ -212,23 +215,18 @@
   (:args)
   (:arg-types (:constant simple-string))
   (:info foreign-symbol)
-  (:temporary (:scs (interior-reg)) lip)
   (:results (res :scs (sap-reg)))
   (:result-types system-area-pointer)
   (:generator 2
-    (load-inline-constant res `(:fixup ,foreign-symbol :foreign-dataref) lip)
-    (inst ldr res (@ res))))
+    (load-foreign-symbol res foreign-symbol :dataref t)))
 
-(defun emit-c-call (vop nfp-save temp temp2 lip cfunc function)
+#+sb-safepoint
+(defconstant thread-saved-csp-slot (- (1+ sb-vm::thread-header-slots)))
+
+(defun emit-c-call (vop nfp-save temp temp2 cfunc function)
   (let ((cur-nfp (current-nfp-tn vop)))
     (when cur-nfp
       (store-stack-tn nfp-save cur-nfp))
-    (if (stringp function)
-        (load-inline-constant cfunc `(:fixup ,function :foreign) lip)
-        (sc-case function
-          (sap-reg (move cfunc function))
-          (sap-stack
-           (load-stack-offset cfunc cur-nfp function))))
     (assemble ()
       #+sb-thread
       (progn
@@ -238,11 +236,21 @@
         (inst adr temp2 return)
         (inst stp cfp-tn temp2 (@ csp-tn))
         (storew-pair csp-tn thread-control-frame-pointer-slot temp thread-control-stack-pointer-slot thread-tn)
-        (inst blr cfunc)
-
+        ;; OK to run GC without stopping this thread from this point
+        ;; on.
+        #+sb-safepoint
+        (storew csp-tn thread-tn thread-saved-csp-slot)
+        (cond ((stringp function)
+               (invoke-foreign-routine function cfunc))
+              (t
+               (sc-case function
+                 (sap-reg (move cfunc function))
+                 (sap-stack
+                  (load-stack-offset cfunc cur-nfp function)))
+               (inst blr cfunc)))
         (loop for reg in (list r0-offset r1-offset r2-offset r3-offset
                                r4-offset r5-offset r6-offset r7-offset
-                               #-darwin r10-offset)
+                               #-darwin r8-offset)
               do
               (inst mov
                     (make-random-tn
@@ -250,13 +258,21 @@
                      :sc (sc-or-lose 'descriptor-reg)
                      :offset reg)
                     0))
+        ;; No longer OK to run GC except at safepoints.
+        #+sb-safepoint
+        (storew zr-tn thread-tn thread-saved-csp-slot)
         (storew zr-tn thread-tn thread-control-stack-pointer-slot))
       return
       #-sb-thread
       (progn
         temp2
-        (load-inline-constant temp '(:fixup "call_into_c" :foreign) lip)
-        (inst blr temp))
+        (if (stringp function)
+            (load-foreign-symbol cfunc function)
+            (sc-case function
+              (sap-reg (move cfunc function))
+              (sap-stack
+              (load-stack-offset cfunc cur-nfp function))))
+        (invoke-foreign-routine "call_into_c" temp))
       (when cur-nfp
         (load-stack-tn cur-nfp nfp-save)))))
 
@@ -266,32 +282,32 @@
                       nl4-offset nl5-offset nl6-offset nl7-offset nl8-offset nl9-offset
                       r0-offset r1-offset r2-offset r3-offset
                       r4-offset r5-offset r6-offset r7-offset
-                      #-darwin r10-offset
+                      #-darwin r8-offset
                       #-sb-thread r11-offset))
           (vars))
       (append
        (loop for gpr in gprs
              collect `(:temporary (:sc any-reg :offset ,gpr :from :eval :to :result)
-                                  ,(car (push (sb-xc:gensym) vars))))
+                                  ,(car (push (gensym) vars))))
        (loop for float to 31
              collect `(:temporary (:sc single-reg :offset ,float :from :eval :to :result)
-                                  ,(car (push (sb-xc:gensym) vars))))
+                                  ,(car (push (gensym) vars))))
        `((:ignore ,@vars))))))
 
 (define-vop (call-out)
   (:args (function :scs (sap-reg sap-stack))
          (args :more t))
   (:results (results :more t))
-  (:ignore args results)
+  (:ignore args results lr)
+  (:temporary (:sc non-descriptor-reg :offset lr-offset) lr)
   (:temporary (:sc any-reg :offset r9-offset
                :from (:argument 0) :to (:result 0)) cfunc)
   (:temporary (:sc control-stack :offset nfp-save-offset) nfp-save)
-  (:temporary (:sc any-reg :offset r8-offset) temp)
+  (:temporary (:sc any-reg :offset #+darwin r8-offset #-darwin r10-offset) temp)
   (:temporary (:sc any-reg :offset lexenv-offset) temp2)
-  (:temporary (:scs (interior-reg)) lip)
   (:vop-var vop)
   (:generator 0
-    (emit-c-call vop nfp-save temp temp2 lip cfunc function))
+    (emit-c-call vop nfp-save temp temp2 cfunc function))
   .
   #. (destroyed-c-registers))
 
@@ -300,7 +316,7 @@
 (define-vop (call-out-named call-out)
   (:args (args :more t))
   (:info function variadic)
-  (:ignore args results variadic))
+  (:ignore args results variadic lr))
 
 (define-vop (alloc-number-stack-space)
   (:info amount)
