@@ -117,6 +117,14 @@
         (t
          (load-immediate-word temp x))))
 
+(defun ccmp-immediate (x &optional (temp tmp-tn))
+  (cond ((not (integerp x))
+         x)
+        ((typep x '(unsigned-byte 5))
+         x)
+        (t
+         (load-immediate-word temp x))))
+
 (define-move-fun (load-immediate 1) (vop x y)
   ((immediate)
    (any-reg descriptor-reg))
@@ -135,7 +143,12 @@
               (tagged (dpb bits (byte 32 32) single-float-widetag)))
          (load-immediate-word y tagged)))
       (symbol
-       (load-symbol y val)))))
+       (load-symbol y val))
+      (structure-object
+       (if (eq val sb-lockless:+tail+)
+           (inst add y null-tn (- lockfree-list-tail-value-offset
+                                  nil-value-offset))
+           (bug "immediate structure-object ~S" val))))))
 
 (define-move-fun (load-number 1) (vop x y)
   ((immediate)
@@ -184,23 +197,19 @@
 ;;;; The Move VOP:
 (define-vop (move)
   (:args (x :target y
-            :scs (any-reg descriptor-reg)
-            :load-if (not (or (location= x y)
-                              (and (sc-is x immediate)
-                                   (sc-is y control-stack)
-                                   (eql (tn-value x) 0))))))
+            :scs (any-reg descriptor-reg zero)
+            :load-if (not (location= x y))))
   (:results (y :scs (any-reg descriptor-reg control-stack)
                :load-if (not (location= x y))))
   (:generator 0
-    (let ((x (if (and (sc-is x immediate)
-                      (eql (tn-value x) 0))
-                 zr-tn
-                 x)))
-      (cond ((location= x y))
-            ((sc-is y control-stack)
-             (store-stack-tn y x))
-            (t
-             (move y x))))))
+    (cond ((location= x y))
+          ((sc-is y control-stack)
+           (store-stack-tn y x))
+          ((and (sc-is x any-reg)
+                (eql (tn-offset x) zr-offset))
+           (inst mov y 0))
+          (t
+           (move y x)))))
 
 (define-move-vop move :move
   (any-reg descriptor-reg)
@@ -210,23 +219,19 @@
 ;;; frame for argument or known value passing.
 (define-vop (move-arg)
   (:args (x :target y
-            :scs (any-reg descriptor-reg)
-            :load-if (not (and (sc-is x immediate)
-                               (sc-is y control-stack)
-                               (eql (tn-value x) 0))))
+            :scs (any-reg descriptor-reg zero))
          (fp :scs (any-reg)
              :load-if (not (sc-is y any-reg descriptor-reg))))
   (:results (y))
   (:generator 0
     (sc-case y
       ((any-reg descriptor-reg)
-       (move y x))
+       (if (and (sc-is x any-reg)
+                (eql (tn-offset x) zr-offset))
+           (inst mov y 0)
+           (move y x)))
       (control-stack
-       (store-stack-offset (if (and (sc-is x immediate)
-                                    (eql (tn-value x) 0))
-                               zr-tn
-                               x)
-                           fp y)))))
+       (store-stack-offset x fp y)))))
 ;;;
 (define-move-vop move-arg :move-arg
   (any-reg descriptor-reg)
@@ -259,10 +264,13 @@
              (load-arg (x load-tn)
                (sc-case x
                  ((constant immediate control-stack)
-                  (let ((load-tn (if (and used-load-tn
-                                          (location= used-load-tn load-tn))
-                                     tmp-tn
-                                     load-tn)))
+                  (let ((load-tn (cond ((not (and used-load-tn
+                                                  (location= used-load-tn load-tn)))
+                                        load-tn)
+                                       ((sb-c::tn-reads load-tn)
+                                        (return-from load-arg))
+                                       (t
+                                        tmp-tn))))
                     (setf used-load-tn load-tn)
                     (sc-case x
                       (constant
@@ -419,7 +427,7 @@
 ;;; ARG is a fixnum or bignum; figure out which and load if necessary.
 (define-vop (move-to-word/integer)
   (:args (x :scs (descriptor-reg)))
-  (:results-var results)
+  (:result-refs results)
   (:results (y :scs (signed-reg unsigned-reg)))
   (:note "integer to untagged word coercion")
   (:generator 4
@@ -453,13 +461,11 @@
 ;;; RESULT may be a bignum, so we have to check.  Use a worst-case
 ;;; cost to make sure people know they may be number consing.
 (define-vop (move-from-signed)
-  (:args (arg :scs (signed-reg unsigned-reg) :target x))
+  (:args (x :scs (signed-reg unsigned-reg) :to :result))
   (:results (y :scs (any-reg descriptor-reg)))
-  (:temporary (:scs (non-descriptor-reg) :from (:argument 0)) x)
   (:temporary (:sc non-descriptor-reg :offset lr-offset) lr)
   (:note "signed word to integer coercion")
   (:generator 20
-    (move x arg)
     (inst adds y x x)
     (inst b :vc DONE)
     (with-fixed-allocation (y lr bignum-widetag (1+ bignum-digits-offset)
@@ -495,13 +501,11 @@
 ;;; result.  Use a worst-case cost to make sure people know they may
 ;;; be number consing.
 (define-vop (move-from-unsigned)
-  (:args (arg :scs (signed-reg unsigned-reg) :target x))
+  (:args (x :scs (signed-reg unsigned-reg) :to :save))
   (:results (y :scs (any-reg descriptor-reg)))
-  (:temporary (:scs (non-descriptor-reg) :from (:argument 0)) x)
   (:temporary (:sc non-descriptor-reg :offset lr-offset) lr)
   (:note "unsigned word to integer coercion")
   (:generator 20
-    (move x arg)
     (inst tst x (ash (1- (ash 1 (- n-word-bits
                                    n-positive-fixnum-bits)))
                      n-positive-fixnum-bits))
