@@ -53,7 +53,13 @@
 (declaim (type (and (vector t) (not simple-array)) *constraint-universe*))
 (defvar *constraint-universe*)
 
-(deftype constraint-y () '(or ctype lvar lambda-var constant))
+(defstruct (vector-length-constraint
+            (:constructor make-vector-length-constraint (var))
+            (:copier nil))
+  (var nil :type lambda-var :read-only t))
+
+(deftype constraint-y () '(or ctype lvar lambda-var constant
+                           vector-length-constraint))
 
 (defstruct (constraint
             (:include sset-element)
@@ -65,7 +71,7 @@
   ;;     X is a LAMBDA-VAR and Y is a CTYPE. The value of X is
   ;;     constrained to be of type Y.
   ;;
-  ;; >, <, or =
+  ;; >, <, >=, <=, or =
   ;;     X is a lambda-var and Y is a CTYPE. The relation holds
   ;;     between X and some object of type Y.
   ;;
@@ -73,21 +79,24 @@
   ;;     X is a LAMBDA-VAR and Y is a LVAR, a LAMBDA-VAR or a CONSTANT.
   ;;     The relation is asserted to hold.
   ;;
-  ;; ARRAY-IN-BOUNDS-P
-  ;;     To handle (array-in-bounds-p array var) and
-  ;;     (array-in-bounds-p array 10) X can be either the lambda-var
-  ;;     of ARRAY or VAR, while Y is either the lambda-var of VAR or a
-  ;;     constant.
-  (kind nil :type (member typep < > = eql
-                          array-in-bounds-p
-                          equality))
+  ;; EQUALITY
+  ;;     Relations between two variables.
+  (kind nil :type (member typep < > = >= <= eql equality))
   ;; The operands to the relation.
-  (x nil :type lambda-var)
+  (x nil :type (or lambda-var vector-length-constraint))
   (y nil :type constraint-y)
   ;; If true, negates the sense of the constraint, so the relation
   ;; does *not* hold.
   (not-p nil :type boolean))
 
+;;; The basic interval type. It can handle open and closed intervals.
+;;; A bound is open if it is a list containing a number, just like
+;;; Lisp says. NIL means unbounded.
+(defstruct (interval (:constructor %make-interval (low high))
+                     (:copier nil))
+  low high)
+(declaim (freeze-type interval))
+
 
 ;;; Historically, CMUCL and SBCL have used a sparse set implementation
 ;;; for which most operations are O(n) (see sset.lisp), but at the
@@ -205,6 +214,10 @@
         (setf (conset-max conset) (1+ number))))
     conset)
 
+  (defun conset-delete (constraint conset)
+    (setf (sbit (conset-vector conset) (%constraint-number constraint)) 0)
+    conset)
+
   (defun conset= (conset1 conset2)
     (let* ((vector1 (conset-vector conset1))
            (vector2 (conset-vector conset2))
@@ -320,7 +333,6 @@
 ;;; through the intersection of such constraints and the constraints in a conset.
 ;;;
 ;;; EQL-var constraints assert that two lambda-vars are EQL.
-;;; Private constraints assert that a lambda-var is EQL or not EQL to a constant.
 ;;; Inheritable constraints are constraints that may be propagated to EQL
 ;;; lambda-vars (along with EQL-var constraints).
 ;;;
@@ -357,11 +369,10 @@
              (setf (car cons) con)))
        (typecase y
          (constant
-          (let ((vec (ensure-vec (lambda-var-private-constraints x))))
+          (let ((vec (ensure-vec (lambda-var-inheritable-constraints x))))
             (vector-push-extend con vec)))
          (lambda-var
-          (let ((vec (if (or (constraint-not-p con)
-                             (eq (constraint-kind con) 'array-in-bounds-p))
+          (let ((vec (if (constraint-not-p con)
                          (ensure-vec (lambda-var-inheritable-constraints x))
                          (ensure-vec (lambda-var-eql-var-constraints x)))))
             (vector-push-extend con vec)))))))
@@ -485,9 +496,6 @@
     `(block nil
        (flet ((body-fun (,symbol)
                 ,@body))
-         (do-conset-constraints-intersection
-             (con (,conset (lambda-var-private-constraints ,variable)))
-           (body-fun con))
          (do-conset-constraints-intersection
              (con (,conset (lambda-var-eql-var-constraints ,variable)))
            (body-fun con))
@@ -539,7 +547,8 @@
                                    (constraint-kind con)
                                    x
                                    y
-                                   (constraint-not-p con))))))))
+                                   (constraint-not-p con)))))))
+  (inherit-equality-constraints vars from-var constraints target))
 
 ;; Add an (EQL LAMBDA-VAR LAMBDA-VAR) constraint on VAR1 and VAR2 and
 ;; inherit each other's constraints.
@@ -630,12 +639,18 @@
                                          consequent-constraints
                                          alternative-constraints
                                          quick-p)
-  (flet ((add (fun x y not-p)
-           (add-complement-constraints quick-p
-                                       fun x y not-p
-                                       constraints
-                                       consequent-constraints
-                                       alternative-constraints))
+  (flet ((add (fun x y &optional no-complement)
+           (if no-complement
+               (when x
+                 (add-test-constraint quick-p
+                                      fun x y nil
+                                      constraints
+                                      consequent-constraints))
+               (add-complement-constraints quick-p
+                                           fun x y nil
+                                           constraints
+                                           consequent-constraints
+                                           alternative-constraints)))
          (prop (triples target)
            (map nil (lambda (constraint)
                       (destructuring-bind (kind x y &optional not-p)
@@ -651,13 +666,13 @@
                  (propagate (fun-info-constraint-propagate-if
                              info)
                             :exit-if-null))
-        (multiple-value-bind (lvar type if else)
+        (multiple-value-bind (lvar type if else no-complement)
             (funcall propagate use constraints)
           (prop if consequent-constraints)
           (prop else alternative-constraints)
           (when (and lvar type)
             (add 'typep (ok-lvar-lambda-var lvar constraints)
-                 type nil)
+                 type no-complement)
             (return-from add-combination-test-constraints)))))
     (let* ((name (lvar-fun-name
                   (basic-combination-fun use)))
@@ -666,31 +681,7 @@
       (when ptype
         (add 'typep (ok-lvar-lambda-var (first args)
                                         constraints)
-             ptype nil)))))
-
-(defun array-in-bounds-p-constraints (constraints index-lvar index-var
-                                      length-lvar)
-  (let ((index-constant
-          (and (not index-var)
-               (let ((use (principal-lvar-use index-lvar)))
-                 (and (ref-p use)
-                      (constant-p (ref-leaf use))
-                      (ref-leaf use)))))
-        (array-lvar
-          (let ((use (principal-lvar-ref-use length-lvar)))
-            (and (combination-p use)
-                 (lvar-fun-is (combination-fun use)
-                              '(vector-length length))
-                 (car (combination-args use))))))
-    (when (and (or index-var index-constant)
-               array-lvar)
-      (let ((array-var (ok-lvar-lambda-var array-lvar constraints)))
-        (when array-var
-          (if index-constant
-              ;; Attach the constaraint to the array if
-              ;; the index is constant
-              (values 'array-in-bounds-p array-var index-constant)
-              (values 'array-in-bounds-p index-var array-var)))))))
+             ptype)))))
 
 ;;; Add test constraints to the consequent and alternative blocks of
 ;;; the test represented by USE.
@@ -704,105 +695,106 @@
     (let ((consequent-constraints (make-conset))
           (alternative-constraints (make-conset))
           (quick-p (policy if (> compilation-speed speed))))
-      (macrolet ((add (fun x y not-p)
-                   `(add-complement-constraints quick-p
-                                                ,fun ,x ,y ,not-p
-                                                constraints
-                                                consequent-constraints
-                                                alternative-constraints)))
-        (typecase use
-          (ref
-           (add 'typep (ok-lvar-lambda-var (ref-lvar use) constraints)
-                (specifier-type 'null) t))
-          (combination
-           (unless (eq (combination-kind use)
-                       :error)
-             (let ((name (uncross
-                          (lvar-fun-name
-                           (basic-combination-fun use))))
-                   (args (basic-combination-args use)))
-               (add-equality-constraints name args
-                                         constraints consequent-constraints alternative-constraints)
-               (case name
-                 ((%typep %instance-typep)
-                  (let ((type (second args)))
-                    (when (constant-lvar-p type)
-                      (let ((val (lvar-value type)))
-                        (add 'typep
-                             (ok-lvar-lambda-var (first args) constraints)
-                             (if (ctype-p val)
-                                 val
-                                 (let ((*compiler-error-context* use))
-                                   (specifier-type val)))
-                             nil)))))
-                 ((eq eql)
-                  (let* ((arg1 (first args))
-                         (var1 (ok-lvar-lambda-var arg1 constraints))
-                         (arg2 (second args))
-                         (var2 (ok-lvar-lambda-var arg2 constraints)))
-                    ;; The code below assumes that the constant is the
-                    ;; second argument in case of variable to constant
-                    ;; comparison which is sometimes true (see source
-                    ;; transformations for EQ, EQL and CHAR=). Fixing
-                    ;; that would result in more constant substitutions
-                    ;; which is not a universally good thing, thus the
-                    ;; unnatural asymmetry of the tests.
-                    (cond ((not var1)
-                           (when var2
-                             (add-test-constraint quick-p
-                                                  'typep var2 (lvar-type arg1)
-                                                  nil constraints
-                                                  consequent-constraints)))
-                          (var2
-                           (add 'eql var1 var2 nil))
-                          ((constant-lvar-p arg2)
-                           (add 'eql var1
-                                (nth-value 1 (lvar-value arg2))
-                                nil))
-                          (t
-                           (add-test-constraint quick-p
-                                                'typep var1 (lvar-type arg2)
-                                                nil constraints
-                                                consequent-constraints)))))
-                 ((< >)
-                  (when (= (length args) 2)
-                    (flet ((handle-array-in-bounds-p (index-arg index-var length-arg)
-                             (multiple-value-bind (kind x y)
-                                 (array-in-bounds-p-constraints constraints index-arg index-var
-                                                                length-arg)
-                               (when kind
-                                 (add-test-constraint quick-p
-                                                      kind x y
-                                                      nil constraints
-                                                      consequent-constraints)))))
-                      (let* ((arg1 (first args))
-                             (var1 (ok-lvar-lambda-var arg1 constraints))
-                             (arg2 (second args))
-                             (var2 (ok-lvar-lambda-var arg2 constraints)))
+      (labels ((add (fun x y not-p)
+                 (add-complement-constraints quick-p
+                                             fun x y not-p
+                                             constraints
+                                             consequent-constraints
+                                             alternative-constraints))
+               (process-node (node)
+                 (typecase node
+                   (ref
+                    (add 'typep (ok-lvar-lambda-var (ref-lvar node) constraints)
+                         (specifier-type 'null) t)
+                    (let ((use (principal-lvar-ref-use (ref-lvar use))))
+                      (when (and use
+                                 (not (ref-p use)))
+                        (process-node use))))
+                   (combination
+                    (unless (eq (combination-kind node) :error)
+                      (let ((name (uncross
+                                   (lvar-fun-name
+                                    (basic-combination-fun node))))
+                            (args (basic-combination-args node)))
+                        (add-equality-constraints name args
+                                                  constraints consequent-constraints alternative-constraints)
                         (case name
-                          (<
-                           (handle-array-in-bounds-p arg1 var1 arg2))
-                          (>
-                           (handle-array-in-bounds-p arg2 var2 arg1)))
-                        (when var1
-                          (add name var1 (lvar-type arg2) nil))
-                        (when var2
-                          (add (if (eq name '<) '> '<) var2 (lvar-type arg1) nil))))))
-                 (=
-                  (when (= (length args) 2)
-                    (let* ((arg1 (first args))
-                           (var1 (ok-lvar-lambda-var arg1 constraints))
-                           (arg2 (second args))
-                           (var2 (ok-lvar-lambda-var arg2 constraints)))
-                      (when var1
-                        (add name var1 (lvar-type arg2) nil))
-                      (when var2
-                        (add name var2 (lvar-type arg1) nil)))))
-                 (t
-                  (add-combination-test-constraints use constraints
-                                                    consequent-constraints
-                                                    alternative-constraints
-                                                    quick-p))))))))
+                          ((%typep %instance-typep)
+                           (let ((type (second args)))
+                             (when (constant-lvar-p type)
+                               (let ((val (lvar-value type)))
+                                 (add 'typep
+                                      (ok-lvar-lambda-var (first args) constraints)
+                                      (if (ctype-p val)
+                                          val
+                                          (let ((*compiler-error-context* node))
+                                            (specifier-type val)))
+                                      nil)))))
+                          ((eq eql)
+                           (let* ((arg1 (first args))
+                                  (var1 (ok-lvar-lambda-var arg1 constraints))
+                                  (arg2 (second args))
+                                  (var2 (ok-lvar-lambda-var arg2 constraints)))
+                             ;; The code below assumes that the constant is the
+                             ;; second argument in case of variable to constant
+                             ;; comparison which is sometimes true (see source
+                             ;; transformations for EQ, EQL and CHAR=). Fixing
+                             ;; that would result in more constant substitutions
+                             ;; which is not a universally good thing, thus the
+                             ;; unnatural asymmetry of the tests.
+                             (cond ((not var1)
+                                    (when var2
+                                      (add-test-constraint quick-p
+                                                           'typep var2 (lvar-type arg1)
+                                                           nil constraints
+                                                           consequent-constraints)))
+                                   (var2
+                                    (add 'eql var1 var2 nil))
+                                   ((constant-lvar-p arg2)
+                                    (add 'eql var1
+                                         (nth-value 1 (lvar-value arg2))
+                                         nil))
+                                   (t
+                                    (add-test-constraint quick-p
+                                                         'typep var1 (lvar-type arg2)
+                                                         nil constraints
+                                                         consequent-constraints)))))
+                          ((< >)
+                           (when (= (length args) 2)
+                             (let* ((arg1 (first args))
+                                    (var1 (ok-lvar-lambda-var arg1 constraints))
+                                    (arg2 (second args))
+                                    (var2 (ok-lvar-lambda-var arg2 constraints)))
+                               (when var1
+                                 (add name var1 (lvar-type arg2) nil))
+                               (when var2
+                                 (add (if (eq name '<) '> '<) var2 (lvar-type arg1) nil)))))
+                          ((<= >=)
+                           (when (= (length args) 2)
+                             (let* ((arg1 (first args))
+                                    (var1 (ok-lvar-lambda-var arg1 constraints))
+                                    (arg2 (second args))
+                                    (var2 (ok-lvar-lambda-var arg2 constraints)))
+                               (when var1
+                                 (add name var1 (lvar-type arg2) nil))
+                               (when var2
+                                 (add (if (eq name '<=) '>= '<=) var2 (lvar-type arg1) nil)))))
+                          (=
+                           (when (= (length args) 2)
+                             (let* ((arg1 (first args))
+                                    (var1 (ok-lvar-lambda-var arg1 constraints))
+                                    (arg2 (second args))
+                                    (var2 (ok-lvar-lambda-var arg2 constraints)))
+                               (when var1
+                                 (add name var1 (lvar-type arg2) nil))
+                               (when var2
+                                 (add name var2 (lvar-type arg1) nil)))))
+                          (t
+                           (add-combination-test-constraints node constraints
+                                                             consequent-constraints
+                                                             alternative-constraints
+                                                             quick-p)))))))))
+        (process-node use))
       (values consequent-constraints alternative-constraints))))
 
 ;;;; Applying constraints
@@ -907,6 +899,24 @@
             (make-numeric-type :low bound)
             (make-numeric-type :high bound))))))
 
+(defun constrain-real (y greater or-equal)
+  (let ((int (type-approximate-interval y)))
+    (when int
+      (flet ((exclude (x)
+               (cond ((not x) nil)
+                     (or-equal x)
+                     ((consp x) x)
+                     (t (list x))))
+             (bound (x)
+               (if greater
+                   (interval-low x)
+                   (interval-high x))))
+        (let ((bound (exclude (bound int))))
+          (when bound
+            (if greater
+                (make-numeric-type :low bound)
+                (make-numeric-type :high bound))))))))
+
 ;;; Return true if LEAF is "visible" from NODE.
 (defun leaf-visible-from-node-p (leaf node)
   (cond
@@ -999,24 +1009,10 @@
                              (when (null not-numeric)
                                (setf not-numeric (alloc-xset)))
                              (add-to-xset (constant-value other) not-numeric)))))))
-            ((< >)
-             (let* ((greater (eq kind '>))
-                    (greater (if not-p (not greater) greater)))
-               (cond ((and (integer-type-p type) (integer-type-p y))
-                      (setf type (constrain-integer-type type y greater not-p)))
-                     ((and (float-type-p type) (float-type-p y))
-                      (setf type (constrain-float-type type y greater not-p)))
-                     ((integer-type-p y)
-                      (let ((real-type (constrain-real-to-integer y greater not-p)))
-                        (when real-type (intersect-result real-type)))))))
-            (=
-             (when (and (numeric-type-p y)
-                        (not not-p))
-               (let* ((low (numeric-type-low y))
-                      (high (numeric-type-high y))
-                      (real (make-numeric-type :low low :high high))
-                      (complex (make-numeric-type :complexp :complex :low low :high high)))
-                 (intersect-result (type-union real complex)))))))))
+            ((< > <= >= =)
+             (let ((after (type-after-comparison kind not-p type y)))
+               (when after
+                 (setf type after))))))))
     (let* ((negated not-type)
            (negated (if (and (null not-set) (null not-fpz))
                         negated
@@ -1032,6 +1028,54 @@
           type
           (type-difference type negated)))))
 
+(defun type-after-comparison (operator not-p current-type type)
+  (case operator
+    ((= eq)
+     (unless not-p
+       (multiple-value-bind (lo hi)
+           (if (numeric-type-p type)
+               (values (numeric-type-low type)
+                       (numeric-type-high type))
+               ;; Doesn't handle infinities
+               (let ((int (type-approximate-interval type)))
+                 (and int
+                      (values
+                       (interval-low int)
+                       (interval-high int)))))
+         (when (or lo hi)
+           (type-intersection current-type
+                              (type-union (make-numeric-type :low lo
+                                                             :high hi)
+                                          (make-numeric-type :complexp :complex
+                                                             :low lo
+                                                             :high hi)))))))
+    (t
+     (multiple-value-bind (greater equal)
+         (if not-p
+             (case operator
+               (> (values nil t))
+               (< (values t t))
+               (>= (values nil nil))
+               (<= (values t nil)))
+             (case operator
+               (> (values t nil))
+               (< (values nil nil))
+               (>= (values t t))
+               (<= (values nil t))))
+       (cond
+         ((and (integer-type-p current-type) (integer-type-p type))
+          (constrain-integer-type current-type type greater equal))
+         ((and (float-type-p current-type) (float-type-p type))
+          (constrain-float-type current-type type greater equal))
+         ((integer-type-p type)
+          (let ((type (constrain-real-to-integer type greater equal)))
+            (when type
+              (type-intersection current-type type))))
+         (t
+          (let ((type (constrain-real type greater equal)))
+            (when type
+              (type-intersection current-type type)))))))))
+
 ;;; Given the set of CONSTRAINTS for a variable and the current set of
 ;;; restrictions from flow analysis IN, set the type for REF
 ;;; accordingly.
@@ -1048,7 +1092,8 @@
   ;; a more useful type, don't propagate their negation except for NIL
   ;; unless SPEED > COMPILATION-SPEED.
   (let ((res (single-value-type (node-derived-type ref)))
-        (constrain-symbols (policy ref (> speed compilation-speed)))
+        (constrain-symbols (policy ref (or (> speed compilation-speed)
+                                           (> debug 1))))
         (not-set (alloc-xset))
         (not-numeric (alloc-xset))
         (not-fpz nil)
@@ -1070,21 +1115,27 @@
             (equality
              (unless (eq (ref-constraints ref)
                          (pushnew con (ref-constraints ref)))
-               (let ((lvar (node-lvar ref))
-                     (principal-lvar (nth-value 1 (principal-lvar-dest-and-lvar (node-lvar ref)))))
-                 (reoptimize-lvar lvar)
-                 (unless (eq lvar principal-lvar)
-                   (reoptimize-lvar principal-lvar)))))
+               (labels ((reoptimize (node)
+                          (when (valued-node-p node)
+                            (let ((lvar (node-lvar node))
+                                  (principal-lvar (nth-value 1 (principal-lvar-end (node-lvar ref)))))
+                              (reoptimize-lvar lvar)
+                              (unless (eq lvar principal-lvar)
+                                (reoptimize-lvar principal-lvar)))))
+                        (try (x)
+                          (when (and (vector-length-constraint-p x)
+                                     (eq (vector-length-constraint-var x) leaf))
+                            (reoptimize (node-dest ref))
+                            t)))
+                 (or (try x)
+                     (try y)
+                     (reoptimize ref)))))
             (typep
              (if not-p
                  (if (member-type-p other)
                      (mapc-member-type-members #'note-not other)
                      (setq not-res (type-union not-res other)))
                  (setq res (type-intersection res other))))
-            (array-in-bounds-p
-             (unless (eq (ref-constraints ref)
-                         (pushnew con (ref-constraints ref)))
-               (reoptimize-lvar (node-lvar ref))))
             (eql
              (let ((other-type (leaf-type other)))
                (if not-p
@@ -1110,31 +1161,10 @@
                         (when (constant-p other) (return)))
                        (t
                         (setq res (type-intersection res other-type))))))))
-            ((< >)
-             (let* ((greater (eq kind '>))
-                    (greater (if not-p (not greater) greater)))
-               (cond
-                 ((and (integer-type-p res) (integer-type-p y))
-                  (setq res
-                        (constrain-integer-type res y greater not-p)))
-                 ((and (float-type-p res) (float-type-p y))
-                  (setq res
-                        (constrain-float-type res y greater not-p)))
-                 ((integer-type-p y)
-                  (let ((type (constrain-real-to-integer y greater not-p)))
-                    (when type
-                      (setf res
-                            (type-intersection res type))))))))
-            (=
-             (when (and (numeric-type-p y)
-                        (not not-p))
-               (setf res
-                     (type-intersection res
-                                        (type-union (make-numeric-type :low (numeric-type-low y)
-                                                                       :high (numeric-type-high y))
-                                                    (make-numeric-type :complexp :complex
-                                                                       :low (numeric-type-low y)
-                                                                       :high (numeric-type-high y)))))))))))
+            ((< > <= >= =)
+             (let ((type (type-after-comparison kind not-p res y)))
+               (when type
+                 (setf res type))))))))
     (cond ((and (if-p (node-dest ref))
                 (or (xset-member-p nil not-set)
                     (csubtypep (specifier-type 'null) not-res)))
@@ -1213,7 +1243,6 @@
          (when preprocess-refs-p
            (constrain-ref-type node gen))))
       (cast
-
        (let* ((lvar (cast-value node))
               (var (ok-lvar-lambda-var lvar gen)))
          (when var
@@ -1227,15 +1256,13 @@
              (destructuring-bind (array dim index)
                  (combination-args check-bound)
                (declare (ignore array))
-               (multiple-value-bind (kind x y)
-                   (array-in-bounds-p-constraints gen index var dim)
-                 (when kind
-                   (conset-add-constraint-to-eql gen kind x y nil))))))))
+               (add-equality-constraints '< (list index dim) gen gen nil))))))
       (cset
        (binding* ((var (set-var node))
                   (nil (lambda-var-p var) :exit-if-null)
                   (nil (lambda-var-constraints var) :exit-if-null))
-         (when (policy node (and (= speed 3) (> speed compilation-speed)))
+         (when (policy node (or (and (= speed 3) (> speed compilation-speed))
+                                (> debug 1)))
            (let ((type (lambda-var-type var)))
              (unless (eq *universal-type* type)
                (do-eql-vars (other (var gen))
@@ -1367,36 +1394,44 @@
 ;;; Join the constraints coming from the predecessors of BLOCK on
 ;;; every constrained variable into the constraint set IN.
 (defun join-type-constraints (in block)
-  (let ((vars '()))
-    ;; Find some set of constrained variables in the predecessors.
-    (dolist (pred (block-pred block))
-      (let ((out (block-out-for-successor pred block)))
-        (when out
-          (do-conset-elements (con out)
-            (let ((kind  (constraint-kind con))
-                  (y     (constraint-y con))
-                  (not-p (constraint-not-p con)))
-              (when (or (member kind '(typep < >))
-                        (and (eq kind 'eql) (or (not not-p)
-                                                (constant-p y)))
-                        (and (eq kind '=) (and (numeric-type-p y)
-                                               (not not-p))))
-                (pushnew (constraint-x con) vars))))
-          (return))))
-    (dolist (var vars)
-      (let ((in-var-type *empty-type*))
-        (dolist (pred (block-pred block))
-          (let ((out (block-out-for-successor pred block)))
-            (when out
-              (setq in-var-type
-                    (type-union in-var-type
-                                (type-from-constraints var out *universal-type*))))))
-        (unless (eq in-var-type *universal-type*)
-          (conset-adjoin (find-or-create-constraint 'typep
-                                                    var
-                                                    in-var-type
-                                                    nil)
-                         in))))))
+  (let ((vars '())
+        (predecessors (block-pred block)))
+    (progn
+      ;;unless (singleton-p predecessors) for some reason it makes the whole thing loop infinitely
+      ;; Find some set of constrained variables in the predecessors.
+      (dolist (pred predecessors)
+        (let ((out (block-out-for-successor pred block)))
+          (when out
+            (do-conset-elements (con out)
+              (let ((kind  (constraint-kind con))
+                    (y     (constraint-y con))
+                    (not-p (constraint-not-p con)))
+                (when (or (member kind '(typep < >))
+                          (and (eq kind 'eql) (or (not not-p)
+                                                  (constant-p y)))
+                          (and (eq kind '=) (and (numeric-type-p y)
+                                                 (not not-p))))
+                  (pushnew (constraint-x con) vars))))
+            (return))))
+      (dolist (var vars)
+        (let ((in-var-type *empty-type*))
+          (dolist (pred (block-pred block))
+            (let ((out (block-out-for-successor pred block)))
+              (when out
+                (setq in-var-type
+                      (type-union in-var-type
+                                  (type-from-constraints var out *universal-type*))))))
+
+          (unless (eq in-var-type *universal-type*)
+            ;; Remove the existing constraints to avoid joining them again later.
+            (do-propagatable-constraints (con (in var))
+              (when (eq (constraint-kind con) 'typep)
+                (conset-delete con in)))
+            (conset-adjoin (find-or-create-constraint 'typep
+                                                      var
+                                                      in-var-type
+                                                      nil)
+                           in)))))))
 
 (defun compute-block-in (block join-types-p)
   (let ((in nil))
