@@ -18,9 +18,15 @@
 ;;; passing locations and return conventions and TNs for local variables.
 (defun gtn-analyze (component)
   (setf (component-info component) (make-ir2-component))
-  (let ((funs (component-lambdas component)))
+  (let ((funs (component-lambdas component))
+        #+fp-and-pc-standard-save
+        (old-fp (make-old-fp-save-location))
+        #+fp-and-pc-standard-save
+        (old-pc (make-return-pc-save-location)))
     (dolist (fun funs)
-      (assign-ir2-physenv fun)
+      (assign-ir2-environment fun
+                              #+fp-and-pc-standard-save old-fp
+                              #+fp-and-pc-standard-save old-pc)
       (assign-ir2-nlx-info fun)
       (assign-lambda-var-tns fun nil)
       (dolist (let (lambda-lets fun))
@@ -29,7 +35,7 @@
   (values))
 
 ;;; We have to allocate the home TNs for variables before we can call
-;;; ASSIGN-IR2-PHYSENV so that we can close over TNs that haven't
+;;; ASSIGN-IR2-ENVIRONMENT so that we can close over TNs that haven't
 ;;; had their home environment assigned yet. Here we evaluate the
 ;;; DEBUG-INFO/SPEED tradeoff to determine how variables are
 ;;; allocated. If SPEED is 3, then all variables are subject to
@@ -59,29 +65,36 @@
                 (not (lambda-var-explicit-value-cell var)))
            ;; Force closed-over indirect LAMBDA-VARs without explicit
            ;; VALUE-CELLs to the stack, and make sure that they are
-           ;; live over the dynamic contour of the physenv.
+           ;; live over the dynamic contour of the environment.
            (setf (tn-sc res) (if ptype-info
                                  (second ptype-info)
                                  (sc-or-lose 'sb-vm::control-stack)))
-           (physenv-live-tn res (lambda-physenv fun)))
+           (environment-live-tn res (lambda-environment fun)))
 
           (debug-variable-p
-           (physenv-debug-live-tn res (lambda-physenv fun))))
+           ;; If it's a constant it may end up being never read,
+           ;; replaced by COERCE-FROM-CONSTANT.
+           ;; Yet it might get saved on the stack, but since it's
+           ;; never read no stack space is allocated for it in the
+           ;; callee frame.
+           (unless (type-singleton-p (leaf-type var))
+             (environment-debug-live-tn res (lambda-environment fun)))))
 
         (setf (tn-leaf res) var)
         (setf (tn-type res) (leaf-type var))
         (setf (leaf-info var) res))))
   (values))
 
-;;; Give CLAMBDA an IR2-PHYSENV structure. (And in order to
+;;; Give CLAMBDA an IR2-ENVIRONMENT structure. (And in order to
 ;;; properly initialize the new structure, we make the TNs which hold
 ;;; environment values and the old-FP/return-PC.)
-(defun assign-ir2-physenv (clambda)
+(defun assign-ir2-environment (clambda #+fp-and-pc-standard-save old-fp
+                                       #+fp-and-pc-standard-save old-pc)
   (declare (type clambda clambda))
-  (let* ((lambda-physenv (lambda-physenv clambda))
+  (let* ((lambda-environment (lambda-environment clambda))
          (indirect-fp-tns)
-         (ir2-physenv-alist
-           (loop for thing in (physenv-closure lambda-physenv)
+         (ir2-environment-alist
+           (loop for thing in (environment-closure lambda-environment)
                  collect
                  (cons thing
                        (etypecase thing
@@ -90,11 +103,11 @@
                                  (make-normal-tn
                                   (primitive-type (leaf-type thing))))
                                 ((not (lambda-var-explicit-value-cell thing))
-                                 (let ((physenv (lambda-physenv (lambda-var-home thing))))
-                                   (or (getf indirect-fp-tns physenv)
+                                 (let ((env (lambda-environment (lambda-var-home thing))))
+                                   (or (getf indirect-fp-tns env)
                                        (let ((tn (make-normal-tn *backend-t-primitive-type*)))
                                          (push tn indirect-fp-tns)
-                                         (push physenv indirect-fp-tns)
+                                         (push env indirect-fp-tns)
                                          tn))))
                                 (t
                                  (make-normal-tn *backend-t-primitive-type*))))
@@ -102,15 +115,27 @@
                           (make-normal-tn *backend-t-primitive-type*))
                          (clambda
                           (make-normal-tn *backend-t-primitive-type*)))))))
-    (let ((res (make-ir2-physenv
-                :closure ir2-physenv-alist
-                :return-pc-pass (make-return-pc-passing-location
-                                 (xep-p clambda)))))
-      (setf (physenv-info lambda-physenv) res)
-      (setf (ir2-physenv-old-fp res)
-            (make-old-fp-save-location lambda-physenv))
-      (setf (ir2-physenv-return-pc res)
-            (make-return-pc-save-location lambda-physenv))))
+    (let ((res (make-ir2-environment
+                :closure ir2-environment-alist
+                :return-pc-pass #+fp-and-pc-standard-save
+                                old-pc
+                                #-fp-and-pc-standard-save
+                                (make-return-pc-passing-location (xep-p clambda)))))
+      (setf (environment-info lambda-environment) res)
+      (setf (ir2-environment-old-fp res)
+            #-fp-and-pc-standard-save
+            (make-old-fp-save-location lambda-environment)
+            #+fp-and-pc-standard-save
+            old-fp)
+      (setf (ir2-environment-return-pc res)
+            #-fp-and-pc-standard-save
+            (make-return-pc-save-location lambda-environment)
+            #+fp-and-pc-standard-save
+            old-pc)
+      #+fp-and-pc-standard-save
+      (progn
+        (push old-fp (ir2-environment-live-tns (environment-info lambda-environment)))
+        (push old-pc (ir2-environment-live-tns (environment-info lambda-environment))))))
 
   (values))
 
@@ -128,7 +153,13 @@
 (defun use-standard-returns (tails)
   (declare (type tail-set tails))
   (let ((funs (tail-set-funs tails)))
-    (or (find-if #'xep-p funs)
+    (or (loop for fun in funs
+              for fun-info = (info :function :info (functional-%source-name fun))
+              when
+              (and fun-info
+                   (ir1-attributep (fun-info-attributes fun-info) unboxed-return))
+              return :unboxed)
+        (find-if #'xep-p funs)
         (some (lambda (fun) (policy fun (>= insert-debug-catch 2))) funs)
         (block punt
           (dolist (fun funs t)
@@ -143,12 +174,12 @@
                                (unless (and (basic-combination-p node)
                                             (eq (basic-combination-info node) :full))
                                  (return)))))))
-                 (when (and (basic-combination-p dest)
-                            (not (node-tail-p dest))
-                            (eq (basic-combination-fun dest) lvar)
-                            (eq (basic-combination-kind dest) :local)
-                            (not (all-returns-tail-calls-p dest)))
-                   (return-from punt nil))))))))))
+                  (when (and (basic-combination-p dest)
+                             (not (node-tail-p dest))
+                             (eq (basic-combination-fun dest) lvar)
+                             (eq (basic-combination-kind dest) :local)
+                             (not (all-returns-tail-calls-p dest)))
+                    (return-from punt nil))))))))))
 
 ;;; If policy indicates, give an efficiency note about our inability to
 ;;; use the known return convention. We try to find a function in the
@@ -195,16 +226,26 @@
       (when (and (eq count :unknown) (not use-standard)
                  (not (eq (tail-set-type tails) *empty-type*)))
         (return-value-efficiency-note tails))
-      (if (or (eq count :unknown) use-standard)
-          (make-return-info :kind :unknown
-                            :count count
-                            :primitive-types ptypes
-                            :types types)
-          (make-return-info :kind :fixed
-                            :count count
-                            :primitive-types ptypes
-                            :types types
-                            :locations (mapcar #'make-normal-tn ptypes))))))
+      (cond ((eq use-standard :unboxed)
+             (make-return-info :kind :unboxed
+                               :count count
+                               :primitive-types ptypes
+                               :types types
+                               :locations
+                               (let ((state (sb-vm::make-fixed-call-args-state)))
+                                 (loop for type in ptypes
+                                       collect (sb-vm::fixed-call-arg-location type state)))))
+            ((or (eq count :unknown) use-standard)
+             (make-return-info :kind :unknown
+                               :count count
+                               :primitive-types ptypes
+                               :types types))
+            (t
+             (make-return-info :kind :fixed
+                               :count count
+                               :primitive-types ptypes
+                               :types types
+                               :locations (mapcar #'make-normal-tn ptypes)))))))
 
 ;;; If TAIL-SET doesn't have any INFO, then make a RETURN-INFO for it.
 (defun assign-return-locations (fun)
@@ -216,7 +257,7 @@
          (return (lambda-return fun)))
     (when (and return
                (xep-p fun))
-      (aver (eq (return-info-kind returns) :unknown))))
+      (aver (memq (return-info-kind returns) '(:unknown :unboxed)))))
   (values))
 
 ;;; Make an IR2-NLX-INFO structure for each NLX entry point recorded.
@@ -226,8 +267,8 @@
 ;;; isn't live afterwards.
 (defun assign-ir2-nlx-info (fun)
   (declare (type clambda fun))
-  (let ((physenv (lambda-physenv fun)))
-    (dolist (nlx (physenv-nlx-info physenv))
+  (let ((env (lambda-environment fun)))
+    (dolist (nlx (environment-nlx-info env))
       (setf (nlx-info-info nlx)
             (make-ir2-nlx-info
              :home (when (member (cleanup-kind (nlx-info-cleanup nlx))
@@ -237,5 +278,14 @@
                          (make-stack-pointer-tn)))
              :save-sp (unless (eq (cleanup-kind (nlx-info-cleanup nlx))
                                   :unwind-protect)
-                        (make-nlx-sp-tn physenv))))))
+                        (make-nlx-sp-tn env))
+             :block-tn (environment-live-tn
+                        (make-normal-tn
+                         (primitive-type-or-lose
+                          (ecase (cleanup-kind (nlx-info-cleanup nlx))
+                            (:catch
+                                'catch-block)
+                            ((:unwind-protect :block :tagbody)
+                             'unwind-block))))
+                        env)))))
   (values))
