@@ -41,56 +41,47 @@
 ;;; lvars.
 (defun propagate-to-args (call fun)
   (declare (type combination call) (type clambda fun))
-  (loop with policy = (lexenv-policy (node-lexenv call))
-        for args on (basic-combination-args call)
-        for var in (lambda-vars fun)
-        for name = (lambda-var-%source-name var)
-        do (assert-lvar-type (car args) (leaf-type var) policy
-                             (if (eq (functional-kind fun) :optional)
-                                 (make-local-call-context fun name)
-                                 name))
-           (unless (leaf-refs var)
-             (flush-dest (car args))
-             (setf (car args) nil)))
+  (do ((args (basic-combination-args call) (cdr args))
+       (vars (lambda-vars fun) (cdr vars))
+       (policy (lexenv-policy (node-lexenv call))))
+      ((null args))
+    (let* ((var (car vars))
+           (name (or (and (lambda-var-arg-info var)
+                          (arg-info-key (lambda-var-arg-info var)))
+                     (lambda-var-%source-name var))))
+      (assert-lvar-type (car args) (leaf-type var) policy
+                        (if (eq (functional-kind fun) :optional)
+                            (make-local-call-context fun name)
+                            name))
+      (unless (leaf-refs var)
+        (flush-dest (car args))
+        (setf (car args) nil))))
+
   (values))
 
-;;; Given a local call CALL to FUN, find the associated argument LVARs
-;;; of CALL corresponding to declared dynamic extent LAMBDA-VARs and
-;;; note them as dynamic extent LVARs. This operation is transitive,
-;;; because dynamic extent is contagious. In particular, the arguments
-;;; of any COMBINATIONs returning a stack-allocatable object in a
-;;; dynamic extent LVAR are dynamic extent as well if the argument
-;;; LVARs contain otherwise-inaccessible stack-allocatable subobjects
-;;; themselves.
-(defun recognize-dynamic-extent-lvars (call fun)
+;;; Given a local call CALL to FUN, mark the associated args of CALL
+;;; corresponding to declared dynamic extent LAMBDA-VARs as dynamic
+;;; extent, setting up the corresponding cleanup. We do this now so
+;;; that the cleanup has the right scope. Environment analysis is
+;;; responsible for actually deciding if the arguments can be
+;;; dynamic-extent allocated, and deals with transitively marking the
+;;; otherwise-inaccessible parts of these values as dynamic extent as
+;;; well. We do this because environment analysis happens after all
+;;; major changes to the dataflow in IR1 have been done and it is
+;;; clear whether a combination can actually stack allocate it's value.
+(defun mark-dynamic-extent-args (call fun)
   (declare (type combination call) (type clambda fun))
-  ;; The block may end up being deleted due to cast optimization
-  ;; caused by USE-GOOD-FOR-DX-P
-  (unless (node-to-be-deleted-p call)
-    (let* ((*dx-lexenv* (node-lexenv call))
-           (dx-lvars
-             (loop for arg in (basic-combination-args call)
-                   for var in (lambda-vars fun)
-                   for dx = (leaf-dynamic-extent var)
-                   when (and dx arg (not (lvar-dynamic-extent arg)))
-                   append (handle-nested-dynamic-extent-lvars dx arg))))
-      (when dx-lvars
-        (let* ((entry (with-ir1-environment-from-node call
-                        (make-entry)))
-               (cleanup (make-cleanup :kind :dynamic-extent
-                                      :mess-up entry
-                                      :nlx-info dx-lvars)))
-          (setf (entry-cleanup entry) cleanup)
-          (insert-node-before call entry)
-          (setf (node-lexenv call)
-                (make-lexenv :default (node-lexenv call)
-                             :cleanup cleanup))
-          ;; Make CALL end its block, so that we have a place to
-          ;; insert cleanup code.
-          (node-ends-block call)
-          (push entry (lambda-entries (node-home-lambda entry)))
-          (dolist (cell dx-lvars)
-            (setf (lvar-dynamic-extent (cdr cell)) cleanup))))))
+  (let (cleanup)
+    (loop for arg in (basic-combination-args call)
+          for var in (lambda-vars fun)
+          do (let ((dx-kind (leaf-dynamic-extent var)))
+               (when (and arg dx-kind (not (lvar-dynamic-extent arg)))
+                 (unless cleanup
+                   (setq cleanup (insert-dynamic-extent-cleanup call)))
+                 (let ((dx-info (make-dx-info :kind dx-kind :value arg
+                                              :cleanup cleanup)))
+                   (setf (lvar-dynamic-extent arg) dx-info)
+                   (push dx-info (cleanup-nlx-info cleanup)))))))
   (values))
 
 ;;; This function handles merging the tail sets if CALL is potentially
@@ -139,7 +130,7 @@
   (propagate-to-args call fun)
   (setf (basic-combination-kind call) :local)
   (sset-adjoin fun (lambda-calls-or-closes (node-home-lambda call)))
-  (recognize-dynamic-extent-lvars call fun)
+  (mark-dynamic-extent-args call fun)
   (merge-tail-sets call fun)
   (change-ref-leaf ref fun)
   (values))
@@ -296,8 +287,7 @@
                          *lexenv*))
            (xep (ir1-convert-lambda (make-xep-lambda-expression fun)
                                     :debug-name (debug-name
-                                                 'xep (leaf-debug-name fun))
-                                    :system-lambda t)))
+                                                 'xep (leaf-debug-name fun)))))
       (setf (functional-kind xep) :external
             (leaf-ever-used xep) t
             (functional-entry-fun xep) fun
@@ -349,8 +339,7 @@
 ;;; do LET conversion here.
 (defun locall-analyze-fun-1 (fun)
   (declare (type functional fun))
-  (let ((refs (leaf-refs fun))
-        (local-p t))
+  (let ((refs (leaf-refs fun)))
     (dolist (ref refs)
       (let* ((lvar (node-lvar ref))
              (dest (when lvar (lvar-dest lvar))))
@@ -364,12 +353,9 @@
                  (when (eq (functional-kind fun) :deleted)
                    (return-from locall-analyze-fun-1))
                  (unless (eq (basic-combination-kind dest) :local)
-                   (reference-entry-point ref)
-                   (setq local-p nil)))
+                   (reference-entry-point ref)))
                 (t
-                 (reference-entry-point ref)
-                 (setq local-p nil))))))
-    (when local-p (note-local-functional fun)))
+                 (reference-entry-point ref)))))))
 
   (values))
 
@@ -430,7 +416,6 @@
            (clean-component component))))
      (unless did-something
        (return))))
-  (initial-eliminate-dead-code clambdas)
   (values))
 
 ;;; If policy is auspicious and CALL is not in an XEP and we don't seem
@@ -450,7 +435,18 @@
               (with-ir1-environment-from-node call
                 (let* ((*inline-expansions*
                          (register-inline-expansion original-functional call))
-                       (*lexenv* (functional-lexenv original-functional)))
+                       (functional-lexenv (functional-lexenv original-functional))
+                       (call-policy (lexenv-policy (node-lexenv call)))
+                       ;; The inline expansion should be converted
+                       ;; with the policy in effect at this call site,
+                       ;; not the policy saved in the original
+                       ;; functional.
+                       (*lexenv*
+                         (if (eq (lexenv-policy functional-lexenv)
+                                  call-policy)
+                             functional-lexenv
+                             (make-lexenv :default functional-lexenv
+                                          :policy call-policy))))
                   (values nil
                           (ir1-convert-lambda
                            (functional-inline-expansion original-functional)
@@ -685,20 +681,18 @@
 ;;; function that rearranges the arguments and calls the entry point.
 ;;; We analyze the new function and the entry point immediately so
 ;;; that everything gets converted during the single pass.
-(defun convert-hairy-fun-entry (ref call entry vars ignores args indef)
+(defun convert-hairy-fun-entry (ref call entry vars ignores args)
   (declare (list vars ignores args) (type ref ref) (type combination call)
            (type clambda entry))
   (let ((new-fun
          (with-ir1-environment-from-node call
            (ir1-convert-lambda
             `(lambda ,vars
-               (declare (ignorable ,@ignores)
-                        (indefinite-extent ,@indef))
+               (declare (ignorable ,@ignores))
                (%funcall ,entry ,@args))
             :debug-name (debug-name 'hairy-function-entry
                                     (lvar-fun-debug-name
-                                     (basic-combination-fun call)))
-            :system-lambda t))))
+                                     (basic-combination-fun call)))))))
     (convert-call ref call new-fun)
     (dolist (ref (leaf-refs entry))
       (convert-call-if-possible ref (lvar-dest (node-lvar ref))))))
@@ -857,9 +851,7 @@
 
         (convert-hairy-fun-entry ref call (optional-dispatch-main-entry fun)
                                  (append temps more-temps)
-                                 (ignores) (call-args)
-                                 (when (optional-rest-p fun)
-                                   more-temps)))))
+                                 (ignores) (call-args)))))
 
   (values))
 
@@ -908,58 +900,11 @@
         (block-succ call-block)
       (unlink-blocks call-block next-block)
       (link-blocks call-block bind-block)
+      (unless (eq (car (ctran-source-path (node-prev call))) 'original-source-path)
+        (setf (ctran-source-path (block-start bind-block))
+              (ctran-source-path (node-prev call))))
       next-block)))
 
-;;; Remove CLAMBDA from the tail set of anything it used to be in the
-;;; same set as; but leave CLAMBDA with a valid tail set value of
-;;; its own, for the benefit of code which might try to pull
-;;; something out of it (e.g. return type).
-(defun depart-from-tail-set (clambda)
-  ;; Until sbcl-0.pre7.37.flaky5.2, we did
-  ;;   (LET ((TAILS (LAMBDA-TAIL-SET CLAMBDA)))
-  ;;     (SETF (TAIL-SET-FUNS TAILS)
-  ;;           (DELETE CLAMBDA (TAIL-SET-FUNS TAILS))))
-  ;;   (SETF (LAMBDA-TAIL-SET CLAMBDA) NIL)
-  ;; here. Apparently the idea behind the (SETF .. NIL) was that since
-  ;; TAIL-SET-FUNS no longer thinks we're in the tail set, it's
-  ;; inconsistent, and perhaps unsafe, for us to think we're in the
-  ;; tail set. Unfortunately..
-  ;;
-  ;; The (SETF .. NIL) caused problems in sbcl-0.pre7.37.flaky5.2 when
-  ;; I was trying to get Python to emit :EXTERNAL LAMBDAs directly
-  ;; (instead of only being able to emit funny little :TOPLEVEL stubs
-  ;; which you called in order to get the address of an external LAMBDA):
-  ;; the external function was defined in terms of internal function,
-  ;; which was LET-converted, and then things blew up downstream when
-  ;; FINALIZE-XEP-DEFINITION tried to find out its DEFINED-TYPE from
-  ;; the now-NILed-out TAIL-SET. So..
-  ;;
-  ;; To deal with this problem, we no longer NIL out
-  ;; (LAMBDA-TAIL-SET CLAMBDA) here. Instead:
-  ;;   * If we're the only function in TAIL-SET-FUNS, it should
-  ;;     be safe to leave ourself linked to it, and it to you.
-  ;;   * If there are other functions in TAIL-SET-FUNS, then we're
-  ;;     afraid of future optimizations on those functions causing
-  ;;     the TAIL-SET object no longer to be valid to describe our
-  ;;     return value. Thus, we delete ourselves from that object;
-  ;;     but we save a newly-allocated tail-set, derived from the old
-  ;;     one, for ourselves, for the use of later code (e.g.
-  ;;     FINALIZE-XEP-DEFINITION) which might want to
-  ;;     know about our return type.
-  (let* ((old-tail-set (lambda-tail-set clambda))
-         (old-tail-set-funs (tail-set-funs old-tail-set)))
-    (unless (= 1 (length old-tail-set-funs))
-      (setf (tail-set-funs old-tail-set)
-            (delete clambda old-tail-set-funs))
-      (let ((new-tail-set (copy-tail-set old-tail-set)))
-        (setf (lambda-tail-set clambda) new-tail-set
-              (tail-set-funs new-tail-set) (list clambda)))))
-  ;; The documentation on TAIL-SET-INFO doesn't tell whether it could
-  ;; remain valid in this case, so we nuke it on the theory that
-  ;; missing information tends to be less dangerous than incorrect
-  ;; information.
-  (setf (tail-set-info (lambda-tail-set clambda)) nil))
-
 ;;; Handle the environment semantics of LET conversion. We add CLAMBDA
 ;;; and its LETs to LETs for the CALL's home function. We merge the
 ;;; calls for CLAMBDA with the calls for the home function, removing
@@ -978,9 +923,9 @@
           (delete clambda (component-lambdas component)))
     (setf (component-reanalyze component) t))
   (setf (lambda-call-lexenv clambda) (node-lexenv call))
-
-  (depart-from-tail-set clambda)
-
+  (let ((tails (lambda-tail-set clambda)))
+    (setf (tail-set-funs tails)
+          (delete clambda (tail-set-funs tails))))
   (let* ((home (node-home-lambda call))
          (home-env (lambda-environment home))
          (env (lambda-environment clambda)))
@@ -1191,28 +1136,24 @@
   (reoptimize-lvar (node-lvar call))
   (values))
 
-;;; Are there any declarations in force to say CLAMBDA shouldn't be
-;;; LET converted?
-(defun declarations-suppress-let-conversion-p (clambda)
-  ;; From the user's point of view, LET-converting something that
-  ;; has a name is inlining it. (The user can't see what we're doing
-  ;; with anonymous things, and suppressing inlining
-  ;; for such things can easily give Python acute indigestion, so
-  ;; we don't.)
-  ;;
-  ;; A functional that is already inline-expanded in this componsne definitely
-  ;; deserves let-conversion -- and in case of main entry points for inline
-  ;; expanded optional dispatch, the main-etry isn't explicitly marked INLINE
-  ;; even if the function really is.
-  (when (and (leaf-has-source-name-p clambda)
-             (not (functional-inline-expanded clambda)))
-    ;; ANSI requires that explicit NOTINLINE be respected.
-    (or (eq (lambda-inlinep clambda) 'notinline)
-        ;; If (= LET-CONVERSION 0) or (= DEBUG 3) we can guess that inlining
-        ;; generally won't be appreciated, but if the user specifically requests
-        ;; inlining, that takes precedence over our general guess.
-        (and (or (policy clambda (or (= let-conversion 0) (= debug 3))))
-             (not (eq (lambda-inlinep clambda) 'inline))))))
+;;; Are there any declarations in force to say FUN shouldn't be LET
+;;; converted?
+(defun declarations-suppress-let-conversion-p (fun)
+  (declare (type clambda fun))
+  ;; From the user's point of view, LET-converting something that has
+  ;; a name is inlining it. (The user can't see what we're doing with
+  ;; anonymous things, and suppressing inlining for such things can
+  ;; easily give Python acute indigestion, so we don't.)
+  (and (leaf-has-source-name-p fun)
+       ;; If FUN is/was an entry point for an OPTIONAL-DISPATCH, then
+       ;; INLINEP information is only recorded on the optional dispatch.
+       (case (functional-inlinep (or (lambda-optional-dispatch fun) fun))
+         ;; If the user specifically requests inlining, that takes
+         ;; precedence over our general guess.
+         (inline nil)
+         ;; ANSI requires that explicit NOTINLINE be respected.
+         (notinline t)
+         (t (policy fun (or (= let-conversion 0) (= debug 3)))))))
 
 ;;; We also don't convert calls to named functions which appear in the
 ;;; initial component, delaying this until optimization. This
@@ -1253,6 +1194,7 @@
                          (leaf-type var))
               (let ((use-component (node-component use)))
                 (propagate-lvar-annotations-to-refs arg var)
+                (propagate-ref-dx use arg var)
                 (update-lvar-dependencies leaf arg)
                 (substitute-leaf-if
                  (lambda (ref)
@@ -1287,8 +1229,7 @@
 (defun maybe-let-convert (clambda &optional component)
   (declare (type clambda clambda)
            (type (or null component) component))
-  (unless (or (declarations-suppress-let-conversion-p clambda)
-              (functional-has-external-references-p clambda))
+  (unless (declarations-suppress-let-conversion-p clambda)
     ;; We only convert to a LET when the function is a normal local
     ;; function, has no XEP, and is referenced in exactly one local
     ;; call. Conversion is also inhibited if the only reference is in
@@ -1346,7 +1287,7 @@
   (declare (type cblock block1 block2))
   (or (eq block1 block2)
       (let ((cleanup2 (block-start-cleanup block2)))
-        (do-nested-cleanups (cleanup (block-end-lexenv block1) t)
+        (do-nested-cleanups (cleanup block1 t)
           (when (eq cleanup cleanup2)
             (return t))
           (case (cleanup-kind cleanup)
@@ -1416,7 +1357,6 @@
   (declare (type clambda fun))
   (when (and (not (functional-kind fun))
              (not (functional-entry-fun fun))
-             (not (functional-has-external-references-p fun))
              ;; If a functional is explicitly inlined, we don't want
              ;; to assignment convert it, as more call-site
              ;; specialization can be done with inlining.
@@ -1424,6 +1364,7 @@
              (not (block-delete-p (lambda-block fun))))
     (let ((outside-calls nil)
           (outside-calls-ctran nil)
+          (outside-calls-lvar nil)
           (outside-calls-env nil)
           (outside-calls-cleanup nil))
       (when (and (dolist (ref (leaf-refs fun) t)
@@ -1439,6 +1380,9 @@
                            (let ((dest-ctran
                                    (or (node-next dest)
                                        (block-start (first (block-succ (node-block dest))))))
+                                 (dest-lvar
+                                   (and (valued-node-p dest)
+                                        (node-lvar dest)))
                                  (dest-env
                                    (node-home-lambda dest))
                                  (dest-cleanup
@@ -1459,13 +1403,15 @@
                                     ;; cleanups with respect to the
                                     ;; messiest common ancestor.
                                     (unless (and (or (eq (node-derived-type dest) *empty-type*)
-                                                     (and (eq outside-calls-ctran dest-ctran)))
+                                                     (and (eq outside-calls-ctran dest-ctran)
+                                                          (eq outside-calls-lvar dest-lvar)))
                                                  (eq outside-calls-env dest-env)
                                                  (eq outside-calls-cleanup dest-cleanup))
                                       (return nil)))
                                    (t
                                     (setq outside-calls-env dest-env)
                                     (setq outside-calls-ctran dest-ctran)
+                                    (setq outside-calls-lvar dest-lvar)
                                     (setq outside-calls-cleanup dest-cleanup)))
                              (push dest outside-calls))))))
                  (ok-initial-convert-p fun))
