@@ -260,7 +260,7 @@
   ;; This is only meaningful in :ARGUMENT and :TEMPORARY operands.
   (target nil :type (or symbol null) :read-only t)
   ;; TEMP is a temporary that holds the TN-REF for this operand.
-  (temp (make-operand-parse-temp) :type symbol :read-only t)
+  (temp (make-operand-parse-temp) :type symbol)
   ;; the time that this operand is first live and the time at which it
   ;; becomes dead again. These are TIME-SPECs, as returned by
   ;; PARSE-TIME-SPEC.
@@ -343,8 +343,6 @@
   ;; info about how to emit MOVE-ARG VOPs for the &MORE operand in
   ;; call/return VOPs
   (move-args nil :type (member nil :local-call :full-call :known-return :fixed))
-  (args-var '.args. :type symbol)
-  (results-var '.results. :type symbol)
   (before-load :unspecified :type (or (member :unspecified) list)))
 (declaim (freeze-type vop-parse))
 (defprinter (vop-parse)
@@ -373,9 +371,9 @@
 ;;; The list of slots in the structure, not including the OPERANDS slot.
 ;;; Order here is insignificant; it happens to be alphabetical.
 (defglobal vop-parse-slot-names
-    '(arg-types args args-var before-load body conditional-p cost guard ignores info-args inherits
+    '(arg-types args before-load body conditional-p cost guard ignores info-args inherits
       ltn-policy more-args more-results move-args name node-var note optional-results result-types
-      results results-var save-p source-location temps translate variant variant-vars vop-var))
+      results save-p source-location temps translate variant variant-vars vop-var))
 ;; A sanity-check. Of course if this fails, the likelihood is that you can't even
 ;; get this far in cross-compilaion. So it's probably not worth much.
 (eval-when (#+sb-xc :compile-toplevel)
@@ -727,8 +725,6 @@
 (defun make-generator-function (parse)
   (declare (type vop-parse parse))
   (let ((n-vop (vop-parse-vop-var parse))
-        (n-args (vop-parse-args-var parse))
-        (n-results (vop-parse-results-var parse))
         (operands (vop-parse-operands parse))
         (n-info (gensym)) (n-variant (gensym))
         (dummy (gensym)))
@@ -754,14 +750,13 @@
           ((:more-argument :more-result))))
 
       `(named-lambda (vop ,(vop-parse-name parse)) (,n-vop)
-         (let* ((,n-args (vop-args ,n-vop))
-                (,n-results (vop-results ,n-vop))
-                ,@(access-operands (vop-parse-args parse)
+         (declare (ignorable ,n-vop))
+         (let* (,@(access-operands (vop-parse-args parse)
                                    (vop-parse-more-args parse)
-                                   n-args)
+                                   `(vop-args ,n-vop))
                 ,@(access-operands (vop-parse-results parse)
                                    (vop-parse-more-results parse)
-                                   n-results)
+                                   `(vop-results ,n-vop))
                 ,@(access-operands (vop-parse-temps parse) nil
                                    `(vop-temps ,n-vop))
                 ,@(when (vop-parse-info-args parse)
@@ -780,8 +775,7 @@
                 ,@(binds))
            (declare (ignore ,@(vop-parse-ignores parse)
                             ,@(and (neq (vop-parse-before-load parse) :unspecified)
-                                   `(,dummy)))
-                    (ignorable ,n-args ,n-results))
+                                   `(,dummy))))
            ,@(loads)
            ;; RETURN-FROM can exit the ASSEMBLE while continuing on with saves.
            (block ,(vop-parse-name parse)
@@ -792,24 +786,18 @@
 (defun make-after-sc-function (parse)
   (let ((unused-temps
           (remove-if-not #'operand-parse-unused-if
-                         (vop-parse-temps parse)))
-        (conditional (typep (vop-parse-conditional-p parse) '(cons (eql :after-sc-selection)))))
-    (when (or unused-temps
-              conditional)
+                         (vop-parse-temps parse))))
+    (when unused-temps
       (let* ((n-vop (vop-parse-vop-var parse))
-             (n-args (vop-parse-args-var parse))
-             (n-results (vop-parse-results-var parse))
              (n-info (gensym))
              (n-variant (gensym))
              (bindings
-               `((,n-args (vop-args ,n-vop))
-                 (,n-results (vop-results ,n-vop))
-                 ,@(access-operands (vop-parse-args parse)
+               `(,@(access-operands (vop-parse-args parse)
                                     (vop-parse-more-args parse)
-                                    n-args)
+                                    `(vop-args ,n-vop))
                  ,@(access-operands (vop-parse-results parse)
                                     (vop-parse-more-results parse)
-                                    n-results)
+                                    `(vop-results ,n-vop))
                  ,@(and unused-temps
                         (access-operands (vop-parse-temps parse) nil
                                          `(vop-temps ,n-vop)))
@@ -836,11 +824,6 @@
         `(lambda (,n-vop)
            (let* ,bindings
              (declare (ignorable ,@(mapcar #'car bindings)))
-             ,@(when conditional
-                 `((setf (car (last (vop-codegen-info ,n-vop)))
-                         (progn ,@(cdr (vop-parse-conditional-p parse))))
-                   (setf (vop-codegen-info ,n-vop)
-                         (butlast (vop-codegen-info ,n-vop)))))
              ,@(loop for op in unused-temps
                      collect `(when ,(operand-parse-unused-if op)
                                 (setf (tn-kind ,(operand-parse-name op)) :unused)))))))))
@@ -1026,19 +1009,27 @@
 
 ;;; the top level parse function: clobber PARSE to represent the
 ;;; specified options.
-(defun parse-define-vop (parse specs)
+(defun parse-define-vop (parse specs inherits)
   (declare (type vop-parse parse) (list specs))
-  (let ((*parse-vop-operand-count* (compute-parse-vop-operand-count parse)))
+  (let ((*parse-vop-operand-count* (compute-parse-vop-operand-count parse))
+        args-p
+        results-p
+        arg-refs
+        arg-refs-p
+        result-refs
+        result-refs-p)
     (dolist (spec specs)
       (unless (consp spec)
         (error "malformed option specification: ~S" spec))
       (case (first spec)
         (:args
+         (setf args-p t)
          (multiple-value-bind (fixed more)
              (parse-vop-operands parse (rest spec) :argument)
            (setf (vop-parse-args parse) fixed)
            (setf (vop-parse-more-args parse) more)))
         (:results
+         (setf results-p t)
          (multiple-value-bind (fixed more)
              (parse-vop-operands parse (rest spec) :result)
            (setf (vop-parse-results parse) fixed)
@@ -1052,8 +1043,8 @@
         (:temporary
          (parse-temporary spec parse))
         (:generator
-            (setf (vop-parse-cost parse)
-                  (vop-spec-arg spec 'unsigned-byte 1 nil))
+         (setf (vop-parse-cost parse)
+               (vop-spec-arg spec 'unsigned-byte 1 nil))
          (setf (vop-parse-body parse) (cddr spec)))
         (:before-load
          (setf (vop-parse-before-load parse) (cdr spec)))
@@ -1074,10 +1065,12 @@
          (setf (vop-parse-cost parse) (vop-spec-arg spec 'unsigned-byte)))
         (:vop-var
          (setf (vop-parse-vop-var parse) (vop-spec-arg spec 'symbol)))
-        (:args-var
-         (setf (vop-parse-args-var parse) (vop-spec-arg spec 'symbol)))
-        (:results-var
-         (setf (vop-parse-results-var parse) (vop-spec-arg spec 'symbol)))
+        (:arg-refs
+         (setf arg-refs-p t
+               arg-refs (cdr spec)))
+        (:result-refs
+         (setf result-refs-p t
+               result-refs (cdr spec)))
         (:move-args
          (setf (vop-parse-move-args parse)
                (vop-spec-arg spec '(member nil :local-call :full-call
@@ -1102,8 +1095,8 @@
         ;; architecture, since the renaming would be a change to the
         ;; interface between
         (:policy
-         (setf (vop-parse-ltn-policy parse)
-               (vop-spec-arg spec 'ltn-policy)))
+            (setf (vop-parse-ltn-policy parse)
+                  (vop-spec-arg spec 'ltn-policy)))
         (:save-p
          (setf (vop-parse-save-p parse)
                (vop-spec-arg spec
@@ -1114,6 +1107,36 @@
                        (rest spec))))
         (t
          (error "unknown option specifier: ~S" (first spec)))))
+    (cond (arg-refs-p
+           (loop with refs = arg-refs
+                 for arg in (if args-p
+                                (vop-parse-args parse)
+                                (setf (vop-parse-args parse)
+                                      (mapcar #'copy-structure (vop-parse-args parse))))
+                 for ref = (pop refs)
+                 when ref
+                 do (setf (operand-parse-temp arg) ref)))
+          ((and inherits
+                args-p)
+           (loop for inherited-arg in (vop-parse-args inherits)
+                 for arg in (vop-parse-args parse)
+                 do (setf (operand-parse-temp arg)
+                          (operand-parse-temp inherited-arg)))))
+    (cond (result-refs-p
+           (loop with refs = result-refs
+                 for result in (if results-p
+                                (vop-parse-results parse)
+                                (setf (vop-parse-results parse)
+                                      (mapcar #'copy-structure (vop-parse-results parse))))
+                 for ref = (pop refs)
+                 when ref
+                 do (setf (operand-parse-temp result) ref)))
+          ((and inherits
+                results-p)
+           (loop for inherited-result in (vop-parse-results inherits)
+                 for result in (vop-parse-results parse)
+                 do (setf (operand-parse-temp result)
+                          (operand-parse-temp inherited-result)))))
     (values)))
 
 ;;;; making costs and restrictions
@@ -1419,8 +1442,6 @@
       :more-args-type ,(when more-args (make-operand-type more-arg))
       :result-types ,(cond ((eq conditional t)
                             :conditional)
-                           ((eql (car conditional) :after-sc-selection)
-                            ''(:conditional :after-sc-selection))
                            (conditional
                             `'(:conditional . ,conditional))
                            (t
@@ -1671,7 +1692,7 @@
       (setf (vop-parse-name parse) name))
     (setf (vop-parse-inherits parse) inherits)
 
-    (parse-define-vop parse specs)
+    (parse-define-vop parse specs inherited-parse)
     (set-vop-parse-operands parse)
     (check-operand-types parse
                          (vop-parse-args parse)
@@ -1736,51 +1757,78 @@
             (temps)
             (results)
             (result-types))
-    (loop for (var arg) in vars
-          for (name this-sc) = var
-          for (nil sc type . rest) = (if this-sc
-                                         var
-                                         prev)
-          for prev = (if this-sc
-                         var
-                         prev)
-          do (cond ((eq name :info)
-                    (infos this-sc)
-                    (input arg))
-                   (arg
-                    (input arg)
-                    (args (list* name :scs (list sc) rest))
-                    (arg-types (or type '*)))
-                   (t
-                    (temps `(:temporary (:sc ,sc ,@rest)
-                                        ,name)))))
-    (loop for result in results
-          for (name this-sc) = result
-          for (nil sc type . rest) = (if this-sc
-                                         result
-                                         prev)
-          for prev = (if this-sc
-                         result
-                         prev)
-          do (results (list* name :scs (list sc) rest))
-             (result-types (or type '*)))
-    `(inline-%primitive
-      ,(eval (%define-vop nil nil
-                          (delete nil
-                                  (list* (and (args)
-                                              (list* :args (args)))
-                                         (and (arg-types)
-                                              (list* :arg-types (arg-types)))
-                                         (and (results)
-                                              (list* :results (results)))
-                                         (and (result-types)
-                                              (list* :result-types (result-types)))
-                                         (and (infos)
-                                              (list* :info (infos)))
-                                         (list* :generator 0 body)
-                                         (temps)))
-                          nil))
-      ,@(input))))
+    (flet ((sc-to-primtype (sc)
+             (case sc
+               (sb-vm::any-reg
+                'fixnum)
+               (sb-vm::unsigned-reg
+                'sb-vm::unsigned-num)
+               (sb-vm::signed-reg
+                'sb-vm::signed-num)
+               (sb-vm::sap-reg
+                'system-area-pointer)
+               (sb-vm::descriptor-reg
+                t)
+               (t
+                '*)))
+           (primtype-to-type (type)
+             (case type
+               (sb-vm::unsigned-num
+                'word)
+               (sb-vm::signed-num
+                'sb-vm:signed-word)
+               (sb-vm::tagged-num
+                'fixnum)
+               (* t)
+               (t (primitive-type-specifier (primitive-type-or-lose type))))))
+      (loop for (var arg) in vars
+            for (name this-sc) = var
+            for (nil sc type . rest) = (if this-sc
+                                           var
+                                           prev)
+            for prev = (if this-sc
+                           var
+                           prev)
+            do (cond ((eq name :info)
+                      (infos this-sc)
+                      (input arg))
+                     (arg
+                      (args (list* name :scs (list sc) rest))
+                      (let ((type (or type (sc-to-primtype sc))))
+                        (arg-types type)
+                        (input `(the ,(primtype-to-type type) ,arg))))
+                     (t
+                      (temps `(:temporary (:sc ,sc ,@rest)
+                                          ,name)))))
+      (loop for result in results
+            for (name this-sc) = result
+            for (nil sc type . rest) = (if this-sc
+                                           result
+                                           prev)
+            for prev = (if this-sc
+                           result
+                           prev)
+            do (results (list* name :scs (list sc) rest))
+               (result-types (or type (sc-to-primtype sc))))
+      `(truly-the
+        (values ,@(mapcar #'primtype-to-type (result-types)) &optional)
+        (inline-%primitive
+         ,(eval (%define-vop nil nil
+                             (delete nil
+                                     (list* (and (args)
+                                                 (list* :args (args)))
+                                            (and (arg-types)
+                                                 (list* :arg-types (arg-types)))
+                                            (and (results)
+                                                 (list* :results (results)))
+                                            (and (result-types)
+                                                 (list* :result-types (result-types)))
+                                            (and (infos)
+                                                 (list* :info (infos)))
+                                            (list* :generator 0 body)
+                                            (temps)))
+                             nil))
+         ,@(input))))))
 
 (macrolet
     ((def ()
