@@ -175,15 +175,7 @@
                                        ((nil) 'unsigned-byte)
                                        ((t) 'signed-byte))
                                      ,width)))))
-    (labels ((reoptimize-node (node name)
-               (setf (node-derived-type node)
-                     (fun-type-returns
-                      (global-ftype name)))
-               (setf (lvar-%derived-type (node-lvar node)) nil)
-               (setf (node-reoptimize node) t)
-               (setf (block-reoptimize (node-block node)) t)
-               (reoptimize-component (node-component node) :maybe))
-             (insert-lvar-cut (lvar)
+    (labels ((insert-lvar-cut (lvar)
                "Insert a LOGAND/MASK-SIGNED-FIELD to cut the value of LVAR
                 to the required bit width. Returns T if any change was made.
 
@@ -247,7 +239,7 @@
                               (let ((lvar (node-lvar node)))
                                 (setf (lvar-%derived-type lvar)
                                       (and (lvar-has-single-use-p lvar)
-                                           (make-values-type :required (list (ctype-of new-value))))))
+                                           (make-values-type (list (ctype-of new-value))))))
                               (setf (block-reoptimize (node-block node)) t)
                               (reoptimize-component (node-component node) :maybe)
                               (values t t)))))))
@@ -300,7 +292,13 @@
                                      (setf did-something (or did-something change)
                                            over-wide (or over-wide wide)))))
                                (when did-something
-                                 (reoptimize-node node name))
+                                 ;; Can't rely on REOPTIMIZE-NODE, as it may neve get reoptimized.
+                                 ;; But the outer functions don't want the type to get
+                                 ;; widened and their VOPs may never be applied.
+                                 (setf (node-derived-type node)
+                                       (fun-type-returns (global-ftype name)))
+                                 (setf (lvar-%derived-type (node-lvar node)) nil)
+                                 (ir1-optimize-combination node))
                                (values t did-something over-wide)))))))))
              (cut-lvar (lvar &key head
                         &aux did-something must-insert over-wide)
@@ -354,6 +352,7 @@
              (cond
                ((eq signedp (cdr w)) (<= width (car w)))
                ((eq signedp nil) (< width (car w))))))
+      (declare (truly-dynamic-extent #'inexact-match))
       (let ((tgt (find-if #'inexact-match twidths)))
         (when tgt
           (return-from best-modular-version
@@ -394,6 +393,8 @@
                                         ; "don't reoptimize this (LOGAND) node any more".
               )))))))
 
+(setf (fun-info-optimizer (fun-info-or-lose 'logandc2)) #'logand-optimizer-optimizer)
+
 (defoptimizer (mask-signed-field optimizer) ((width x) node)
   (let ((result-type (single-value-type (node-derived-type node))))
     (multiple-value-bind (low high)
@@ -469,13 +470,12 @@
                 (when (<= width ,width)
                   (cond ((or (and (constant-lvar-p count)
                                   (plusp (lvar-value count)))
-                             (csubtypep count-type
-                                        (specifier-type '(and unsigned-byte fixnum))))
+                             (csubtypep count-type (specifier-type 'word)))
                          (cut-to-width integer ,kind width ,signedp)
                          ',left-name)
                         #+(or arm64 x86-64)
                         ((and (not (constant-lvar-p count))
-                              (csubtypep count-type (specifier-type 'fixnum))
+                              (csubtypep count-type (specifier-type 'sb-vm:signed-word))
                               ;; Unknown sign
                               (not (csubtypep count-type (specifier-type '(integer * 0))))
                               (not (csubtypep count-type (specifier-type '(integer 0 *))))
