@@ -48,7 +48,7 @@
   (:args (symbol :scs (descriptor-reg)))
   (:results (result :scs (unsigned-reg)))
   (:result-types positive-fixnum)
-  (:translate sb-impl::symbol-package-id)
+  (:translate symbol-package-id)
   (:policy :fast-safe)
   (:generator 5
    (inst cmpld symbol null-tn)
@@ -80,7 +80,7 @@
   (:vop-var vop)
   (:ignore name)
   (:generator 1
-    (emit-gc-store-barrier object nil (list t1) (vop-nth-arg 1 vop) value)
+    (emit-gengc-barrier object nil (list t1) (vop-nth-arg 1 vop) value)
     (storew value object offset lowtag)))
 
 (define-vop (compare-and-swap-slot)
@@ -93,7 +93,7 @@
   (:results (result :scs (descriptor-reg) :from :load))
   (:vop-var vop)
   (:generator 5
-    (emit-gc-store-barrier object nil (list temp) (vop-nth-arg 2 vop) new)
+    (emit-gengc-barrier object nil (list temp) (vop-nth-arg 2 vop) new)
     (inst sync)
     (inst li temp (- (* offset n-word-bytes) lowtag))
     LOOP
@@ -118,7 +118,7 @@
   (:policy :fast-safe)
   (:vop-var vop)
   (:generator 15
-    (emit-gc-store-barrier symbol nil (list temp) (vop-nth-arg 2 vop) new)
+    (emit-gengc-barrier symbol nil (list temp) (vop-nth-arg 2 vop) new)
     (inst sync)
     (load-tls-index temp symbol)
     ;; Thread-local area, no synchronization needed.
@@ -202,7 +202,7 @@
     (inst stdx value thread-base-tn tls-slot)
     (inst b DONE)
     GLOBAL-VALUE
-    (emit-gc-store-barrier symbol nil (list tls-slot) (vop-nth-arg 1 vop) value)
+    (emit-gengc-barrier symbol nil (list tls-slot) (vop-nth-arg 1 vop) value)
     (storew value symbol symbol-value-slot other-pointer-lowtag)
     DONE))
 
@@ -279,7 +279,7 @@
   (:args (symbol :scs (descriptor-reg)))
   (:results (res :scs (any-reg)))
   (:result-types positive-fixnum)
-  (:args-var args)
+  (:arg-refs args)
   (:generator 4
     (when (not-nil-tn-ref-p args)
       (loadw res symbol symbol-hash-slot other-pointer-lowtag)
@@ -294,8 +294,6 @@
 
 ;;;; Fdefinition (fdefn) objects.
 
-(define-vop (fdefn-fun cell-ref) ; does not translate anything
-  (:variant fdefn-fun-slot other-pointer-lowtag))
 (define-vop (untagged-fdefn-fun cell-ref) ; does not translate anything
   (:variant fdefn-fun-slot 0))
 
@@ -348,7 +346,7 @@
   (:temporary (:scs (non-descriptor-reg)) type)
   (:results (result :scs (descriptor-reg)))
   (:generator 38
-    (emit-gc-store-barrier fdefn nil (list type))
+    (emit-gengc-barrier fdefn nil (list type))
     (let ((normal-fn (gen-label)))
       (load-type type function (- fun-pointer-lowtag))
       (inst cmpdi type simple-fun-widetag)
@@ -456,10 +454,11 @@
 (define-vop (closure-init)
   (:args (object :scs (descriptor-reg))
          (value :scs (descriptor-reg any-reg)))
-  (:info offset)
+  (:info offset dx)
   (:temporary (:scs (non-descriptor-reg)) temp)
   (:generator 4
-    (emit-gc-store-barrier object nil (list temp))
+    (unless dx
+      (emit-gengc-barrier object nil (list temp)))
     (storew value object (+ closure-info-offset offset) fun-pointer-lowtag)))
 
 (define-vop (closure-init-from-fp)
@@ -470,9 +469,6 @@
 
 ;;;; Value Cell hackery.
 
-(define-vop (value-cell-ref cell-ref)
-  (:variant value-cell-value-slot other-pointer-lowtag))
-
 (define-vop (value-cell-set cell-set)
   (:variant value-cell-value-slot other-pointer-lowtag))
 
@@ -540,7 +536,7 @@
 #-sb-xc-host
 (defun code-header-ref (code index)
   (declare (index index))
-  (binding* (((start count) (sb-vm::code-header-fdefn-range code))
+  (binding* (((start count) (code-header-fdefn-range code))
              (end (+ start count)))
     (values (if (and (>= index start) (< index end))
                 (%primitive code-header-ref+tag code index other-pointer-lowtag)
@@ -560,7 +556,7 @@
     (inst ld temp thread-base-tn (ash thread-card-table-slot word-shift))
     (pseudo-atomic (pa-flag)
       ;; Compute card mark index
-      (inst rldicl card object (- 64 gencgc-card-shift) (make-fixup nil :gc-barrier))
+      (inst rldicl card object (- 64 gencgc-card-shift) (make-fixup nil :card-table-index-mask))
       ;; Touch the card mark byte.
       (inst stbx thread-base-tn temp card) ; THREAD-TN's low byte is 0
       ;; set 'written' flag in the code header
