@@ -19,24 +19,29 @@
         (inst bne temp zero-tn target)
         (inst beq temp zero-tn target))))
 
-(defun %test-fixnum-and-headers (value temp target not-p headers &key value-tn-ref)
+(defun %test-fixnum-and-headers (value temp target not-p headers
+                                 &key value-tn-ref
+                                      immediate-tested)
   (let ((drop-through (gen-label)))
     (assemble ()
       (inst andi temp value fixnum-tag-mask)
       (inst beq temp zero-tn (if not-p drop-through target)))
     (%test-headers value temp target not-p nil headers
                    :drop-through drop-through
-                   :value-tn-ref value-tn-ref)))
+                   :value-tn-ref value-tn-ref
+                   :immediate-tested immediate-tested)))
 
 #+64-bit
 (defun %test-fixnum-immediate-and-headers (value temp target not-p immediate
-                                           headers &key value-tn-ref)
+                                           headers &key value-tn-ref
+                                                        immediate-tested)
   (let ((drop-through (gen-label)))
     (inst andi temp value fixnum-tag-mask)
     (inst beq temp zero-tn (if not-p drop-through target))
     (%test-immediate-and-headers value temp target not-p immediate headers
                                  :drop-through drop-through
-                                 :value-tn-ref value-tn-ref)))
+                                 :value-tn-ref value-tn-ref
+                                 :immediate-tested immediate-tested)))
 
 
 (defun %test-immediate (value temp target not-p immediate)
@@ -56,7 +61,7 @@
         (inst beq temp zero-tn target))))
 
 (defun %test-headers (value temp target not-p function-p headers
-                      &key (drop-through (gen-label)) value-tn-ref)
+                      &key (drop-through (gen-label)) value-tn-ref immediate-tested)
   (let ((lowtag (if function-p fun-pointer-lowtag other-pointer-lowtag)))
     (multiple-value-bind
         (when-true when-false)
@@ -68,7 +73,7 @@
       (assemble ()
         (unless (and value-tn-ref
                      (eq lowtag other-pointer-lowtag)
-                     (other-pointer-tn-ref-p value-tn-ref))
+                     (other-pointer-tn-ref-p value-tn-ref nil immediate-tested))
           (%test-lowtag value temp when-false t lowtag))
         (load-type temp value (- lowtag))
         (let ((delta 0))
@@ -103,13 +108,15 @@
 
 (defun %test-immediate-and-headers (value temp target not-p immediate headers
                                     &key (drop-through (gen-label))
-                                         value-tn-ref)
+                                         value-tn-ref
+                                         immediate-tested)
   (inst andi temp value widetag-mask)
   (inst xori temp temp immediate)
   (inst beq temp zero-tn (if not-p drop-through target))
   (%test-headers value temp target not-p nil headers
                  :drop-through drop-through
-                 :value-tn-ref value-tn-ref))
+                 :value-tn-ref value-tn-ref
+                 :immediate-tested immediate-tested))
 
 
 ;;;; Other integer ranges.
