@@ -170,6 +170,7 @@
                                                ,(car immediates)
                                                ',(canonicalize-widetags
                                                   headers)
+                                               :immediate-tested '(fixnum ,(car immediates))
                                                ,@other-args))
          (immediates
           (if (= n-word-bits 64)
@@ -180,6 +181,7 @@
          (headers
           `(%test-fixnum-and-headers ,value ,temp ,target ,not-p
                                      ',(canonicalize-widetags headers)
+                                     :immediate-tested '(fixnum)
                                      ,@other-args))
          (t
           `(%test-fixnum ,value ,temp ,target ,not-p
@@ -191,6 +193,7 @@
               `(%test-immediate-and-headers ,value ,temp ,target ,not-p
                                             ,(car immediates)
                                             ',(canonicalize-widetags headers)
+                                            :immediate-tested '(,(car immediates))
                                             ,@other-args)
               (error "can't mix testing of immediates with testing of headers")))
          (lowtags
@@ -224,7 +227,7 @@
   (:temporary (:sc unsigned-reg #+x86 :offset #+x86 eax-offset) temp)
   (:conditional)
   (:info target not-p)
-  (:args-var args)
+  (:arg-refs args)
   (:policy :fast-safe))
 ;; A vop that accepts a computed set of widetags.
 (define-vop (%other-pointer-subtype-p type-predicate)
@@ -243,14 +246,14 @@
   (:temporary (:sc non-descriptor-reg) temp)
   (:conditional)
   (:info target not-p)
-  (:args-var args)
+  (:arg-refs args)
   (:policy :fast-safe))
 ;; A vop that accepts a computed set of widetags.
 (define-vop (%other-pointer-subtype-p type-predicate)
   (:translate %other-pointer-subtype-p)
   (:info target not-p widetags)
   (:arg-types * (:constant t)) ; voodoo - 'target' and 'not-p' are absent
-  (:args-var args)
+  (:arg-refs args)
   (:generator 15 ; arbitrary
     (%test-headers value temp target not-p nil (canonicalize-widetags widetags)
                    :value-tn-ref args))))
