@@ -79,7 +79,7 @@
            (return-from ir1-transform-type-predicate t))
           ((eq type *empty-type*)
            (return-from ir1-transform-type-predicate nil)))
-    (let ((intersect (type-intersection2 type otype)))
+    (let ((intersect (type-intersection type otype)))
       ;; I guess the theory here is that an intersection type
       ;; is never a singleton, because if we could see that it was
       ;; a singleton, it wouldn't be an intersection.
@@ -90,25 +90,74 @@
       ;; If the object type is known to be (OR NULL <type>),
       ;; it is almost always cheaper to test for not EQ to NIL.
       ;; There is one exception:
-      ;;  - FIXNUMP is possibly cheapear than comparison to NIL, or definitely
+      ;;  - FIXNUMP is possibly cheaper than comparison to NIL, or definitely
       ;;    not worse. For x86, NIL is a 4-byte immediate operand,
       ;;    for lack of a null-tn register. FIXNUM-TAG-MASK is only 1 byte.
       (when (type= otype (type-union (specifier-type 'null) type))
         (let ((difference (type-difference type (specifier-type 'null))))
           (unless (type= difference (specifier-type 'fixnum))
             (return-from  ir1-transform-type-predicate `(not (null object))))))
-      (cond ((typep type 'alien-type-type)
-                    ;; We don't transform alien type tests until here, because
-                    ;; once we do that the rest of the type system can no longer
-                    ;; reason about them properly -- so we'd miss out on type
-                    ;; derivation, etc.
-                    (delay-ir1-transform node :optimize)
-                    (let ((alien-type (alien-type-type-alien-type type)))
-                      ;; If it's a lisp-rep-type, the CTYPE should be one already.
-                      (aver (not (compute-lisp-rep-type alien-type)))
-                      `(sb-alien::alien-value-typep object ',alien-type)))
-            (t
-             (give-up-ir1-transform))))))
+      (flet ((memory-type-test-p (type)
+               (types-equal-or-intersect
+                type
+                (specifier-type
+                 '(not (or fixnum #+64-bit single-float
+                                  boolean character
+                        function list))))))
+        (cond ((typep type 'alien-type-type)
+               ;; We don't transform alien type tests until here, because
+               ;; once we do that the rest of the type system can no longer
+               ;; reason about them properly -- so we'd miss out on type
+               ;; derivation, etc.
+               (delay-ir1-transform node :ir1-phases)
+               (let ((alien-type (alien-type-type-alien-type type)))
+                 ;; If it's a lisp-rep-type, the CTYPE should be one already.
+                 (aver (not (compute-lisp-rep-type alien-type)))
+                 `(sb-alien::alien-value-typep object ',alien-type)))
+              ;; (typep (the (or list fixnum) x) 'integer) =>
+              ;; (typep x 'fixnum)
+              ((let ((current (combination-fun-source-name node))
+                     (new-predicate
+                       (or
+                        (backend-type-predicate intersect)
+                        ;; Remove bounds from numeric types
+                        (and (csubtypep intersect (specifier-type 'real))
+                             (macrolet ((up (&rest types)
+                                          `(cond ,@(loop for (type predicate) on types by #'cddr
+                                                         collect
+                                                         `((and (csubtypep intersect (specifier-type ',type))
+                                                                (csubtypep (specifier-type ',type) type))
+                                                           ',predicate)))))
+                               (up fixnum fixnump
+                                   integer integerp
+                                   rational rationalp
+                                   single-float single-float-p
+                                   double-float double-float-p
+                                   float floatp
+                                   real realp))))))
+                 (when (and new-predicate
+                            (neq new-predicate current)
+                            ;; Some subtypes are more expensive to check
+                            (not (and (eq current 'listp)
+                                      (eq new-predicate 'consp)))
+                            (not (and (eq current 'functionp)
+                                      (eq new-predicate 'compiled-function-p)))
+                            (not (eq current 'characterp))
+                            (not (eq new-predicate #+64-bit 'signed-byte-64-p
+                                                   #-64-bit 'signed-byte-32-p))
+                            (not (eq new-predicate #+64-bit 'unsigned-byte-64-p
+                                                   #-64-bit 'unsigned-byte-32-p)))
+                   `(,new-predicate object))))
+              ;; (typep (the float x) 'double-float) =>
+              ;; (typep x 'single-float)
+              ((and (memory-type-test-p type)
+                    (let* ((diff (type-difference otype type))
+                           (pred (and (not (memory-type-test-p diff))
+                                      (backend-type-predicate diff))))
+                      (when pred
+                        `(not (,pred object))))))
+              (t
+               (give-up-ir1-transform)))))))
 
 ;;; Flush %TYPEP tests whose result is known at compile time.
 (deftransform %typep ((object type) * * :node node)
@@ -326,14 +375,18 @@
               `(or (eq ,object ,low) (eq ,object ,high)))
              #+(or x86 x86-64 arm arm64) ;; Not implemented elsewhere yet
              ((and (eql (numeric-type-class type) 'integer)
-                   (or (eql low 0) (eql low 1))
-                   (fixnump (numeric-type-high type)))
-              (let ((mod-p
-                      `(fixnum-mod-p ,object ,(numeric-type-high type))))
+                   (or (eql low 0)
+                       (and (eql low 1)
+                            (not (eql high most-positive-fixnum))))
+                   (fixnump high))
+              (let ((mod-p `(fixnum-mod-p ,object ,high)))
                 (if (eql low 1)
                     `(and (not (eq ,object 0))
                           ,mod-p)
                     mod-p)))
+             ((type= type (specifier-type '(or word sb-vm:signed-word)))
+              `(or (typep ,object 'sb-vm:signed-word)
+                   (typep ,object 'word)))
              (t
               `(and (typep ,object ',base)
                     ,(transform-numeric-bound-test object type base)))))
@@ -507,6 +560,64 @@
              other
              `((typep ,object '(or ,@(mapcar #'type-specifier other))))))))))
 
+(defun source-transform-union-numeric-typep (object types)
+  (cond ((not (every #'numeric-type-p types))
+         nil)
+        ((and (= (length types) 2)
+              ;; (and subtype-of-integer (not (eql x)))
+              ;; don't test a range.
+              ;; (and subtype-of-integer (not (integer x y)))
+              (destructuring-bind (b a) types
+                (and (eq (numeric-type-class a) 'integer)
+                     (eq (numeric-type-class b) 'integer)
+                     (flet ((check (a b)
+                              (let* ((a-hi (numeric-type-high a))
+                                     (a-lo (numeric-type-low a))
+                                     (b-hi (numeric-type-high b))
+                                     (b-lo (numeric-type-low b)))
+                                (when (and a-hi b-lo
+                                           (not (eql a-lo a-hi))
+                                           (not (eql b-lo b-hi))
+                                           (> b-lo a-hi))
+                                  (let* (typecheck
+                                         (a
+                                           (%source-transform-typep object
+                                                                    `(integer ,(or a-lo '*) ,(or b-hi '*))))
+                                         (b `(not
+                                              ,(cond ((= (1+ a-hi)
+                                                         (1- b-lo))
+                                                      `(eql ,object ,(1+ a-hi)))
+                                                     (t
+                                                      (setf typecheck t)
+                                                      (%source-transform-typep object
+                                                                               `(integer (,a-hi) (,b-lo))))))))
+                                    (if typecheck
+                                        `(and ,a ,b)
+                                        `(and ,b ,a)))))))
+                       (or (check a b)
+                           (check b a)))))))
+        ((and (= (length types) 2)
+              ;; (or (integer * fixnum-x) (integer fixnum-y))
+              ;; only check for bignump and not its value.
+              (destructuring-bind (b a) types
+                (and (eq (numeric-type-class a) 'integer)
+                     (eq (numeric-type-class b) 'integer)
+                     (flet ((check (a b)
+                              (let* ((a-hi (numeric-type-high a))
+                                     (a-lo (numeric-type-low a))
+                                     (b-hi (numeric-type-high b))
+                                     (b-lo (numeric-type-low b)))
+                                (when (and (fixnump a-hi)
+                                           (fixnump b-lo)
+                                           (not a-lo)
+                                           (not b-hi))
+                                  `(or (and (fixnump ,object)
+                                            (or (>= ,object ,b-lo)
+                                                (<= ,object ,a-hi)))
+                                       (bignump ,object))))))
+                       (or (check a b)
+                           (check b a)))))))))
+
 ;;; Do source transformation for TYPEP of a known union type. If a
 ;;; union type contains LIST, then we pull that out and make it into a
 ;;; single LISTP call.
@@ -535,6 +646,7 @@
                                 (remove type-symbol types)))))))
           ((group-vector-type-length-tests object types))
           ((group-vector-length-type-tests object types))
+          ((source-transform-union-numeric-typep object types))
           (t
            (multiple-value-bind (widetags more-types)
                (sb-kernel::widetags-from-union-type types)
@@ -554,11 +666,32 @@
                                     `(typep ,object ',(type-specifier x)))
                                   more-types))))))))))
 
-;;; Do source transformation for TYPEP of a known intersection type.
 (defun source-transform-intersection-typep (object type)
-  `(and ,@(mapcar (lambda (x)
-                    `(typep ,object ',(type-specifier x)))
-                  (intersection-type-types type))))
+  (let (types
+        negated)
+    ;; Group negated types into a union type which might be better
+    ;; handled by source-transform-union-typep above.
+    (loop for type in (intersection-type-types type)
+          do
+          (cond ((hairy-type-p type)
+                 ;; These might impose some sort of an order.
+                 (setf negated nil)
+                 (return))
+                ((typep type 'negation-type)
+                 (push (negation-type-type type) negated))
+                (t
+                 (push type types))))
+    (cond (negated
+           `(and ,@(and types
+                        `((typep ,object
+                                 '(and ,@(mapcar #'type-specifier types)))))
+                 (not
+                  (typep ,object
+                         '(or ,@(mapcar #'type-specifier negated))))))
+          (t
+           `(and ,@(mapcar (lambda (x)
+                             `(typep ,object ',(type-specifier x)))
+                           (intersection-type-types type)))))))
 
 ;;; If necessary recurse to check the cons type.
 (defun source-transform-cons-typep
@@ -636,31 +769,23 @@
 
 #+sb-simd-pack
 (defun source-transform-simd-pack-typep (object type)
-  (if (type= type (specifier-type 'simd-pack))
+  (let ((mask (simd-pack-type-tag-mask type)))
+    (if (= mask sb-kernel::+simd-pack-wild+)
       `(simd-pack-p ,object)
-      (let ((n-tag (gensym "TAG")))
-        `(and
-          (simd-pack-p ,object)
-          (let ((,n-tag (%simd-pack-tag ,object)))
-            (or ,@(loop
-                    for bit across (simd-pack-type-element-type type)
-                    for i from 0
-                    if (= bit 1)
-                      collect `(eql ,n-tag ,i))))))))
+      `(and (simd-pack-p ,object)
+            ,(if (= (logcount mask) 1)
+                 `(eql (%simd-pack-tag ,object) ,(sb-vm::simd-pack-mask->tag mask))
+                 `(logbitp (%simd-pack-tag ,object) ,mask))))))
 
 #+sb-simd-pack-256
 (defun source-transform-simd-pack-256-typep (object type)
-  (if (type= type (specifier-type 'simd-pack-256))
-      `(simd-pack-256-p ,object)
-      (let ((n-tag (gensym "TAG")))
-        `(and
-          (simd-pack-256-p ,object)
-          (let ((,n-tag (%simd-pack-256-tag ,object)))
-            (or ,@(loop
-                    for bit across (simd-pack-256-type-element-type type)
-                    for i from 0
-                    if (= bit 1)
-                      collect `(eql ,n-tag ,i))))))))
+  (let ((mask (simd-pack-256-type-tag-mask type)))
+    (if (= mask sb-kernel::+simd-pack-wild+)
+        `(simd-pack-256-p ,object)
+        `(and (simd-pack-256-p ,object)
+              ,(if (= (logcount mask) 1)
+                   `(eql (%simd-pack-256-tag ,object) ,(sb-vm::simd-pack-mask->tag mask))
+                   `(logbitp (%simd-pack-256-tag ,object) ,mask))))))
 
 ;;; Return the predicate and type from the most specific entry in
 ;;; *TYPE-PREDICATES* that is a supertype of TYPE.
@@ -888,15 +1013,14 @@
 
 #-(or x86 x86-64) ; vop-translated for these 2
 (defmacro layout-depthoid-ge (layout depthoid)
-  `(>= (wrapper-depthoid ,layout) ,depthoid))
-(symbol-macrolet ((get-hash #+metaspace 'layout-clos-hash #-metaspace 'wrapper-clos-hash)
-                  (get-flags #+metaspace 'layout-flags #-metaspace 'wrapper-flags))
+  `(>= (layout-depthoid ,layout) ,depthoid))
+(symbol-macrolet ((get-hash 'layout-clos-hash)
+                  (get-flags 'layout-flags))
 (defun transform-instance-typep (classoid)
   (binding*
       ((name (classoid-name classoid))
-       (wrapper (let ((res (info :type :compiler-layout name)))
-                 (when (and res (not (wrapper-invalid res))) res)))
-       (layout (and wrapper (wrapper-friend wrapper)))
+       (layout (let ((res (info :type :compiler-layout name)))
+                 (when (and res (not (layout-invalid res))) res)))
        ((lowtag lowtag-test slot-reader)
         (cond ((csubtypep classoid (specifier-type 'funcallable-instance))
                (values sb-vm:fun-pointer-lowtag
@@ -904,7 +1028,7 @@
               ((csubtypep classoid (specifier-type 'instance))
                (values sb-vm:instance-pointer-lowtag
                        '(%instancep object) '(%instance-layout object)))))
-       (depthoid (if wrapper (wrapper-depthoid wrapper) -1))
+       (depthoid (if layout (layout-depthoid layout) -1))
        (type (make-symbol "TYPE")))
     (declare (ignorable layout))
 
@@ -932,16 +1056,16 @@
           ;; I think that means we should know the lowtag always. Nonetheless, this isn't
           ;; an important scenario, and only if you _do_ seal a class could this case be
           ;; reached; users rarely seal their classes since the standard doesn't say how.
-          ((and wrapper
+          ((and layout
                 (eq (classoid-state classoid) :sealed)
                 (not (classoid-subclasses classoid)))
            (cond ((and (eq lowtag sb-vm:instance-pointer-lowtag)
                        (vop-existsp :translate structure-typep))
-                  `(structure-typep object ,wrapper))
+                  `(structure-typep object ,layout))
                  (lowtag-test
                   `(and ,lowtag-test
                         ,(if (vop-existsp :translate layout-eq)
-                             `(layout-eq object ,wrapper ,lowtag)
+                             `(layout-eq object ,layout ,lowtag)
                              `(eq ,slot-reader ,layout))))
                  (t
                   ;; `(eq ,layout
@@ -951,10 +1075,10 @@
                   ;;        (cond ((%instancep object) (%instance-layout object))
                   ;;              ((funcallable-instance-p object) (%fun-layout object))
                   ;;              (t ,(find-layout 't)))))
-                  (bug "Unexpected metatype for ~S" wrapper))))
+                  (bug "Unexpected metatype for ~S" layout))))
 
           ;; All other structure types
-          ((and (typep classoid 'structure-classoid) wrapper)
+          ((and (typep classoid 'structure-classoid) layout)
             ;; structure type tests; hierarchical layout depths
             (aver (eql lowtag sb-vm:instance-pointer-lowtag))
             ;; we used to check for invalid layouts here, but in fact that's both unnecessary and
@@ -962,14 +1086,14 @@
             ;; because it is quite legitimate to pass an object with an invalid layout
             ;; to a structure type test.
            (if (vop-existsp :translate structure-typep)
-               ;; A single VOP is easier to optimize later in ir2opt.
-               `(structure-typep object ,wrapper)
+               ;; A single VOP is easier to optimize later
+               `(structure-typep object ,layout)
                `(and (%instancep object)
                      ,(if (<= depthoid sb-kernel::layout-id-vector-fixed-capacity)
-                          `(%structure-is-a (%instance-layout object) ,wrapper)
+                          `(%structure-is-a (%instance-layout object) ,layout)
                           `(let ((,type (%instance-layout object)))
                              (and (layout-depthoid-ge ,type ,depthoid)
-                                  (%structure-is-a ,type ,wrapper)))))))
+                                  (%structure-is-a ,type ,layout)))))))
 
           ((> depthoid 0)
            ;; fixed-depth ancestors of non-structure types:
@@ -1035,8 +1159,12 @@
              (or
               (let ((pred (backend-type-predicate ctype)))
                 (when pred `(,pred ,object)))
-              (let ((pred (backend-type-predicate (type-negation ctype))))
-                (when pred `(not (,pred ,object)))))
+              (let* ((negated (type-negation ctype))
+                     (pred (backend-type-predicate negated)))
+                (cond (pred
+                       `(not (,pred ,object)))
+                      ((numeric-type-p negated)
+                       `(not ,(%source-transform-typep object (type-specifier negated)))))))
            #+sb-xc-host
            (sb-kernel::cross-type-warning
              nil))
@@ -1434,3 +1562,45 @@
 
 ;;; BIGNUMP is simpler than INTEGERP, so if we can rule out FIXNUM then ...
 (deftransform integerp ((x) ((not fixnum)) * :important nil) '(bignump x))
+
+(deftransform structure-typep ((object type) (t (constant-arg t)))
+  (let* ((layout (lvar-value type))
+         (type (case layout
+                 (#.+condition-layout-flag+ (specifier-type 'condition))
+                 (#.+pathname-layout-flag+  (specifier-type 'pathname))
+                 (#.+structure-layout-flag+ (specifier-type 'structure-object))
+                 (t
+                  (layout-classoid layout))))
+         (diff (type-difference (lvar-type object) type))
+         (pred (backend-type-predicate diff)))
+    (if pred
+        `(not (,pred object))
+        (give-up-ir1-transform))))
+
+(deftransform classoid-cell-typep ((cell object) ((constant-arg t) t))
+  (let* ((type (specifier-type (classoid-cell-name (lvar-value cell))))
+         (diff (type-difference (lvar-type object) type))
+         (pred (backend-type-predicate diff)))
+    (if pred
+        `(not (,pred object))
+        (give-up-ir1-transform))))
+
+(when-vop-existsp (:translate fixnum-mod-p)
+  (deftransform fixnum-mod-p ((x mod) (t (constant-arg fixnum)) * :important nil)
+    (let* ((type (lvar-type x))
+           (mod (lvar-value mod))
+           (intersect (type-intersection type (specifier-type 'fixnum)))
+           (mod-type (specifier-type `(mod ,(1+ mod)))))
+      (cond ((csubtypep intersect mod-type)
+             `(fixnump x))
+            ((and (csubtypep type (specifier-type 'fixnum))
+                  (csubtypep (type-intersection type (specifier-type 'unsigned-byte))
+                             mod-type))
+             `(>= x 0))
+            ((let ((int (type-approximate-interval intersect)))
+               (when int
+                 (let ((power-of-two (1- (ash 1 (integer-length (interval-high int))))))
+                   (when (< 0 power-of-two mod)
+                     `(fixnum-mod-p x ,power-of-two))))))
+            (t
+             (give-up-ir1-transform))))))
