@@ -26,7 +26,6 @@
 #include <assert.h>
 #include <errno.h>
 #include "sbcl.h"
-#include "./signal.h"
 #include "os.h"
 #include "arch.h"
 #include "globals.h"
@@ -81,6 +80,9 @@ static void dragonfly_init();
 #ifdef LISP_FEATURE_X86
 #include <machine/cpu.h>
 #endif
+#ifdef LISP_FEATURE_SB_FUTEX
+#include <sys/futex.h>
+#endif
 
 static void openbsd_init();
 #endif
@@ -129,25 +131,27 @@ os_alloc_gc_space(int space_id, int attributes, os_vm_address_t addr, os_vm_size
     int protection;
     int flags = 0;
 
-#if defined(LISP_FEATURE_OPENBSD) && defined(MAP_STACK)
-        /* OpenBSD requires MAP_STACK for pages used as stack.
-         * Note that FreeBSD has a MAP_STACK with different behavior. */
-    if (space_id == THREAD_STRUCT_CORE_SPACE_ID) flags = MAP_STACK;
-#endif
-
     // FIXME: This probaby needs to use MAP_TRYFIXED
     if (attributes & IS_GUARD_PAGE)
         protection = OS_VM_PROT_NONE;
     else
-#ifndef LISP_FEATURE_DARWIN_JIT
+#if defined(LISP_FEATURE_OPENBSD) && defined(MAP_STACK)
+    /* OpenBSD requires MAP_STACK for pages used as stack (and RW protection).
+     * Note that FreeBSD has a MAP_STACK with different behavior. */
+    if (space_id == THREAD_STRUCT_CORE_SPACE_ID) {
+        protection = OS_VM_PROT_READ | OS_VM_PROT_WRITE;
+        flags |= MAP_STACK;
+    } else {
         protection = OS_VM_PROT_ALL;
-#else
+    }
+
+#elif defined(LISP_FEATURE_DARWIN_JIT)
     if (jit) {
         if (jit == 2)
             protection = OS_VM_PROT_ALL;
         else
             protection = OS_VM_PROT_READ | OS_VM_PROT_WRITE;
-        flags = MAP_JIT;
+        flags |= MAP_JIT;
     }
     else if (executable) {
         protection = OS_VM_PROT_READ | OS_VM_PROT_EXECUTE;
@@ -155,6 +159,9 @@ os_alloc_gc_space(int space_id, int attributes, os_vm_address_t addr, os_vm_size
     else {
         protection = OS_VM_PROT_READ | OS_VM_PROT_WRITE;
     }
+
+#else
+        protection = OS_VM_PROT_ALL;
 #endif
 
     attributes &= ~IS_GUARD_PAGE;
@@ -187,7 +194,7 @@ os_alloc_gc_space(int space_id, int attributes, os_vm_address_t addr, os_vm_size
        Except for MAP_FIXED mappings, the system will never replace existing mappings. */
 
     // ALLOCATE_LOW seems never to get what we want
-    if (!(attributes & MOVABLE) || (attributes & ALLOCATE_LOW)) flags = MAP_FIXED;
+    if (!(attributes & MOVABLE) || (attributes & ALLOCATE_LOW)) flags |= MAP_FIXED;
 #endif
 
 #ifdef MAP_EXCL // not defined in OpenBSD, NetBSD, DragonFlyBSD
@@ -264,8 +271,6 @@ memory_fault_handler(int signal, siginfo_t *siginfo, os_context_t *context)
     os_restore_tls_segment_register(context);
 #endif
 
-    FSHOW((stderr, "Memory fault at: %p, PC: %p\n", fault_addr, OS_CONTEXT_PC(context)));
-
 #ifdef LISP_FEATURE_SB_SAFEPOINT
     if (handle_safepoint_violation(context, fault_addr)) return;
 #endif
@@ -407,17 +412,17 @@ static void freebsd_init()
 
 #ifdef LISP_FEATURE_SB_FUTEX
 int
-futex_wait(int *lock_word, long oldval, long sec, unsigned long usec)
+futex_wait(int *lock_word, int oldval, long sec, unsigned long usec)
 {
     struct timespec timeout;
     int ret;
 
     if (sec < 0)
-        ret = _umtx_op((void *)lock_word, UMTX_OP_WAIT, oldval, 0, 0);
+        ret = _umtx_op(lock_word, UMTX_OP_WAIT_UINT_PRIVATE, oldval, 0, 0);
     else {
         timeout.tv_sec = sec;
         timeout.tv_nsec = usec * 1000;
-        ret = _umtx_op((void *)lock_word, UMTX_OP_WAIT, oldval, (void*)sizeof timeout, &timeout);
+        ret = _umtx_op(lock_word, UMTX_OP_WAIT_UINT_PRIVATE, oldval, (void*)sizeof timeout, &timeout);
     }
     if (ret == 0) return 0;
     // technically we would not need to check any of the error codes if the lisp side
@@ -430,7 +435,7 @@ futex_wait(int *lock_word, long oldval, long sec, unsigned long usec)
 int
 futex_wake(int *lock_word, int n)
 {
-    return _umtx_op((void *)lock_word, UMTX_OP_WAKE, n, 0, 0);
+    return _umtx_op(lock_word, UMTX_OP_WAKE_PRIVATE, n, 0, 0);
 }
 #endif
 #endif /* __FreeBSD__ */
@@ -601,4 +606,25 @@ os_dlsym(void *handle, const char *symbol)
     return ret;
 }
 
+#ifdef LISP_FEATURE_SB_FUTEX
+int
+futex_wait(int *lock_word, int oldval, long sec, unsigned long usec)
+{
+    struct timespec timeout = {.tv_sec = sec, .tv_nsec = usec * 1000};
+    int ret = futex(lock_word, FUTEX_WAIT_PRIVATE, oldval, sec < 0 ? NULL : &timeout, NULL);
+
+    if (ret == 0) return 0;
+    if (errno == ETIMEDOUT) return 1;
+    if (errno == EINTR) return 2;
+    return -1;
+}
+
+int
+futex_wake(int *lock_word, int n)
+{
+    futex(lock_word, FUTEX_WAKE_PRIVATE, n, NULL, NULL);
+    return 0;
+}
+#endif
+
 #endif
