@@ -45,12 +45,12 @@
 #define _FORTIFY_SOURCE 0
 #endif
 
-#include "brothertree.h"
 #include "code.h"
 #include "forwarding-ptr.h"
 #include "gc.h"
 #include "gc-internal.h"
 #include "gc-private.h"
+#include "genesis/brothertree.h"
 #include "genesis/cons.h"
 #include "genesis/gc-tables.h"
 #include "genesis/hash-table.h"
@@ -65,7 +65,7 @@
 #include "var-io.h"
 #include <stdlib.h>
 #include <stdio.h>
-#include "tlsf-bsd/tlsf/tlsf.h"
+#include "../../tlsf-bsd/tlsf/tlsf.h"
 
 #define WORDS_PER_PAGE ((int)IMMOBILE_CARD_BYTES/N_WORD_BYTES)
 #define DOUBLEWORDS_PER_PAGE (WORDS_PER_PAGE/2)
@@ -82,7 +82,7 @@ FILE * logfile;
 #  define dprintf(arg)
 #endif
 
-static void defrag_immobile_space(boolean verbose);
+static void defrag_immobile_space(bool verbose);
 
 uword_t FIXEDOBJ_SPACE_START, TEXT_SPACE_START;
 uword_t immobile_space_lower_bound, immobile_space_max_offset;
@@ -143,7 +143,7 @@ static int n_bitmap_elts; // length of array measured in 'int's
 // chained through the word after their header.
 lispobj codeblob_freelist;
 
-boolean immobile_card_protected_p(void* addr)
+bool immobile_card_protected_p(void* addr)
 {
     low_page_index_t page;
     page = find_text_page_index(addr);
@@ -168,8 +168,6 @@ static const unsigned block_header_prev_free_bit = 1 << 9;
  * So there's really no need to do anything about it.
 static const unsigned block_header_oversized = 1 << 10;
 */
-
-#define IMMOBILE_CARD_SHIFT 12
 void *tlsf_alloc_codeblob(tlsf_t tlsf, int requested_nwords)
 {
     // The size we request is 1 word less, because the allocator's block header
@@ -186,7 +184,7 @@ void *tlsf_alloc_codeblob(tlsf_t tlsf, int requested_nwords)
     lispobj* end = (lispobj*)c + nwords;
     if (end > text_space_highwatermark) text_space_highwatermark = end;
     // Adjust the scan start if this became the lowest addressable in-use block on its page
-    low_page_index_t tlsf_page = ((char*)c - (char*)tlsf_mem_start) >> IMMOBILE_CARD_SHIFT;
+    low_page_index_t tlsf_page = ((char*)c - (char*)tlsf_mem_start) / IMMOBILE_CARD_BYTES;
     int offset = (uword_t)c & (IMMOBILE_CARD_BYTES-1);
     if (offset < tlsf_page_sso[tlsf_page]) tlsf_page_sso[tlsf_page] = offset;
     text_page_genmask[find_text_page_index(c)] |= 1;
@@ -224,7 +222,7 @@ void tlsf_unalloc_codeblob(tlsf_t tlsf, struct code* code)
         gc_assert(!((uword_t)text_space_highwatermark & LOWTAG_MASK));
     }
     // See if the page scan start needs to change
-    low_page_index_t tlsf_page = ((char*)code - (char*)tlsf_mem_start) >> IMMOBILE_CARD_SHIFT;
+    low_page_index_t tlsf_page = ((char*)code - (char*)tlsf_mem_start) / IMMOBILE_CARD_BYTES;
     int offset = (uword_t)code & (IMMOBILE_CARD_BYTES-1);
     if (offset == tlsf_page_sso[tlsf_page]) {
         lispobj* next = end;
@@ -234,7 +232,7 @@ void tlsf_unalloc_codeblob(tlsf_t tlsf, struct code* code)
         }
         // If the next used block is on the same page, then it becomes the page scan start
         // even if it the ending sentinel block (which counts as "used").
-        if ((((uword_t)next ^ (uword_t)code) >> IMMOBILE_CARD_SHIFT) == 0) {
+        if ((((uword_t)next ^ (uword_t)code) / IMMOBILE_CARD_BYTES) == 0) {
             tlsf_page_sso[tlsf_page] = (uword_t)next & (IMMOBILE_CARD_BYTES-1);
         } else {
             tlsf_page_sso[tlsf_page] = USHRT_MAX;
@@ -337,7 +335,7 @@ long immobile_alloc_collisions;
      masking. if the next address is above or equal to the page start,
      store it in the hint, otherwise mark the page full */
 
-lispobj AMD64_SYSV_ABI
+lispobj
 alloc_immobile_fixedobj(int size_class, int spacing_words, uword_t header)
 {
   size_class = fixnum_value(size_class);
@@ -450,7 +448,7 @@ enliven_immobile_obj(lispobj *ptr, int rescan) // a native pointer
         bits |= OBJ_WRITTEN_FLAG;
     assign_generation(ptr, bits | new_space);
     low_page_index_t page_index = find_fixedobj_page_index(ptr);
-    boolean is_text = 0;
+    bool is_text = 0;
 
     if (page_index < 0) {
         page_index = find_text_page_index(ptr);
@@ -521,7 +519,7 @@ lispobj* search_immobile_code(char* ptr) {
     } else if (ptr < (char*)text_space_highwatermark) {
         lispobj node = brothertree_find_lesseql((uword_t)ptr,
                                                 SYMBOL(IMMOBILE_CODEBLOB_TREE)->value);
-        if (node != NIL) candidate = (lispobj*)((struct binary_node*)INSTANCE(node))->key;
+        if (node != NIL) candidate = (lispobj*)((struct binary_node*)INSTANCE(node))->uw_key;
     }
     if (candidate && widetag_of(candidate) == CODE_HEADER_WIDETAG) {
         int nwords = code_total_nwords((struct code*)candidate);
@@ -533,7 +531,7 @@ lispobj* search_immobile_code(char* ptr) {
 /* If 'addr' points to an immobile object, then make the object
    live by promotion. But if the object is not in the generation
    being collected, do nothing */
-boolean immobile_space_preserve_pointer(void* addr)
+bool immobile_space_preserve_pointer(void* addr)
 {
     unsigned char genmask = compacting_p() ? 1<<from_space : 0xff;
     lispobj* object_start;
@@ -610,7 +608,7 @@ static void full_scavenge_immobile_newspace()
         do {
             lispobj* limit = (lispobj*)text_page_address(page) + WORDS_PER_PAGE;
             if (limit > text_space_highwatermark) limit = text_space_highwatermark;
-            int n_words;
+            sword_t n_words;
             for ( ; obj < limit ; obj += n_words ) {
                 lispobj header = *obj;
                 if (immobile_obj_gen_bits(obj) == new_space) {
@@ -717,7 +715,8 @@ scavenge_immobile_roots(generation_index_t min_gen, generation_index_t max_gen)
         if (!obj) { ++page; continue; }
         do {
             lispobj* limit = (lispobj*)text_page_address(page) + WORDS_PER_PAGE;
-            int n_words, gen;
+            sword_t n_words;
+            int gen;
             if (limit > text_space_highwatermark) limit = text_space_highwatermark;
             for ( ; obj < limit ; obj += n_words ) {
                 lispobj header = *obj;
@@ -826,7 +825,7 @@ range_points_to_younger_p(lispobj* obj, lispobj* end,
 // Scan a fixed-size object for old-to-young pointers.
 // Since fixed-size objects are boxed and on known boundaries,
 // we never start in the middle of random bytes, so the answer is exact.
-static boolean
+static bool
 fixedobj_points_to_younger_p(lispobj* obj, int n_words,
                              int gen, int keep_gen, int new_gen)
 {
@@ -868,7 +867,7 @@ fixedobj_points_to_younger_p(lispobj* obj, int n_words,
 // to younger generations, which we detect by a cleared WP status bit.
 // The bit is cleared on any write, though, even of a non-pointer,
 // so this unfortunately has to be tested much more often than we'd like.
-static inline boolean can_wp_fixedobj_page(page_index_t page, int keep_gen, int new_gen)
+static inline bool can_wp_fixedobj_page(page_index_t page, int keep_gen, int new_gen)
 {
     int obj_spacing = fixedobj_page_obj_align(page);
     lispobj* obj = fixedobj_page_address(page);
@@ -885,7 +884,7 @@ static inline boolean can_wp_fixedobj_page(page_index_t page, int keep_gen, int
 }
 
 // Return 1 if any header on 'page' is in the remembered set.
-static inline boolean can_wp_text_page(page_index_t page)
+static inline bool can_wp_text_page(page_index_t page)
 {
     lispobj *begin = text_page_address(page);
     lispobj *end   = begin + WORDS_PER_PAGE;
@@ -1060,7 +1059,8 @@ sweep_text_pages(int raise)
         // wp_it is 1 if we should try to write-protect it now.
         // If already write-protected, skip the tests.
         int wp_it = ENABLE_PAGE_PROTECTION && text_page_touched(page);
-        int size, gen;
+        sword_t size;
+        int gen;
 
         for ( ; obj < limit ; obj += size ) {
             lispobj word = *obj;
@@ -1119,7 +1119,7 @@ void gc_init_immobile()
     logfile = stderr;
 #endif
     int n_fixedobj_pages = FIXEDOBJ_SPACE_SIZE / IMMOBILE_CARD_BYTES;
-    int n_text_pages = TEXT_SPACE_SIZE / IMMOBILE_CARD_BYTES;
+    int n_text_pages = text_space_size / IMMOBILE_CARD_BYTES;
     fixedobj_pages = calloc(n_fixedobj_pages, sizeof(struct fixedobj_page));
     gc_assert(fixedobj_pages);
 
@@ -1182,7 +1182,7 @@ void immobile_space_coreparse(uword_t fixedobj_len, uword_t text_len)
             lispobj header = *obj;
             if (!fixnump(header)) {
                 gc_assert(other_immediate_lowtag_p(*obj));
-                int size = object_size2(obj, header);
+                sword_t size = object_size2(obj, header);
                 fixedobj_pages[page].attr.parts.obj_align = size;
                 fixedobj_pages[page].gens |= 1 << immobile_obj_gen_bits(obj);
                 if (gen != 0 && ENABLE_PAGE_PROTECTION)
@@ -1288,7 +1288,7 @@ void prepare_immobile_space_for_final_gc()
     SYMBOL(IMMOBILE_CODEBLOB_TREE)->value = NIL;
 }
 
-int* code_component_order;
+uword_t* code_component_order;
 
 int compute_codeblob_offsets_nwords(int* pcount)
 {
@@ -1304,13 +1304,16 @@ int compute_codeblob_offsets_nwords(int* pcount)
  * 'coreparse' causes all pages in dynamic space to be pseudo-static, but
  * each immobile object stores its own generation, so this must be done at
  * save time, or else it would require touching every object on startup */
-void prepare_immobile_space_for_save(boolean verbose)
+void prepare_immobile_space_for_save(bool verbose)
 {
     if (verbose) {
         printf("[defragmenting immobile space... ");
         fflush(stdout);
     }
+
+#ifdef LISP_FEATURE_X86_64
     defrag_immobile_space(verbose);
+#endif
 
     lispobj* obj = (lispobj*)FIXEDOBJ_SPACE_START;
     lispobj* limit = fixedobj_free_pointer;
@@ -1455,25 +1458,25 @@ static void copy_back(uword_t space_start, struct tempspace* tempspace,
     int new_usage = tempspace->n_bytes;
     memcpy((char*)space_start, tempspace->start, tempspace->n_bytes);
     uword_t end = space_start + new_usage;
-    if (new_usage < old_usage) bzero((char*)end, old_usage - new_usage);
+    if (new_usage < old_usage) memset((char*)end, 0, old_usage - new_usage);
     *pfree_ptr = (lispobj*)end;
 }
 
-static inline boolean tempspace_p(char* addr)
+static inline bool tempspace_p(char* addr)
 {
     return (addr >= fixedobj_tempspace.start &&
             addr < fixedobj_tempspace.start + fixedobj_tempspace.n_bytes)
         || (addr >= text_tempspace.start &&
             addr < text_tempspace.start + text_tempspace.n_bytes);
 }
-static inline boolean known_space_p(lispobj ptr)
+static inline bool known_space_p(lispobj ptr)
 {
     return find_page_index((char*)ptr) >= 0
         || tempspace_p((char*)ptr)
         || immobile_space_p(ptr)
         || (STATIC_SPACE_OBJECTS_START <= ptr && ptr < STATIC_SPACE_END);
 }
-static boolean forwardable_ptr_p(lispobj ptr)
+static bool forwardable_ptr_p(lispobj ptr)
 {
     return is_lisp_pointer(ptr) &&
            known_space_p(ptr) &&
@@ -1507,20 +1510,13 @@ static lispobj adjust_fun_entrypoint(lispobj raw_addr)
  * and return the layout's address in tempspace. */
 static struct layout* fix_object_layout(lispobj* obj)
 {
-    // This works on instances, funcallable instances (and/or closures)
-    // but the latter only if the layout is in the header word.
-#ifdef LISP_FEATURE_COMPACT_INSTANCE_HEADER
-    gc_assert(widetag_of(obj) == INSTANCE_WIDETAG
-              || widetag_of(obj) == FUNCALLABLE_INSTANCE_WIDETAG
-              || widetag_of(obj) == CLOSURE_WIDETAG);
-#else
-    gc_assert(widetag_of(obj) == INSTANCE_WIDETAG);
+#ifndef LISP_FEATURE_COMPACT_INSTANCE_HEADER
+    // Without compact instance headers, there are no layouts in immobile-space,
+    return LAYOUT(layout_of(obj));
 #endif
+    gc_assert(instanceoid_widetag_p(widetag_of(obj)));
     lispobj layout = layout_of(obj);
     if (layout == 0) return 0;
-#ifdef LISP_FEATURE_METASPACE
-    return LAYOUT(layout);
-#else
     if (forwarding_pointer_p(native_pointer(layout))) { // usually
         layout = forwarding_pointer_value(native_pointer(layout));
         layout_of(obj) = layout;
@@ -1529,7 +1525,6 @@ static struct layout* fix_object_layout(lispobj* obj)
     gc_assert(header_widetag(native_layout->header) == INSTANCE_WIDETAG);
     gc_assert(layoutp(make_lispobj(native_layout, INSTANCE_POINTER_LOWTAG)));
     return native_layout;
-#endif
 }
 
 static void apply_absolute_fixups(lispobj, struct code*);
@@ -1541,7 +1536,7 @@ static void fixup_space(lispobj* where, size_t n_words)
 {
     lispobj* end = where + n_words;
     int widetag;
-    long size;
+    sword_t size;
     struct code* code;
 
     while (where < end) {
@@ -1573,7 +1568,9 @@ static void fixup_space(lispobj* where, size_t n_words)
           // Fixup the constant pool.
           code = (struct code*)where;
           adjust_words(where+2, code_header_words(code)-2);
+#ifdef LISP_FEATURE_X86_64
           apply_absolute_fixups(code->fixups, code);
+#endif
           break;
         case CLOSURE_WIDETAG:
           where[1] = adjust_fun_entrypoint(where[1]);
@@ -1602,7 +1599,7 @@ static void fixup_space(lispobj* where, size_t n_words)
               struct vector* kv_vector = (struct vector*)where;
               lispobj* data = kv_vector->data;
               gc_assert(vector_len(kv_vector) >= 5);
-              boolean needs_rehash = 0;
+              bool needs_rehash = 0;
               unsigned int hwm = KV_PAIRS_HIGH_WATER_MARK(data);
               unsigned int i;
               // FIXME: we're disregarding the hash vector.
@@ -1645,8 +1642,8 @@ static void fixup_space(lispobj* where, size_t n_words)
     }
 }
 
-int* immobile_space_reloc_index;
-int* immobile_space_relocs;
+uword_t* immobile_space_reloc_index;
+uword_t* immobile_space_relocs;
 
 // Take and return an untagged pointer, or 0 if the object did not survive GC.
 static lispobj* get_load_address(lispobj* old)
@@ -1770,19 +1767,19 @@ static void place_layout(lispobj* obj,
         layout_size_class_nwords(size_class_index) << WORD_SHIFT;
 }
 
-static void defrag_immobile_space(boolean verbose)
+static void defrag_immobile_space(bool verbose)
 {
     int i;
 
-    int *components = code_component_order;
+    uword_t *components = code_component_order;
 
     // Count the number of symbols, fdefns, and layouts that will be relocated
     int obj_type_histo[64];
     struct { int size, count; } sym_kind_histo[N_SYMBOL_KINDS];
-    bzero(obj_type_histo, sizeof obj_type_histo);
-    bzero(sym_kind_histo, sizeof sym_kind_histo);
+    memset(obj_type_histo, 0, sizeof obj_type_histo);
+    memset(sym_kind_histo, 0, sizeof sym_kind_histo);
     struct size_class layout_size_class[MAX_LAYOUT_DEFRAG_SIZE_CLASSES];
-    bzero(layout_size_class, sizeof layout_size_class);
+    memset(layout_size_class, 0, sizeof layout_size_class);
 
 #if DEFRAGMENT_FIXEDOBJ_SUBSPACE
     // Find the starting address of fixed-size objects that will undergo defrag.
@@ -1822,9 +1819,6 @@ static void defrag_immobile_space(boolean verbose)
             } while (NEXT_FIXEDOBJ(obj, obj_spacing) <= limit);
         }
     }
-#ifndef LISP_FEATURE_METASPACE
-    gc_assert(obj_type_histo[INSTANCE_WIDETAG/4]);
-#endif
 
     // Calculate space needed for fixedobj pages after defrag.
     // page order is: layouts, symbols, fdefns.
@@ -1859,15 +1853,8 @@ static void defrag_immobile_space(boolean verbose)
     int n_code_bytes = 0;
 
     if (components) {
-#ifdef LISP_FEATURE_METASPACE
-        /* Skip the 0th entry when assigning new addresses, because it is
-           SB-FASL::*ASSEMBLER-ROUTINES* which is in read-only space and not
-           itself relocatable, but does contain absolute fixups */
-        for (i=1 ; components[i*2] ; ++i) {
-#else
         for (i=0 ; components[i*2] ; ++i) {
-#endif
-            lispobj* addr = (lispobj*)(long)components[i*2];
+            lispobj* addr = (lispobj*)components[i*2];
             gc_assert(lowtag_of((lispobj)addr) == OTHER_POINTER_LOWTAG);
             addr = native_pointer((lispobj)addr);
             int widetag = widetag_of(addr);
@@ -1927,7 +1914,7 @@ static void defrag_immobile_space(boolean verbose)
 
 #if DEFRAGMENT_FIXEDOBJ_SUBSPACE
     char* alloc_ptrs[64];
-    bzero(alloc_ptrs, sizeof alloc_ptrs);
+    memset(alloc_ptrs, 0, sizeof alloc_ptrs);
     alloc_ptrs[INSTANCE_WIDETAG/4] = layout_alloc_ptr;
     alloc_ptrs[FDEFN_WIDETAG/4] = fdefn_alloc_ptr;
 
@@ -2080,10 +2067,6 @@ static void apply_absolute_fixups(lispobj fixups, struct code* code)
         if (asm_routines_start <= ptr && ptr < asm_routines_end) {
             continue;
         }
-#ifdef LISP_FEATURE_METASPACE
-        // Pointers to metaspace will never move, at least not for the time being.
-        if (ptr >= READ_ONLY_SPACE_START && ptr < READ_ONLY_SPACE_END) continue;
-#endif
         if (find_fixedobj_page_index((void*)ptr) >= 0) {
             header_addr = search_immobile_space((void*)ptr);
             gc_assert(header_addr);
@@ -2156,7 +2139,7 @@ void dump_immobile_text(lispobj* where, lispobj* end, FILE*f)
                     text_page_touched(page)?"":" WP");
             prevpage = page;
         }
-        fprintf(f, "%p: %lx\n", where, *where);
+        fprintf(f, "%p: %"OBJ_FMTX"\n", where, *where);
     }
 }
 
@@ -2185,3 +2168,16 @@ void dump_immobile_text(lispobj* where, lispobj* end, FILE*f)
       }
 }
 #endif
+
+void* expropriate_memory_from_tlsf(size_t amount)
+{
+  extern void* tlsf_pool_shrink(void*,void*,size_t);
+  char* end = (char*)TEXT_SPACE_START + text_space_size;
+  char* start = tlsf_pool_shrink(tlsf_control, end, amount);
+#ifdef TLSF_CONFIG_DEBUG
+  tlsf_check(tlsf_control);
+  tlsf_check_pool(tlsf_mem_start);
+  //fprintf(stderr, "TLSF integrity checks passed\n");
+#endif
+  return start;
+}
