@@ -21,6 +21,7 @@
 #include "runtime.h"
 
 #include <inttypes.h>
+#include <stdbool.h>
 
 #if defined(LISP_FEATURE_GENCGC) && !defined(ENABLE_PAGE_PROTECTION)
 /* Should we use page protection to help avoid the scavenging of pages
@@ -67,7 +68,7 @@
 extern os_vm_size_t os_vm_page_size;
 
 #if defined LISP_FEATURE_WIN32 || defined LISP_FEATURE_LINUX
-boolean os_preinit(char *argv[], char *envp[]);
+int os_preinit(char *argv[], char *envp[]);
 #else
 #define os_preinit(dummy1,dummy2) (0)
 #endif
@@ -140,9 +141,9 @@ extern void os_protect(os_vm_address_t addr,
 
 /* Return true for an address (with or without lowtag bits) within
  * any range of memory understood by the garbage collector. */
-extern boolean gc_managed_addr_p(lispobj addr);
+extern bool gc_managed_addr_p(lispobj addr);
 /* As for above, but consider only the heap spaces, not stacks */
-extern boolean gc_managed_heap_space_p(lispobj addr);
+extern bool gc_managed_heap_space_p(lispobj addr);
 
 /* Given a signal context, return the address for storage of the
  * register, of the specified offset, for that context. The offset is
@@ -226,8 +227,8 @@ extern char *os_get_runtime_executable_path();
      typedef sem_t os_sem_t;
 #  endif
    void os_sem_init(os_sem_t *sem, unsigned int value);
-   void os_sem_wait(os_sem_t *sem, char *what);
-   void os_sem_post(os_sem_t *sem, char *what);
+   void os_sem_wait(os_sem_t *sem);
+   void os_sem_post(os_sem_t *sem);
    void os_sem_destroy(os_sem_t *sem);
 #endif
 
