@@ -33,9 +33,7 @@
 #include <inttypes.h>
 #include <setjmp.h>
 
-struct dyndebug_config dyndebug_config = {
-    QSHOW == 2,
-};
+struct dyndebug_config dyndebug_config;
 
 void
 dyndebug_init()
@@ -54,7 +52,6 @@ dyndebug_init()
     char *names[DYNDEBUG_NFLAGS];
     int *ptrs[DYNDEBUG_NFLAGS];
 
-    dyndebug_init1(fshow,          "FSHOW");
     dyndebug_init1(gencgc_verbose, "GENCGC_VERBOSE");
     dyndebug_init1(safepoints,     "SAFEPOINTS");
     dyndebug_init1(seh,            "SEH");
@@ -117,12 +114,8 @@ dyndebug_init()
 #undef DYNDEBUG_NFLAGS
 }
 
-#include "monitor.h"
 #include "vars.h"
 #include "os.h"
-#ifdef LISP_FEATURE_GENCGC
-#include "gencgc-alloc-region.h" /* genesis/thread.h needs this */
-#endif
 #include "genesis/static-symbols.h"
 #include "genesis/primitive-objects.h"
 #include "genesis/static-symbols.h"
@@ -131,7 +124,7 @@ dyndebug_init()
 static int max_lines = 20, cur_lines = 0;
 static int max_depth = 5, brief_depth = 2, cur_depth = 0;
 static int max_length = 5;
-static boolean dont_descend = 0, skip_newline = 0;
+static bool dont_descend = 0, skip_newline = 0;
 static int cur_clock = 0;
 
 static void print_obj(char *prefix, lispobj obj);
@@ -151,7 +144,7 @@ static void indent(int in)
 }
 
 static jmp_buf ldb_print_nlx;
-static boolean continue_p(boolean newline)
+static bool continue_p(bool newline)
 {
     char buffer[256];
 
@@ -367,18 +360,10 @@ static void brief_struct(lispobj obj)
 #include "genesis/layout.h"
 #include "genesis/defstruct-description.h"
 #include "genesis/defstruct-slot-description.h"
-static boolean tagged_slot_p(struct layout *layout, int slot_index)
+static bool tagged_slot_p(struct layout *layout, int slot_index)
 {
-    // Since we're doing this scan, we could return the name
-    // and exact raw type.
-#ifdef LISP_FEATURE_METASPACE
-    struct wrapper *wrapper = (void*)(layout->friend-INSTANCE_POINTER_LOWTAG);
-    if (instancep(wrapper->_info)) {
-        struct defstruct_description* dd = (void*)(wrapper->_info-INSTANCE_POINTER_LOWTAG);
-#else
     if (instancep(layout->_info)) {
         struct defstruct_description* dd = (void*)(layout->_info-INSTANCE_POINTER_LOWTAG);
-#endif
         lispobj slots = dd->slots;
         for ( ; slots != NIL ; slots = CONS(slots)->cdr ) {
             struct defstruct_slot_description* dsd =
@@ -758,7 +743,7 @@ static void print_obj(char *prefix, lispobj obj)
     int type = lowtag_of(obj);
     struct var *var = lookup_by_obj(obj);
     char buffer[256];
-    boolean verbose = cur_depth < brief_depth;
+    bool verbose = cur_depth < brief_depth;
 
     if (!continue_p(verbose))
         return;
@@ -845,7 +830,6 @@ void brief_print(lispobj obj)
 // hence all the forwarding checks.
 
 #include "forwarding-ptr.h"
-#include "genesis/classoid.h"
 struct vector * symbol_name(struct symbol* sym)
 {
   if (forwarding_pointer_p((lispobj*)sym))
@@ -860,22 +844,14 @@ struct vector * classoid_name(lispobj * classoid)
   if (forwarding_pointer_p(classoid))
       classoid = native_pointer(forwarding_pointer_value(classoid));
   // Classoids are named by symbols even though a CLASS name is arbitrary (theoretically)
-  lispobj sym = ((struct classoid*)classoid)->name;
+  lispobj sym = classoid[CLASSOID_NAME_WORDINDEX];
   return lowtag_of(sym) != OTHER_POINTER_LOWTAG ? NULL : symbol_name(SYMBOL(sym));
 }
 struct vector * layout_classoid_name(lispobj * layout)
 {
-#ifdef LISP_FEATURE_METASPACE
-  // layout can't be forwarded, but wrapper could be
-  lispobj* wrapper = native_pointer(((struct layout*)layout)->friend);
-  if (forwarding_pointer_p(wrapper))
-      wrapper = native_pointer(forwarding_pointer_value(wrapper));
-  lispobj classoid = ((struct wrapper*)wrapper)->classoid;
-#else
   if (forwarding_pointer_p(layout))
       layout = native_pointer(forwarding_pointer_value(layout));
   lispobj classoid = ((struct layout*)layout)->classoid;
-#endif
   return instancep(classoid) ? classoid_name(native_pointer(classoid)) : NULL;
 }
 struct vector * instance_classoid_name(lispobj * instance)
