@@ -36,7 +36,7 @@
 #include "genesis/cons.h"
 #include "genesis/fdefn.h"
 #include "interr.h"
-#include "alloc.h"
+#include "genesis/sap.h"
 #include "gc-internal.h"
 #include "interrupt.h"
 #include "lispregs.h"
@@ -99,7 +99,6 @@ const char* gc_phase_names[GC_NPHASES] = {
  * time will move to GC_INVOKED and then wait for the WITHOUT-GCING
  * thread to finish up, then proceed to GC_COLLECT. */
 
-#ifdef LISP_FEATURE_SB_THREAD
 #define CURRENT_THREAD_VAR(name) \
     struct thread *name = get_sb_vm_thread()
 #define THREAD_STOP_PENDING(th) \
@@ -107,14 +106,8 @@ const char* gc_phase_names[GC_NPHASES] = {
 #define SET_THREAD_STOP_PENDING(th,state) \
     write_TLS(STOP_FOR_GC_PENDING,state,th)
 #define WITH_ALL_THREADS_LOCK \
-    mutex_acquire(&all_threads_lock); \
-    RUN_BODY_ONCE(all_threads_lock, mutex_release(&all_threads_lock))
-#else
-#define CURRENT_THREAD_VAR(name)
-#define THREAD_STOP_PENDING(th) NIL
-#define SET_THREAD_STOP_PENDING(th,state)
-#define WITH_ALL_THREADS_LOCK
-#endif
+    ignore_value(mutex_acquire(&all_threads_lock)); \
+    RUN_BODY_ONCE(all_threads_lock, ignore_value(mutex_release(&all_threads_lock)))
 
 #if !defined(LISP_FEATURE_WIN32)
 /* win32-os.c covers these, but there is no unixlike-os.c, so the normal
@@ -187,10 +180,8 @@ void
 gc_state_lock()
 {
     odxprint(safepoints,"GC state to be locked");
-#ifdef LISP_FEATURE_SB_THREAD
     int result = mutex_acquire(&gc_state.lock);
     gc_assert(result);
-#endif
     if (gc_state.master) {
         fprintf(stderr,"GC state lock glitch [%p] in thread %p phase %d (%s)\n",
                 gc_state.master,get_sb_vm_thread(),gc_state.phase,
@@ -210,10 +201,8 @@ gc_state_unlock()
              gc_state.phase, gc_phase_names[gc_state.phase]);
     gc_assert(get_sb_vm_thread()==gc_state.master);
     gc_state.master = NULL;
-#ifdef LISP_FEATURE_SB_THREAD
     int result = mutex_release(&gc_state.lock);
     gc_assert(result);
-#endif
     odxprint(safepoints,"%s","GC state unlocked");
 }
 
@@ -228,13 +217,7 @@ gc_state_wait(gc_phase_t phase)
     gc_assert(gc_state.master == self);
     gc_state.master = NULL;
     while(gc_state.phase != phase && !(phase == GC_QUIET && (gc_state.phase > GC_QUIET))) {
-#ifdef LISP_FEATURE_WIN32
-        SleepConditionVariableCS(&gc_state.phase_cond[phase], &gc_state.lock, INFINITE);
-#elif defined LISP_FEATURE_SB_THREAD
-        pthread_cond_wait(&gc_state.phase_cond[phase],&gc_state.lock);
-#else
-        lose("gc_state_wait() blocks, but we're #-SB-THREAD");
-#endif
+        CONDITION_VAR_WAIT(&gc_state.phase_cond[phase], &gc_state.lock);
     }
     gc_assert(gc_state.master == NULL);
     gc_state.master = self;
@@ -283,7 +266,7 @@ static inline gc_phase_t gc_phase_next(gc_phase_t old) {
     return (old+1) % GC_NPHASES;
 }
 
-static inline boolean
+static inline bool
 thread_blocks_gc(struct thread *thread)
 {
     return read_TLS(GC_INHIBIT,thread)==LISP_T;
@@ -305,8 +288,8 @@ thread_blocks_gc(struct thread *thread)
    There is a full VM page reserved for this word; page permissions
    are switched to read-only for race-free examine + wait + use
    scenarios. */
-static inline boolean
-set_thread_csp_access(struct thread* th, boolean writable)
+static inline bool
+set_thread_csp_access(struct thread* th, bool writable)
 {
     os_protect((char*)th - (THREAD_HEADER_SLOTS*N_WORD_BYTES) - THREAD_CSP_PAGE_SIZE,
                THREAD_CSP_PAGE_SIZE,
@@ -336,7 +319,7 @@ static inline void gc_notify_early()
             if (p==gc_state.collector)
                 continue;
             odxprint(safepoints,"notifying thread %p csp %p",p,csp_around_foreign_call(p));
-            boolean was_in_lisp = !set_thread_csp_access(p,0);
+            bool was_in_lisp = !set_thread_csp_access(p,0);
             if (was_in_lisp) {
                 /* Threads "in-lisp" block leaving GC_MESSAGE, as we
                  * need them to hit their CSP or the GSP, and we unmap
@@ -370,7 +353,7 @@ static inline void gc_notify_final()
             if (p == gc_state.collector)
                 continue;
             odxprint(safepoints,"notifying thread %p csp %p",p,csp_around_foreign_call(p));
-            boolean was_in_lisp = !set_thread_csp_access(p,0);
+            bool was_in_lisp = !set_thread_csp_access(p,0);
             if (was_in_lisp) {
                 gc_state.phase_wait[GC_SETTLED]++;
                 SET_THREAD_STOP_PENDING(p, LISP_T);
@@ -383,7 +366,7 @@ static inline void gc_done()
 {
     CURRENT_THREAD_VAR(self);
     struct thread *p;
-    boolean inhibit = (read_TLS(GC_INHIBIT,self)==LISP_T);
+    bool inhibit = (read_TLS(GC_INHIBIT,self)==LISP_T);
 
     odxprint(safepoints,"%s","global denotification");
     WITH_ALL_THREADS_LOCK {
@@ -453,11 +436,7 @@ static inline void gc_advance(gc_phase_t cur, gc_phase_t old) {
         gc_state.phase = gc_phase_next(gc_state.phase);
         odxprint(safepoints,"no blockers, direct advance to %d (%s)",gc_state.phase,gc_phase_names[gc_state.phase]);
         gc_handle_phase();
-#ifdef LISP_FEATURE_WIN32
-        WakeAllConditionVariable(&gc_state.phase_cond[gc_state.phase]);
-#elif defined LISP_FEATURE_SB_THREAD
-        pthread_cond_broadcast(&gc_state.phase_cond[gc_state.phase]);
-#endif
+        CONDITION_VAR_WAKE_ALL(&gc_state.phase_cond[gc_state.phase]);
     }
     odxprint(safepoints,"going to wait for %d threads",gc_state.phase_wait[gc_state.phase]);
     gc_state_wait(cur);
@@ -622,7 +601,7 @@ assert_on_stack(struct thread *th, void *esp)
 }
 
 /// Similar to the one in gc-common, but without the sigmask test.
-static boolean can_invoke_post_gc(struct thread* th)
+static bool can_invoke_post_gc(struct thread* th)
 {
     lispobj obj = th->lisp_thread;
     if (!obj) return 0;
@@ -671,7 +650,7 @@ int check_pending_gc(__attribute((unused)) os_context_t *ctx)
 void thread_in_lisp_raised(os_context_t *ctxptr)
 {
     struct thread *self = get_sb_vm_thread();
-    boolean check_gc_and_thruptions = 0;
+    bool check_gc_and_thruptions = 0;
     odxprint(safepoints,"%s","thread_in_lisp_raised");
 
     /* Either we just hit the GSP trap, or we took a PIT stop and
@@ -750,7 +729,7 @@ void thread_in_lisp_raised(os_context_t *ctxptr)
 void thread_in_safety_transition(os_context_t *ctxptr)
 {
     struct thread *self = get_sb_vm_thread();
-    boolean was_in_alien;
+    bool was_in_alien;
 
     odxprint(safepoints,"%s","GC safety transition");
     WITH_GC_STATE_LOCK {
@@ -810,7 +789,7 @@ void thread_in_safety_transition(os_context_t *ctxptr)
 void thread_interrupted(os_context_t *ctxptr)
 {
     struct thread *self = get_sb_vm_thread();
-    boolean gc_active, was_in_alien;
+    bool gc_active, was_in_alien;
 
     odxprint(safepoints,"%s","pending interrupt trap");
     WITH_GC_STATE_LOCK {
@@ -852,6 +831,7 @@ gc_stop_the_world()
         switch(gc_state.phase) {
         case GC_NONE:
             gc_advance(GC_QUIET,gc_state.phase);
+            /* FALLTHRU */
         case GC_FLIGHT:
         case GC_MESSAGE:
         case GC_INVOKED:
@@ -866,6 +846,7 @@ gc_stop_the_world()
             } else {
                 gc_state_wait(GC_QUIET);
             }
+            /* FALLTHRU */
         case GC_QUIET:
             /* Some number of threads were trying to get to GC_QUIET.
              * But this thread is sufficient to be able to leave
