@@ -12,21 +12,22 @@
 #ifndef _SAVE_H_
 #define _SAVE_H_
 #include <limits.h>
+#include <stdbool.h>
 #include "core.h"
 
 #define COMPRESSION_LEVEL_NONE INT_MIN
 
-void do_destructive_cleanup_before_save(lispobj init_function);
+void unwind_binding_stack(void);
 
-extern FILE *prepare_to_save(char *filename, boolean prepend_runtime, void **runtime_bytes, size_t *runtime_size);
-extern boolean save_runtime_to_filehandle(FILE *output, void *runtime_bytes,
+extern FILE *prepare_to_save(char *filename, bool prepend_runtime, void **runtime_bytes, size_t *runtime_size);
+extern bool save_runtime_to_filehandle(FILE *output, void *runtime_bytes,
                                           size_t runtime_size, int application_type);
-extern boolean save_to_filehandle(FILE *file, char *filename, lispobj initfun,
-                                  boolean make_executable, boolean keep_runtime_options,
+extern bool save_to_filehandle(FILE *file, char *filename, lispobj init_function,
+                                  bool make_executable, bool keep_runtime_options,
                                   int core_compression_level);
-extern boolean save(char *filename, lispobj initfun, boolean prepend_runtime,
-                    boolean keep_runtime_options,
-                    boolean compressed_core, int core_compression_level,
+extern bool save(char *filename, lispobj init_function, bool prepend_runtime,
+                    bool keep_runtime_options,
+                    bool compressed_core, int core_compression_level,
                     int application_type);
 
 #endif
