@@ -18,7 +18,6 @@
 #include "os.h"
 #include "arch.h"
 #include "lispregs.h"
-#include "alloc.h"
 #include "interrupt.h"
 #include "interr.h"
 #include "breakpoint.h"
@@ -53,6 +52,18 @@ arch_get_bad_addr(int sig, siginfo_t *code, os_context_t *context)
  * want to get to, and on OS, which determines how we get to it.)
  */
 
+void visit_context_registers(void (*proc)(os_context_register_t,void*),
+                             os_context_t *context, void* arg)
+{
+    proc(os_context_pc(context), arg);
+    proc(*os_context_register_addr(context,reg_EAX), arg);
+    proc(*os_context_register_addr(context,reg_ECX), arg);
+    proc(*os_context_register_addr(context,reg_EDX), arg);
+    proc(*os_context_register_addr(context,reg_EBX), arg);
+    proc(*os_context_register_addr(context,reg_ESI), arg);
+    proc(*os_context_register_addr(context,reg_EDI), arg);
+}
+
 int *
 os_context_flags_addr(os_context_t *context)
 {
@@ -116,8 +127,6 @@ void arch_skip_instruction(os_context_t *context)
             break;
         }
 
-    FSHOW((stderr,
-           "/[arch_skip_inst resuming at %x]\n", OS_CONTEXT_PC(context)));
 }
 
 unsigned char *
@@ -126,23 +135,15 @@ arch_internal_error_arguments(os_context_t *context)
     return 1 + (unsigned char *)(OS_CONTEXT_PC(context));
 }
 
-boolean
-arch_pseudo_atomic_atomic(os_context_t *context)
-{
-    return get_pseudo_atomic_atomic(get_sb_vm_thread());
+bool arch_pseudo_atomic_atomic(struct thread *thread) {
+    return get_pseudo_atomic_atomic(thread);
 }
 
-void
-arch_set_pseudo_atomic_interrupted(os_context_t *context)
-{
-    struct thread *thread = get_sb_vm_thread();
+void arch_set_pseudo_atomic_interrupted(struct thread *thread) {
     set_pseudo_atomic_interrupted(thread);
 }
 
-void
-arch_clear_pseudo_atomic_interrupted(os_context_t *context)
-{
-    struct thread *thread = get_sb_vm_thread();
+void arch_clear_pseudo_atomic_interrupted(struct thread *thread) {
     clear_pseudo_atomic_interrupted(thread);
 }
 
