@@ -1053,7 +1053,7 @@
    ((2 2) t)
    ((3 2) nil)))
 
-(with-test (:name :ash-signed-negation-overflow)
+(with-test (:name :ash-signed-negation-overflow :fails-on :arm)
   (checked-compile-and-assert
       ()
       `(lambda (a b)
@@ -1132,3 +1132,42 @@
       ((1 2) nil)
       ((1 1) t)
       ((3 0) 3)))
+
+(with-test (:name :integer-fixnum-<)
+  (checked-compile-and-assert
+   ()
+   `(lambda (f b &optional a c d e g h i j k l m n o p q r s)
+      (declare (fixnum f))
+      (eval (list 'list a b c d e f g h i j k l m n o p q r s t))
+      (and (integerp b)
+           (< f b)))
+   ((1 t) nil)
+   ((1 10) t)))
+
+(with-test (:name :cast-externally-checkable-p-satisfies)
+  (checked-compile-and-assert
+   ()
+   `(lambda (u s)
+      (the (unsigned-byte 64) (+ (the (or null (unsigned-byte 64)) u) (the fixnum s))))
+   ((1 2) 3)))
+
+(with-test (:name :rem-derive-type)
+  (flet ((test (form type)
+           (assert
+            (type-specifiers-equal
+             (caddr
+              (sb-kernel:%simple-fun-type
+               (checked-compile
+                `(lambda (a b)
+                   ,form))))
+             `(values ,type &optional)))))
+    (test `(rem (the fixnum a) (the integer b))
+          'fixnum)
+    (test `(rem (the fixnum a) (the (integer 0 20) b))
+          '(integer -19 19))
+    (test `(rem (the (unsigned-byte 32) a) (the (integer 0 20) b))
+          '(integer 0 19))
+    (test `(rem (the (signed-byte 32) a) (the (unsigned-byte 32) b))
+          '(signed-byte 32))
+    (test `(rem (the (signed-byte 8) a) (the (unsigned-byte 7) b))
+          '(integer -126 126))))
