@@ -119,31 +119,6 @@
               (error () :good))
             :good))
 
-;;; bug 282
-;;;
-;;; Verify type checking policy in full calls: the callee is supposed
-;;; to perform check, but the results should not be used before the
-;;; check will be actually performed.
-(locally
-    (declare (optimize (safety 3)))
-  (flet ((bar (f a)
-           (declare (type (simple-array (unsigned-byte 32) (*)) a))
-           (declare (type (function (fixnum)) f))
-           (funcall f (aref a 0))))
-    #-64-bit
-    (assert
-     (eval `(let ((n (1+ most-positive-fixnum)))
-              (if (not (typep n '(unsigned-byte 32)))
-                  (warn 'style-warning
-                        "~@<This test is written for platforms with ~
-                        ~@<(proper-subtypep 'fixnum '(unsigned-byte 32))~:@>.~:@>")
-                  (block nil
-                    (funcall ,#'bar
-                             (lambda (x) (when (eql x n) (return t)))
-                             (make-array 1 :element-type '(unsigned-byte 32)
-                                         :initial-element n))
-                    nil)))))))
-
 ;;; bug 261
 (let ((x (list (the (values &optional fixnum) (eval '(values))))))
   (assert (equal x '(nil))))
@@ -202,3 +177,13 @@
   (multiple-value-bind (ok tested) (make-array-fill-nil)
     ;; should not inhibit stack allocation, should check that NIL is OK
     (assert (and ok (eql tested 1)))))
+
+(defun push-values-constants (k)
+  (let (*)
+    (if k
+        (eval k)
+        (load-time-value t))))
+
+(with-test (:name :push-values-constants)
+  (assert (eql (push-values-constants 1) 1))
+  (assert (eql (push-values-constants nil) t)))
