@@ -45,11 +45,23 @@
                   (declare (ignore c))
                   :exhausted)))))
 
+;;; Regression test: we used to misconfigure the stack for new threads on win32.
+#+sb-thread
+(with-test (:name (:exhaust :basic :new-thread))
+  (assert (eq :exhausted
+              (sb-thread:join-thread
+               (sb-thread:make-thread
+                (lambda ()
+                  (handler-case
+                      (recurse)
+                    (storage-condition (c)
+                      (declare (ignore c))
+                      :exhausted))))))))
+
 ;;; Check that non-local control transfers restore the stack
 ;;; exhaustion checking after unwinding -- and that previous test
 ;;; didn't break it.
-(with-test (:name (:exhaust :non-local-control)
-                  :skipped-on :win32)
+(with-test (:name (:exhaust :non-local-control))
   (let ((exhaust-count 0)
         (recurse-count 0))
     (tagbody
@@ -66,8 +78,7 @@
 
 ;;; Check that we can safely use user-provided restarts to
 ;;; unwind.
-(with-test (:name (:exhaust :restarts)
-                  :skipped-on :win32)
+(with-test (:name (:exhaust :restarts))
   (let ((exhaust-count 0)
         (recurse-count 0))
     (block nil
@@ -82,6 +93,44 @@
              (recurse)))))
     (assert (= exhaust-count recurse-count *count*))))
 
+;;; We can get away with writing to the stack on unwind as long as we don't
+;;; write on the Windows-managed control stack guard page /after/ triggering the
+;;; memory protection set up by win32_reset_stack_overflow_guard_page(). This is
+;;; certainly not fool proof, though.
+(with-test (:name (:exhaust :write-to-stack-on-unwind)
+            :skipped-on (not :win32))
+  (let ((count 0))
+    (labels ((recurse-and-write-to-stack-on-unwind ()
+               (let ((x (random 1.0)))
+                 (unwind-protect
+                      (recurse-and-write-to-stack-on-unwind)
+                   (incf x (random 1.0))))))
+      (block nil
+        (handler-bind ((storage-condition (lambda (c)
+                                            (format t "Got ~s, unwinding...~%" c)
+                                            (incf count)
+                                            (return))))
+          (recurse-and-write-to-stack-on-unwind))))
+    (assert (eql count 1))))
+
+;;; On Windows, we can freely write to the stack while handling
+;;; STORAGE-CONDITION because its equivalent to CONTROL_STACK_RETURN_GUARD_PAGE
+;;; is only set up while unwinding from the STACK_OVERFLOW_EXCEPTION handler.
+;;; On other platforms, while handling the initial STORAGE-CONDITION, this test
+;;; will re-trigger the CONTROL_STACK_RETURN_GUARD_PAGE and
+;;; CONTROL_STACK_GUARD_PAGE, and issue a CORRUPTION WARNING.
+(with-test (:name (:exhaust :write-to-stack-in-handler)
+            :skipped-on (not :win32))
+  (labels ((recurse-and-write-to-stack-on-error ()
+             (let ((x 0))
+               (handler-bind ((storage-condition (lambda (c)
+                                                   (declare (ignore c))
+                                                   (setq x (random 1.0)))))
+                 (recurse-and-write-to-stack-on-error)
+                 x))))
+    (handler-case (recurse-and-write-to-stack-on-error)
+      (storage-condition ()))))
+
 (with-test (:name (:exhaust :binding-stack))
   (let ((ok nil)
         (symbols (loop repeat 1024 collect (gensym)))
@@ -97,7 +146,7 @@
       (assert ok))))
 
 (with-test (:name (:exhaust :alien-stack)
-                  :skipped-on (or (not :c-stack-is-control-stack)))
+                  :skipped-on (not :c-stack-is-control-stack))
   (let ((ok nil))
     (labels ((exhaust-alien-stack (i)
                (with-alien ((integer-array (array int 500)))
