@@ -10,11 +10,13 @@
           (sb-kernel:find-layout 'class)))
 
 (with-test (:name :compact-instance-header-layout
-            :skipped-on (:not (:and :compact-instance-header :soft-card-marks)))
+            :skipped-on (or :win32
+                            (:not (:and :compact-instance-header :soft-card-marks)))
+            :fails-on (and :darwin :x86-64)) ;; can't compile the .so
   (unless (probe-file "gc-testlib.so")
     (sb-ext:run-program "sh"
                         `("run-compiler.sh" "-sbcl-pic" "-sbcl-shared"
-                          #+darwin ,@'("-flat_namespace" "-undefined" "suppress")
+                          #+darwin ,@'("-flat_namespace" "-undefined" "suppress"  "-mmacosx-version-min=10.7")
                           "-I../src/runtime"
                           "gc-testlib.c" "-o" "gc-testlib.so")
                         :search t
