@@ -14,27 +14,33 @@
 (in-package :cl-user)
 
 (defvar *weak-vect* (make-weak-vector 8))
-(with-test (:name :weak-vector)
+(defmacro wvref (v i) `(sb-int:weak-vector-ref ,v ,i))
+(with-test (:name :weak-vector
+            :fails-on :win32)
   (let ((a *weak-vect*)
         (random-symbol (make-symbol "FRED")))
     (flet ((x ()
-             (setf (aref a 0) (cons 'foo 'bar)
-                   (aref a 1) (format nil "Time is: ~D~%" (get-internal-real-time))
-                   (aref a 2) 'interned-symbol
-                   (aref a 3) random-symbol
-                   (aref a 4) 18
-                   (aref a 5) (+ most-positive-fixnum 1 (random 100) (random 100))
-                   (aref a 6) (make-hash-table))))
+             (setf (wvref a 0) (cons 'foo 'bar)
+                   (wvref a 1) (format nil "Time is: ~D~%" (get-internal-real-time))
+                   (wvref a 2) 'interned-symbol
+                   (wvref a 3) random-symbol
+                   (wvref a 4) 18
+                   (wvref a 5) (+ most-positive-fixnum 1 (random 100) (random 100))
+                   (wvref a 6) (make-hash-table))))
       (declare (notinline x)) ;; Leave all the values below the stack pointer for
       (x))                    ;; scrub-control-stack to work
     (assert (weak-vector-p a))
     (sb-sys:scrub-control-stack)
     (gc)
-    (assert (eq (aref a 2) 'interned-symbol))
-    (assert (eq (aref a 3) random-symbol))
-    (assert (= (aref a 4) 18))
+    (assert (eq (wvref a 2) 'interned-symbol))
+    (assert (eq (wvref a 3) random-symbol))
+    (assert (= (wvref a 4) 18))
     ;; broken cells are the cons, string, bignum, hash-table, plus one NIL
     ;; cell that was never assigned into
+    (assert (null (wvref a 0)))
+    (assert (null (wvref a 1)))
+    (assert (null (wvref a 5)))
+    (assert (null (wvref a 6)))
     *weak-vect*))
 
 ;; Assert something about *CURRENT-THREAD* seeing objects that it just consed.
@@ -105,7 +111,7 @@
     ;; but seems like it'll be OK for a while.
     ;; I see only 4 weak pointers in the baseline image.
     ;; Really we could just assert /= 1000.
-    (assert (< (length l) 15))))
+    (assert (< (length l) 80))))
 
 ;; check that WITHOUT-INTERRUPTS doesn't block SIG_STOP_FOR_GC
 (with-test (:name :gc-without-interrupts
@@ -151,7 +157,7 @@
   ;; GENERATION-OF broke when fdefns stopped storing a generation in word 0.
   ;; Normally we expect to see SB-VM:+PSEUDO-STATIC-GENERATION+
   ;; but allow for varied definition of CORE_PAGE_GENERATION.
-  (assert (= (sb-kernel:generation-of (sb-int:find-fdefn 'car))
+  (assert (= (sb-kernel:generation-of (sb-int:find-fdefn '(setf car)))
              (sb-kernel:generation-of #'car))))
 
 (with-test (:name :static-fdefn-space)
@@ -224,6 +230,7 @@
     (assert (= (sb-kernel:get-lisp-obj-address *pin-test-object*)
                *pin-test-object-address*))))
 
+(import 'sb-kernel:%make-lisp-obj)
 #+gencgc
 (defun ensure-code/data-separation ()
   (let* ((n-bits (+ sb-vm:next-free-page 10))
@@ -236,7 +243,7 @@
        ;; M-A-O disables GC, therefore GET-LISP-OBJ-ADDRESS is safe
        (let ((obj-addr (sb-kernel:get-lisp-obj-address obj))
              (array (cond ((member type `(,sb-vm:code-header-widetag
-                                          #+compact-instance-header
+                                          #+executable-funinstances
                                           ,sb-vm:funcallable-instance-widetag))
                            (incf total-code-size size)
                            code-bits)
@@ -251,6 +258,25 @@
                                                         (1- size))))
                do (setf (sbit array index) 1))))
      :dynamic)
+    (let ((p (position 1 (bit-and code-bits data-bits))))
+      (when p
+        (format t "~&code+data: page index ~d, generation ~D~%"
+                p (slot (deref sb-vm::page-table p) 'sb-vm::gen))
+        (assert (zerop (slot (deref sb-vm::page-table p) 'sb-vm::start)))
+        (let ((base (+ (* p sb-vm:gencgc-page-bytes)
+                       sb-vm:dynamic-space-start)))
+          ;; This mapping operation may fail if the page's first object is not at the base
+          ;; or it ends with a page-spanning object. But this diagnostic logic should
+          ;; never be invoked. If it is, you should find the cause of that rather than
+          ;; worry about this slightly dubious use of map-objects-in-range.
+          (sb-vm::map-objects-in-range
+           (lambda (obj widetag size)
+             (declare (ignore widetag size))
+             (format t "~x ~s~%" (sb-kernel:get-lisp-obj-address obj) (type-of obj)))
+           (%make-lisp-obj base)
+           (%make-lisp-obj (+ base
+                              (ash (slot (deref sb-vm::page-table p) 'sb-vm::words-used*)
+                                   sb-vm:word-shift)))))))
     (assert (not (find 1 (bit-and code-bits data-bits))))
     (let* ((code-bytes-consumed
              (* (count 1 code-bits) sb-vm:gencgc-page-bytes))
@@ -260,10 +286,7 @@
       ;; Some have as little as .5% space wasted.
       (assert (<= waste (* 3/100 code-bytes-consumed))))))
 
-
-
-(with-test (:name :code/data-separation
-            :skipped-on (not :gencgc))
+(with-test (:name :code/data-separation)
   (compile 'ensure-code/data-separation)
   (ensure-code/data-separation))
 
@@ -367,27 +390,19 @@
     (setq working nil)
     (sb-thread:join-thread gc-thread)))
 
-;;; This loop doesn't work for #+use-cons-region (it will run forever)
-;;; because you can't hit the end of a page. Also, it's not clear that it should
-;;; ever have worked, because the check at the bottom of lisp_alloc() for whether
-;;; we're near the end of a page will discard the last few objects in favor of
-;;; returning a brand new region.  I guess as long as the inline allocator is
-;;; always used, it's fine. But the inline code wasn't always used on x86.
-;;; So how did that work?
 (defun use-up-thread-region ()
   ;; cons until the thread-local allocation buffer uses up a page
   (loop
-   (let* ((c (cons 1 2))
+   (let* ((c (make-array 0))
           (end (+ (sb-kernel:get-lisp-obj-address c)
-                  (- sb-vm:list-pointer-lowtag)
+                  (- sb-vm:other-pointer-lowtag)
                   (* 2 sb-vm:n-word-bytes))))
      (when (zerop (logand end (1- sb-vm:gencgc-page-bytes)))
        (return)))))
 (defglobal *go* nil)
 
 #+sb-thread
-(with-test (:name :c-call-save-p :skipped-on (or :interpreter
-                                                 :use-cons-region))
+(with-test (:name :c-call-save-p :skipped-on :interpreter)
   ;; Surely there's a better way to assert that registers get onto the stack
   ;; (so they can be seen by GC) than by random hammering on (LIST (LIST ...)).
   ;; This should probably be in gc-testlib.c. Or better yet: get rid of #+sb-safepoint
