@@ -344,12 +344,12 @@
   (test-this-object 'equal (find-class 'class)))
 
 (with-test (:name :remhash-eq-comparable-in-equalp-table)
-  ;; EQUALP tables worked a little better, because more objects have
-  ;; are hashed non-address-sensitively by EQUALP-HASH relative to EQUAL-HASH,
+  ;; EQUALP tables worked a little better, because more objects are
+  ;; hashed non-address-sensitively by EQUALP-HASH relative to EQUAL-HASH,
   ;; and those objects have comparators that descend.
   ;; However, there are still some things hashed by address:
   (test-this-object 'equalp (make-weak-pointer "bleep"))
-  (test-this-object 'equalp (sb-int:find-fdefn 'cons))
+  (test-this-object 'equalp (sb-int:find-fdefn '(setf car)))
   (test-this-object 'equalp #'car)
   (test-this-object 'equalp (constantly 5))
   (test-this-object 'equal (sb-sys:int-sap 0)))
@@ -360,7 +360,7 @@
 ;;; This affected the performance of TYPECASE.
 (with-test (:name :sxhash-on-layout)
   (dolist (x '(pathname cons array))
-    (let ((l (sb-kernel:wrapper-friend (sb-kernel:find-layout x))))
+    (let ((l (sb-kernel:find-layout x)))
       (assert (= (sxhash l) (sb-kernel:layout-clos-hash l))))))
 
 (with-test (:name :equalp-table-fixnum-equal-to-float)
@@ -368,16 +368,15 @@
     (assert (eql (setf (gethash 3d0 table) 1)
                  (gethash 3   table)))))
 
+;;; Check that hashing a stringlike thing which is possibly NIL uses
+;;; a specialized hasher (after prechecking for NIL via the transform).
 (with-test (:name :transform-sxhash-string-and-bv)
-  (let ((f (compile
-            nil
-            '(lambda (x y)
-               (logxor (ash (sxhash (truly-the (or string null) x)) -3)
-                       (sxhash (truly-the (or bit-vector null) y))))))
-        (fdefn1 (sb-int:find-fdefn 'sb-kernel:%sxhash-string))
-        (fdefn2 (sb-int:find-fdefn 'sb-kernel:%sxhash-bit-vector)))
-    (every (lambda (x) (member x `(,fdefn1 ,fdefn2)))
-           (ctu:find-code-constants f))))
+  (dolist (case `((bit-vector sb-kernel:%sxhash-bit-vector)
+                  (string sb-kernel:%sxhash-string)
+                  (simple-bit-vector sb-kernel:%sxhash-simple-bit-vector)
+                  (simple-string sb-kernel:%sxhash-simple-string)))
+    (let ((f `(lambda (x) (sxhash (truly-the (or null ,(car case)) x)))))
+      (assert (equal (ctu:ir1-named-calls f) (cdr case))))))
 
 (with-test (:name :sxhash-on-displaced-string
             :fails-on :sbcl)
@@ -404,7 +403,8 @@
     ;; Also the same issue exists with bit-vectors.
     (assert-error (sxhash displaced-string))))
 
-(with-test (:name :array-psxhash-non-consing :skipped-on :interpreter)
+(with-test (:name :array-psxhash-non-consing :skipped-on :interpreter
+            :fails-on :ppc64)
    (let ((a (make-array 1000 :element-type 'double-float
                         :initial-element (+ 0d0 #+(or arm64 x86-64)
                                                 1d300))))
@@ -468,15 +468,15 @@
                  (when #-64-bit t #+64-bit (not (eq hasher 'sxhash))
                    (assert (>= (length (remove-duplicates field)) 8))))))))
     (try 'sxhash)
-    (try 'sb-int:good-hash-word->fixnum)))
+    (try 'sb-int:murmur-hash-word/fixnum)))
 
 ;;; Ensure that all layout-clos-hash values have a 1 somewhere
 ;;; such that LOGANDing any number of nonzero hashes is nonzero.
 (with-test (:name :layout-hashes-constant-1-bit)
   (let ((combined most-positive-fixnum))
-    (maphash (lambda (classoid wrapper)
+    (maphash (lambda (classoid layout)
                (declare (ignore classoid))
-               (let ((hash (sb-kernel:wrapper-clos-hash wrapper)))
+               (let ((hash (sb-kernel:layout-clos-hash layout)))
                  (setq combined (logand combined hash))))
              (sb-kernel:classoid-subclasses (sb-kernel:find-classoid 't)))
     (assert (/= 0 combined))))
@@ -498,7 +498,7 @@
     (loop repeat (the fixnum n-iter)
           do
        (let* ((n (random limit random-state))
-              (lisp-hash (sb-impl::murmur-fmix-word n))
+              (lisp-hash (sb-impl:murmur-fmix-word-for-unit-test n))
               (c-hash (c-murmur-fmix n)))
          (assert (= lisp-hash c-hash))))))
 (compile 'murmur-compare)
