@@ -33,7 +33,7 @@
                               ,(format nil "This is ~D" i))))))
   (def-100slots))
 
-(assert (typep (sb-kernel:wrapper-bitmap
+(assert (typep (sb-kernel:layout-bitmap
                 (sb-kernel::find-layout 'biggy)) 'bignum))
 
 (defvar *x* nil)
@@ -60,14 +60,14 @@
 ;; Run it twice to make sure things really worked.
 
 (let ((*y* (make-biggy))
-      (*x* (sb-kernel:wrapper-bitmap
+      (*x* (sb-kernel:layout-bitmap
             (sb-kernel::find-layout 'biggy))))
   (sb-ext:gc :gen 1))
 (princ 'did-pass-1) (terpri)
 (force-output)
 
 (let ((*y* (make-biggy))
-      (*x* (sb-kernel:wrapper-bitmap
+      (*x* (sb-kernel:layout-bitmap
             (sb-kernel::find-layout 'biggy))))
   (sb-ext:gc :gen 1))
 (princ 'did-pass-2) (terpri)
@@ -99,7 +99,7 @@
   (c 'cee))                                      ;    13       9
 
 (defvar *afoo* (make-foo1))
-(assert (= (sb-kernel:wrapper-length (sb-kernel:wrapper-of *afoo*))
+(assert (= (sb-kernel:layout-length (sb-kernel:layout-of *afoo*))
            (sb-kernel:%instance-length *afoo*)))
 (with-test (:name :tagged-slot-iterator-macro)
   ;; on 32-bit, the logical length is 14, which means 15 words (with header),
@@ -172,7 +172,3 @@
 (with-test (:name :copy-structure-bignum-bitmap)
   (assert (zerop (foo-lotsaslots-s0
                   (copy-structure (make-foo-lotsaslots))))))
-
-(load "compiler-test-util.lisp")
-(with-test (:name :copy-structure-efficient-case)
-  (assert (not (ctu:find-named-callees #'copy-structure :name 'ash))))
