@@ -17,7 +17,7 @@
 ;;;; the expected reference to the one and only thread.
 ;;;; And also sb-safepoint gets a crash in C.
 #-(and gencgc sb-thread (not sb-safepoint)
-       (or (and arm64 (not darwin)) ppc64 x86-64))
+       (or arm64 ppc64 x86-64))
 (invoke-restart 'run-tests::skip-file)
 
 (setq sb-ext:*evaluator-mode* :compile)
@@ -35,17 +35,17 @@
            (ecase root
              (:tls
               (scrubstack)
-              (sb-ext:search-roots wp :criterion :oldest :gc t :print nil))
+              (sb-ext:search-roots wp :criterion :oldest :print nil))
              (:bindings ; bind *FRED* again so the old value is on the binding stack
               (let ((*fred* 1))
                 (scrubstack)
-                (sb-ext:search-roots wp :criterion :oldest :gc t :print nil)))
+                (sb-ext:search-roots wp :criterion :oldest :print nil)))
              (:stack
               ; put the OBJ back on the control stack
               ; and also ensure that *FRED* is not a root.
               (setq obj *fred* *fred* nil)
               (scrubstack)
-              (sb-ext:search-roots wp :criterion :oldest :gc t :print nil)))))
+              (sb-ext:search-roots wp :criterion :oldest :print nil)))))
       (assert paths)
       (let* ((path (cdar paths))
              (root (car path)))
@@ -69,7 +69,7 @@
 (defun f0 ()
   (let* ((c (cons 1 2))
          (wp (make-weak-pointer c)))
-    (let ((paths (sb-ext:search-roots wp :criterion :static :gc t :print nil)))
+    (let ((paths (sb-ext:search-roots wp :criterion :static :print nil)))
       (assert paths)
       (let* ((path (car paths))
              (nodes (cdr path)))
@@ -88,7 +88,7 @@
 
 ;;; Sample output:
 ;;; Path to "hi":
-;;;  6       1000209AB3 [   1] a package-hashtable
+;;;  6       1000209AB3 [   1] a symbol-hashset
 ;;;  1       10048F145F [  29] a (simple-vector 37)
 ;;;  1         503B403F [   2] COMMON-LISP-USER::*TOP*
 ;;;  0       1004B885B7 [   6] a cons = (P Q R ...) ; = (NTHCDR 6 object)
@@ -120,8 +120,7 @@
 
 (defvar *foo*)
 #+gencgc
-(with-test (:name (sb-ext:search-roots :simple-fun)
-            :broken-on (and :darwin :arm64))
+(with-test (:name (sb-ext:search-roots :simple-fun))
   ;; Tracing a path to a simple fun wasn't working at some point
   ;; because of failure to employ fun_code_header in the right place.
   (setq *foo* (compile nil '(lambda () 42)))
@@ -130,4 +129,4 @@
 
 #+gencgc
 (with-test (:name (sb-ext:search-roots :ignore-immediate))
-  (sb-ext:search-roots (make-weak-pointer 48) :gc t :print nil))
+  (sb-ext:search-roots (make-weak-pointer 48) :print nil))
