@@ -11,6 +11,9 @@
 
 #-x86-64 (invoke-restart 'run-tests::skip-file)
 
+(load "compiler-test-util.lisp")
+(import 'ctu:disassembly-lines)
+
 ;;; This trivial function failed to compile due to rev 88d078fe
 (defun foo (&key k)
   (make-list (reduce #'max (mapcar #'length k))))
@@ -39,22 +42,6 @@
               sb-vm:other-pointer-lowtag)))
     (assert (> a sb-vm:static-space-start))))
 
-(load "compiler-test-util.lisp")
-(defun disassembly-lines (fun)
-  ;; FIXME: I don't remember what this override of the hook is for.
-  (sb-int:encapsulate 'sb-disassem::add-debugging-hooks 'test
-                      (lambda (f &rest args) (declare (ignore f args))))
-  (prog1
-      (mapcar (lambda (x) (string-left-trim " ;" x))
-              (cddr
-               (split-string
-                (with-output-to-string (s)
-                  (let ((sb-disassem:*disassem-location-column-width* 0)
-                        (*print-pretty* nil))
-                    (disassemble fun :stream s)))
-                #\newline)))
-    (sb-int:unencapsulate 'sb-disassem::add-debugging-hooks 'test)))
-
 (sb-vm::define-vop (tryme)
     (:generator 1 (sb-assem:inst mov :byte (sb-vm::ea :gs sb-vm::rax-tn) 0)))
 (with-test (:name :try-gs-segment)
@@ -71,14 +58,15 @@
         (setq lines (nbutlast lines)))
       ;; Remove safepoint traps
       (setq lines (remove-if (lambda (x) (search "; safepoint" x)) lines))
-      ;; If the last 4 lines are of the expected form
-      ;;   MOV RSP, RBP / CLC / POP RBP / RET
+      ;; If the last 3 lines are of the expected form
+      ;;   LEAVE / CLC / RET
       ;; then strip them out
       (if (and remove-epilogue
-               (every #'search
-                      '("MOV RSP, RBP" "CLC" "POP RBP" "RET")
-                      (subseq lines (- (length lines) 4))))
-          (butlast lines 4)
+               (let ((last3 (subseq lines (- (length lines) 3))))
+                 (and (search "LEAVE" (first last3))
+                      (search "CLC" (second last3))
+                      (search "RET" (third last3)))))
+          (butlast lines 3)
           lines)))
 (defun disasm-load (safety symbol)
   ;; This lambda has a name because if it doesn't, then the name
@@ -179,7 +167,7 @@
          (index
           (position "OBJECT-NOT-TYPE-ERROR" lines :test 'search)))
     (let ((line (nth (+ index 2) lines)))
-      (assert (search "; #<SB-KERNEL:WRAPPER " line))
+      (assert (search "; #<SB-KERNEL:LAYOUT " line))
       (assert (search " SB-ASSEM:LABEL" line)))))
 
 #+immobile-code ; uses SB-C::*COMPILE-TO-MEMORY-SPACE*
@@ -199,7 +187,7 @@
     (assert (find-line "CALL" "FUNCTION GENSYM"))
     (assert (find-line "JMP" "FUNCTION PRINT")))))
 
-(with-test (:name :c-call :skipped-on (and :win32 :x86-64))
+(with-test (:name :c-call :skipped-on :win32)
   (let* ((lines (split-string
                  (with-output-to-string (s)
                    (let ((sb-disassem:*disassem-location-column-width* 0))
@@ -535,16 +523,20 @@
     ;; Aside from ECASE failure, there are no other JMPs
     (assert (= (count-assembly-labels lines) 1))))
 
+;;; Assert that the ECASE-FAILURE vop emits a trap and that we don't call ERROR
+;;; (cutting down on the code size for each ECASE)
 (with-test (:name :ecase-failure-trap)
-  (assert (null (ctu:find-named-callees
-                 (checked-compile `(lambda (x)
-                                     (ecase x (:a 1) (:b 2) (:c 3)))))))
-  (assert (null (ctu:find-named-callees
-                 (checked-compile `(lambda (x)
+  ;; test ECASE
+  (assert (ctu:asm-search "ECASE-FAILURE-ERROR"
+                                  `(lambda (x)
+                                     (ecase x (:a 1) (:b 2) (:c 3)))))
+  ;; test ETYPECASE
+  (assert (ctu:asm-search "ETYPECASE-FAILURE-ERROR"
+                                  `(lambda (x)
                                      (etypecase x
                                        ((integer 1 20) 'hi)
                                        ((cons (eql :thing)) 'wat)
-                                       (bit-vector 'hi-again))))))))
+                                       (bit-vector 'hi-again))))))
 
 (with-test (:name :symbol-case-optimization-levels)
   (let ((cases
@@ -683,15 +675,15 @@
   ;; component.
   (let ((names
           (mapcar (lambda (x)
-                    (sb-kernel:classoid-name (sb-kernel:wrapper-classoid x)))
-                  (ctu:find-code-constants #'sb-kernel:%%typep :type 'sb-kernel:wrapper))))
+                    (sb-kernel:classoid-name (sb-kernel:layout-classoid x)))
+                  (ctu:find-code-constants #'sb-kernel:%%typep :type 'sb-kernel:layout))))
     (assert (null (set-difference names
                                   '(sb-kernel:ctype
                                     sb-kernel:unknown-type
                                     sb-kernel:fun-designator-type
                                     sb-c::abstract-lexenv
                                     sb-kernel::classoid-cell
-                                    sb-kernel:wrapper
+                                    sb-kernel:layout
                                     sb-kernel:classoid
                                     sb-kernel:built-in-classoid
                                     #-immobile-space null))))))
@@ -710,17 +702,18 @@
      (loop for line in (split-string (with-output-to-string (string)
                                        (disassemble f :stream string))
                                      #\newline)
-             thereis (and (search "WRAPPER for" line)
+             thereis (and (search "LAYOUT for" line)
                           (search "CMP DWORD PTR" line)))))
 
 (with-test (:name :thread-local-unbound)
+  (declare (optimize safety))
   (let ((c (nth-value 1 (ignore-errors sb-c::*compilation*))))
     (assert (eq (cell-error-name c) 'sb-c::*compilation*))))
 
 #+immobile-code
 (with-test (:name :debug-fun-from-pc-more-robust)
   ;; This test verifies that debug-fun-from-pc does not croak when the PC points
-  ;; within a trampoline allocated to wrap a closure in a simple-funifying wrapper
+  ;; within a trampoline allocated to wrap a closure in a simple-funifying layout
   ;; for installation into a global symbol.
   (let ((closure (funcall (compile nil '(lambda (x)  (lambda () x))) 0))
         (symbol (gensym)))
@@ -957,6 +950,7 @@
                     (when (/= linecount expect-n)
                       (warn "~S was ~d is ~d" type expect-n linecount))))))))))
 
+#+nil ; gotta figure out how to make this insensitive to standard asm boilerplate
 (with-test (:name :many-interesting-array-types
                   :skipped-on (:or (:not :sb-unicode)
                                    (:not :immobile-space)))
@@ -976,6 +970,7 @@
   ;; Previously there would have been one from BOUNDP and one from SYMBOL-VALUE.
   (let ((lines (disassembly-lines
                 '(lambda ()
+                  (declare (optimize safety))
                   (if (boundp 'sb-c::*compilation*) sb-c::*compilation*) '(hi)))))
     (dolist (line lines)
       (assert (not (search "ERROR" line))))
@@ -1144,7 +1139,7 @@
 
 (sb-vm::define-vop (trythis)
   (:generator 1
-   (sb-vm::inst and sb-vm::rax-tn (sb-c:make-fixup nil :gc-barrier))))
+   (sb-vm::inst and sb-vm::rax-tn (sb-c:make-fixup nil :card-table-index-mask))))
 (defun zook ()
   (sb-sys:%primitive trythis)
   nil)
@@ -1191,48 +1186,45 @@
     (split-string (get-output-stream-string string-stream)
                   #\newline)))
 
-(defun assert-has-gc-barrier (match-string lines &optional (skip 0))
-
-  (let (found)
-;; Look for something like this in the trace output:
-;; "VOP CLOSURE-INIT t9[RBX(d)] :NORMAL t13[RSI(d)] :NORMAL {0} "
-;; "        MOV     0, #<TN t14[RAX(u)] :NORMAL>, #<TN t9[RBX(d)] :NORMAL>"
-;; "        SHR     0, #<TN t14[RAX(u)] :NORMAL>, 10"
-;; "        AND     6, #<TN t14[RAX(u)] :NORMAL>, #S(FIXUP :NAME NIL :FLAVOR GC-BARRIER :OFFSET 0)"
-;; "        MOV     4, PTR [R12+RAX+0], 0"
-;; "        MOV     7, PTR [RBX+5], #<TN t13[RSI(d)] :NORMAL>"
-    (loop while lines
-          do
-          (when (and (search match-string (car lines))
-                     (minusp (decf skip)))
-            (setq found t)
-            (let ((barrier-fixup-line (nth 3 lines)))
-              (assert (search ":FLAVOR GC-BARRIER" barrier-fixup-line))
-              (return)))
-          (pop lines))
-    ;; Fail if we didn't find the sentinel CLOSURE-INIT
-    (assert found)))
+;;; Return the list of vops (as strings) that emitted a GC barrier.
+(defun find-gc-barriers (lambda-expression)
+  (let ((lines
+         (compiler-trace-output-lines lambda-expression))
+        (current-vop)
+        (result))
+    ;; FIXME: can this miss vops on a label line, or do we always print labels
+    ;; on their own line?
+    (dolist (line lines (nreverse result))
+     (let ((line (pop lines)))
+       (cond ((and (>= (length line) 4) (string= line "VOP " :end1 4))
+              (let ((string (subseq line 4 (position #\space line :start 5))))
+                (setq current-vop string)))
+             ((search ":FLAVOR CARD-TABLE-INDEX-MASK" line)
+              (push current-vop result)))))))
 
 (with-test (:name :closure-init-gc-barrier)
-  (let ((lines
-         (compiler-trace-output-lines
+  (let ((vops-with-barrier
+         (find-gc-barriers
            '(lambda (address)
-             (declare (sb-vm:word address))
-             (let ((sap (sb-sys:int-sap address)))
-               (lambda () sap))))))
-    (assert-has-gc-barrier "CLOSURE-INIT" lines)))
+              (let ((sap (sb-sys:int-sap (truly-the sb-vm:word address))))
+                ;; The compiler conses the SAP *after* making the closure, so the store
+                ;; of the sap into the closure needs a barrier, as it's an old->young store.
+                (lambda () sap))))))
+    (assert (equal vops-with-barrier '("CLOSURE-INIT")))))
 
 (defstruct (point (:constructor make-point (x))) x (y 0) (z 0))
 (with-test (:name :structure-init-gc-barrier)
-  (let ((lines
-          (compiler-trace-output-lines
+  (let ((vops-with-barrier
+          (find-gc-barriers
            '(lambda (val)
              (declare (double-float val) (inline make-point))
+             ;; Similarly to the closure-init test, DOUBLE-FLOAT reg is cast to
+             ;; DESCRIPTOR-REG only after making the object. This seems like a
+             ;; low-hanging-fruit optimization to flip the order.
              (let ((neg (- val))) (make-point neg))))))
-    (assert-has-gc-barrier "SET-SLOT" lines
-                           #-compact-instance-header 1)))
+    (assert (equal vops-with-barrier '("SET-SLOT")))))
 
-(with-test (:name :system-tlabs)
+(with-test (:name :system-tlabs :skipped-on (not :sb-thread))
   (when (find-symbol "SYS-ALLOC-TRAMP" "SB-VM")
     (assert (loop for line in (disassembly-lines 'sb-impl:test-make-packed-info)
                   thereis (search "SYS-ALLOC-TRAMP" line)))
@@ -1253,7 +1245,7 @@
 
 #+immobile-space
 (with-test (:name :disassemble-alien-linkage-table-ref
-                  :fails-on (not :sb-thread))
+                  :fails-on (or (not :sb-thread) :win32))
   (dolist (memspace '(:dynamic :immobile))
     (let ((sb-c::*compile-to-memory-space* memspace))
       (assert (find-in-disassembly
@@ -1268,8 +1260,7 @@
 
 ;;; This tests that the x86-64 disasembler does not crash
 ;;; on LEA with a rip-relative operand and no label.
-(with-test (:name (disassemble :no-labels)
-                  :skipped-on (not :x86-64))
+(with-test (:name (disassemble :no-labels))
   (let* ((lines
           (split-string
            (with-output-to-string (stream)
@@ -1283,3 +1274,113 @@
     (disassemble 'sb-impl::inspector
                  :use-labels nil
                  :stream (make-broadcast-stream))))
+
+#+immobile-space
+(with-test (:name (disassemble :static-call))
+  (dolist (sym '(sb-kernel:two-arg-* sb-kernel:two-arg-/
+                 sb-kernel:two-arg-+ sb-kernel:two-arg--))
+    (let ((f (let ((sb-c:*compile-to-memory-space* :dynamic))
+               (compile nil `(lambda (x y) (,sym x y))))))
+      (assert (loop for line in (disassembly-lines f)
+                    thereis (search (princ-to-string sym) line))))))
+
+(with-test (:name :closure-gc-barrier)
+  (let ((vops-with-barrier
+         (find-gc-barriers
+           '(lambda ()
+             (let ((c (cons 1 2)))
+               (lambda (x)
+                 (setf (car c) x)))))))
+    (assert (equal vops-with-barrier '("SET-SLOT")))))
+
+(defstruct (wordpair (:predicate nil) (:copier nil)) a b)
+(defun func (x) (wordpair-b x))
+(with-test (:name :non-stack-instance-set)
+  ;; This is a trivial test of the control case for :stack-instance-set
+  (let ((vops-with-barrier
+         (find-gc-barriers
+          '(lambda ()
+            (declare (inline make-wordpair))
+            (let ((pair (make-wordpair :a 'foo)))
+              (setf (wordpair-b pair) "hi")
+              (values (func pair)))))))
+    (assert (equal vops-with-barrier '("INSTANCE-INDEX-SET")))))
+
+(with-test (:name :stack-instance-set)
+  (let ((vops-with-barrier
+         (find-gc-barriers
+          '(lambda ()
+            (declare (inline make-wordpair))
+            (let ((pair (make-wordpair :a 'foo)))
+              (declare (dynamic-extent pair))
+              (setf (wordpair-b pair) "hi")
+              (values (func pair)))))))
+    (assert (not vops-with-barrier))))
+
+(declaim (freeze-type wordpair))
+(with-test (:name :stack-instance-copy)
+  (let ((vops-with-barrier
+         (find-gc-barriers
+          '(lambda (x)
+            (let ((copy (copy-structure (truly-the wordpair x))))
+              (declare (dynamic-extent copy))
+              (values (func copy)))))))
+    (assert (not vops-with-barrier))))
+
+;;; word-sized add, subtract, multiply vops which yield either a fixnum
+;;; or bignum where the bignum can have 1, 2, or 3 bigdigits.
+(defparameter unsigned-word-test-inputs
+  (loop for i from 0 by (ash 1 56) repeat 256 collect i))
+
+(defparameter signed-word-test-inputs
+  (funcall
+   (compile nil ;; no interpreter stubs
+            `(lambda ()
+               (loop for word in unsigned-word-test-inputs
+                     collect (let ((b (sb-bignum:%allocate-bignum 1)))
+                               (setf (sb-bignum:%bignum-ref b 0) word)
+                               (sb-bignum::%normalize-bignum b 1)))))))
+
+(defun check-result (fun x y actual)
+  (let ((expect (funcall fun x y)))
+    (unless (eql actual expect)
+      (when (typep expect 'bignum)
+        (sb-vm:hexdump expect)
+        (terpri)
+        (sb-vm:hexdump actual))
+      (error "Failure @ ~X ~A ~X, expect ~D (~A) got ~D~%"
+             x fun y expect
+             (typecase expect
+               (fixnum 'fixnum)
+               (bignum (format nil "~d-word bignum"
+                               (sb-bignum:%bignum-length expect))))
+             actual))))
+
+(macrolet ((test-op (op)
+             `(progn
+                (format t "~&Testing ~A~%" ',op)
+                (dolist (x signed-word-test-inputs)
+                  (dolist (y signed-word-test-inputs)
+                    (check-result
+                     ',op x y
+                     (,op (the sb-vm:signed-word x) (the sb-vm:signed-word y))))))))
+  (defun test-signed ()
+    (test-op +)
+    (test-op -)
+    (test-op *)))
+
+(macrolet ((test-op (op)
+             `(progn
+                (format t "~&Testing ~A~%" ',op)
+                (dolist (x unsigned-word-test-inputs)
+                  (dolist (y unsigned-word-test-inputs)
+                    (check-result
+                     ',op x y
+                     (,op (the sb-vm:word x) (the sb-vm:word y))))))))
+  (defun test-unsigned ()
+    (test-op +)
+    (test-op -)
+    (test-op *)))
+
+(with-test (:name :signed-vops) (test-signed))
+(with-test (:name :unsigned-vops) (test-unsigned))
